#!/usr/bin/python

# FDCC-PC
#
# Logiciel de pilotage pour la centrale free-dcc 2017
#
# Ce logiciel permet
# - de selectioner les locomotives et les conduire avec des souris virtuelles
# - programmer les decodeurs
# - permettre a des programme de se connecter par reseau pour conduire les locomotive (ex cabines)
# - afficher un tco
# - tester les entrees/sorties
# - realiser des automatismes avec des equations ou des scripts
#
# Historique
# - 2018 jan 30: correction plantage dans la partie signalisation (merci a Franck pour avoir souleve le bug) -> ok
#                modification menu cantons (suite a une discussion avec Franck)
# - 2017 jul 26: ajout de la cac en mode script -> ok
# - 2017 jul 24: ajout de la cac (conduite automatique en mode cantonnement) -> ok
# - 2017 jul 21: ajout du module de signalisation -> ok
# - 2017 jul 12: amelioration de l'affichage des tco -> ok
# - 2017 jun 21: ajout des souris Android -> ok
# - 2017 mai 22: ajout du suivi des trains (cantonnement)
# - 2017 mai 18: optimisation de l'affichage de la fenetre des locos -> ok
# - 2017 mai 18: mise sous exception du chnagement de couleur des rectangles locos (bug chez Francois) 
# - 2017 mai 17: mise sous exception du changement de titre de la fenetre          (bug chez Francois et JFC) 
# - 2017 mai 11: support de plusieurs centrales -> ok
# - 2017 mai 05: ajout des effets speciaux (pour les leds et neo)
# - 2017 mai 02: ajout du support des tco deportes de fdcc_tco_net.py -> ok
# - 2017 avr 21: ajout des itineraires DA (destruction automatique) -> ok
# - 2017 avr 20: passage des itineraires du mode direct au mode TP (trace permanent) avec reservation des aiguillages -> ok
# - 2017 avr 14: ajout des annonces sonores (fonctione que sous Windows) -> ok
#                changement de loco et couleur des boulettes de selection des souris en fonction des souris
# - 2017 avr 11: ajout des neopixels, pwm, servos, ana -> ok
#                redraw en permanence des regus virtuels pour suivre les modifications script, souris et cabines -> ok
# - 2017 avr 07: 1ere Release 
# - 2017 mar 24: feedback des souris (4 a la fois) -> ok
# - 2017 mar 23: aiguillages avec Arduino -> ok
#                entrees min -> ok
#                ajout centrale B -> ok
# - 2017 mar 22: connexion des cabines minicab par le reseau -> ok
#                itineraires directs -> ok
#                plusieurs programmes par script -> ok
# - 2017 mar 21: AU            -> ok
# - 2017 mar 20: tco           -> ok
# - 2017 mar 17: tdv           -> ok
#                debug         -> ok
#                script        -> ok
# - 2017 mar 15: rs232         -> ok
#                prog_cv       -> ok
# - 2017 mar 14: creation (avec l'aide du prototype fdcc 2016)
#                ecran loco et regu virtuels
#
# TBD
# - utiliser les potars ds la fenetre neo
# - gagner du temps a l'init
# - update tco: optimiser affichage si varaibles ne chnangent pas
#
# BUGs
# - le programme ne se ferme pas toujours correctement (non retour a la ligne de commande)
#
# LIMITATIONS
# - fonctionne uniquement sur Python3 pour le moment
# - les commandes bal et depl des anciennes versions ne sont pas supportees
#
# FUTUR
# - BMVU
# - moteurs pas a pas
# - plaques tournantes
#
# FUN
# - tester avec protocol 6050
# - essayer sur Raspberry PI  <---- ok (rame un peu sur la 1B, super sur la 3)
#


#---------------------------------------------------------------------------------------
# IMPORT DES MODULES
#---------------------------------------------------------------------------------------

try:
    import tkinter
except:
    print("ERREUR: fdcc_pc ne fonctionne pas avec Python 2, utilisez Python 3 !") 
    quit()
    
import tkinter.messagebox
import os
import time
import sys
import tkinter.filedialog
import threading
import tkinter.colorchooser
import random
import subprocess
import socket
from tkinter.simpledialog import askinteger
from tkinter.simpledialog import askstring
from tkinter.colorchooser import askcolor  #python3                      ((0.0, 222, 255.99), '#63deaa')  //0-256!!!!
#from tkColorChooser import askcolor       #python2   color=askcolor()   ((99, 222, 170), '#63deaa')

try:
    import winsound
except ImportError:
    print("Cannot load winsound")
    


#---------------------------------------------------------------------------------------
# U S E R   C O N F I G   V A R I A B L E S
#---------------------------------------------------------------------------------------

# Liste des ports series sur lesquels sont connectes les centrales
# Pensez a renseigner le numero de la carte dans les centrales
# Avec 1 port  serie , le logiciel s'attend a avoir la carte 0    
# Avec 2 ports series, le logiciel s'attend a avoir les cartes 0 et 1 ...  
# Ex sous Windows:  
# - LIST_SERIAL_PORTS = [ "\\\\.\\COM6" ] 
# - LIST_SERIAL_PORTS = [ "\\\\.\\COM6", "\\\\.\\COM4" ] 
# Ex sous Linux:
# - LIST_SERIAL_PORTS = [ "/dev/ttyACM0" ] 
# - LIST_SERIAL_PORTS = [ "/dev/ttyUSB0" ] 
# - LIST_SERIAL_PORTS = [ "/dev/ttyACM0", "/dev/ttyACM1" ] 
LIST_SERIAL_PORTS = [ "/dev/ttyUSB000000" ] 

# Numero du port TCP-IP pour y connecter les cabines et TCO distants 
fdcc_server_port = 1234

# vitesses a utiliser pour la conduite automatique en mode cantonnement (crans 0-28) 
CAC_MAX  = 28
CAC_RAL  = 8
CAC_STOP = 0
 
    
#---------------------------------------------------------------------------------------
# G L O B A L   V A R I A B L E S
#---------------------------------------------------------------------------------------
   
SZ_TITLE = "Free-DCC-2017-PC v20180130"
print(SZ_TITLE)
normal_dbg = 0
    
top = tkinter.Tk()

print("PYTHON VERSION = " + sys.version[0:6] )
python_vers = 3


#---------------------------------------------------------------------------------------
# RS232
#---------------------------------------------------------------------------------------

rs232_ok = 1
try:
    import serial
    print("RS232: PYSERIAL VERSION = " + serial.VERSION)
except ImportError:
    rs232_ok = 0
    print("RS232: Cannot load serial lib")

MAX_BOARD_NB = len(LIST_SERIAL_PORTS)


# Commandes du protocol "UU" (toute autre commande recue est sans effet)

RS232_CMD_PING     = 0x00   # PING      0x00 -> 0x83
RS232_CMD_VERS     = 0x01   # VERS      0x00 -> 0x84 -> protocol -> carte -> rev

RS232_CMD_INIT     = 0x78   # INIT      0x78 + 0.0.CLRIN.AIG.PHA.CLI.LED.OUT
RS232_CMD_CFG      = 0x79   # CFG       0x79 + action(2=aig_off, 3=aig_on)
RS232_CMD_AU0      = 0x70   # AU#       0x70
RS232_CMD_AU1      = 0x71   # AU        0x71
RS232_CMD_AUGET    = 0x72   # AUGET     0x72 -> AU

RS232_CMD_LOCO     = 0x10   # LOCO      0x10 + ADR(1-50)  + VIT(xxS[0-28])  + FCT (xxxF4F3F2F1F0)
RS232_CMD_LOCOIN   = 0x11   # LOCO-IN   0x11 + ADR(1-50) -> VIT(xxS[0-28]) -> FCT (xxxF4F3F2F1F0)
RS232_CMD_LOCOPGM  = 0x12   # LOCOPGM   0x12 + 0x55 + 0xAA ++ CV(1-1024) + DAT(0-255)
RS232_CMD_LOCOMODE = 0x13   # LOCOMODE  0x13 + mode (0=DCC, 1=ANALOGIQE, 2=0, 3=1)

RS232_CMD_MOUSE    = 0x60   # MOUSE     0x60 + #mou(0-9) + adr    (ne fait plus: + vit + fct)
RS232_CMD_MOUSEIN  = 0x61   # MOUSE-IN  0x61 + #mou(0-9) -> adr                  (adr.bit7=disconnected)
RS232_CMD_MOUSEIN2 = 0x62   # MOUSE-IN2 0x62 + #mou(0-9) -> adr -> vit -> fct    (adr.bit7=disconnected)

RS232_CMD_AIGDIR   = 0x20   # AIG-DIR  0x20 + #aig(0-95)
RS232_CMD_AIGDEV   = 0x21   # AIG-DEV  0x21 + #aig(0-95)
RS232_CMD_AIGPOS   = 0x22   # AIG-POS  0x21 + #aig -> T1.N6.N5.N4.N3.N2.N1.N0 + inv.pulsems(3bits).T0.pos1.pos0.cmd

RS232_CMD_OUT0     = 0x30   # OUT-OFF  0x30 + #out(0-63)
RS232_CMD_OUT1     = 0x31   # OUT-ON   0x31 + #out(0-63)
RS232_CMD_PWM      = 0x32   # PWM      0x32 + #pwm(0-19) + val   #0..51=I2C 48=pwm0..51=pwm3
RS232_CMD_SRV      = 0x33   # SRV      0x33 + #srv(0-15) + val   #0..47=I2C

RS232_CMD_LEDOFF   = 0x40   # LED-OFF  0x40 + #led(0-255)
RS232_CMD_LEDON    = 0x41   # LED-ON   0x41 + #led(0-255)
RS232_CMD_LEDCLI0  = 0x42   # LED-CLI0 0x42 + #led(0-255)
RS232_CMD_LEDCLI1  = 0x43   # LED-CLI1 0x43 + #led(0-255)
RS232_CMD_LEDPHA0  = 0x44   # LED-PHA0 0x44 + #led(0-255)
RS232_CMD_LEDPHA1  = 0x45   # LED-PHA1 0x45 + #led(0-255)
RS232_CMD_FX0      = 0x46   # FX-OFF   0x46 + #fx(0-63)
RS232_CMD_FX1      = 0x47   # FX-ON    0x47 + #fx(0-63)
RS232_CMD_NEO      = 0x48   # NEO      0x48 + #neo(0-59) + R + V + B

RS232_CMD_IN       = 0x50   # IN       0x50 -> in7..0 -> in15..8 -> ... -> in 255..248
RS232_CMD_INMEM    = 0x51   # IN-MEM   0x51 "
RS232_CMD_INMCLR   = 0x53   # IN-MCLR  0x53 "
RS232_CMD_AN       = 0x58   # AN       0x58 + #an(0-11) -> val   //0-19:mouses 10=AN0 11=AN1

RS232_CMD_BYTE     = 0x49   # BYTE     0x49 + #byte + val
RS232_CMD_BIT      = 0x4a   # BIT      0x4a + #bit  + val


# Valeurs de travail
# - valeurs actuelles initialisees avec la valeur specifiee au debut, ici 0
# - anciennes valeurs initialisees avec la valeur specifiee au debut, ici -1 pour forcer a updater a l'init

rs232_loco_nb = 50
rs232_in_nb   = 256
rs232_out_nb  = 64
rs232_led_nb  = 256
rs232_aig_nb  = 96
rs232_neo_nb  = 60
rs232_pwm_nb  = 52
rs232_srv_nb  = 48
rs232_an_nb   = 2
rs232_fx_nb   = 64


# les locomotives et souris ne sont controlees que par la centrale 0
class c_rs232_loc:
    def __init__(self):
    
        self.au_state = 0
        self.au_set   = 0
        self.au_clr   = 0

        self.vit      = [ 0 for i in range(rs232_loco_nb+1)]  #0not used 1-50
        self.sens     = [ 1 for i in range(rs232_loco_nb+1)]  #1=+=AV 0=-=AR
        self.fct0     = [ 0 for i in range(rs232_loco_nb+1)]
        self.fct1     = [ 0 for i in range(rs232_loco_nb+1)]
        self.fct2     = [ 0 for i in range(rs232_loco_nb+1)]
        self.fct3     = [ 0 for i in range(rs232_loco_nb+1)]
        self.fct4     = [ 0 for i in range(rs232_loco_nb+1)]
        self.old_vit  = [ -1 for i in range(rs232_loco_nb+1)] 
        self.old_sens = [ -1 for i in range(rs232_loco_nb+1)]
        self.old_fct0 = [ -1 for i in range(rs232_loco_nb+1)]
        self.old_fct1 = [ -1 for i in range(rs232_loco_nb+1)]
        self.old_fct2 = [ -1 for i in range(rs232_loco_nb+1)]
        self.old_fct3 = [ -1 for i in range(rs232_loco_nb+1)]
        self.old_fct4 = [ -1 for i in range(rs232_loco_nb+1)]

        self.cv_prog_go    = 0
        self.cv_prog_adr_h = 0
        self.cv_prog_adr_l = 0
        self.cv_prog_dat   = 0

        self.souris_adr  = [ 0 for i in range(10)]
        self.old_souris_adr  = [ 0 for i in range(10)]
        self.souris_get = 0
        self.souris_get_nb = 5
        self.in_mouse_sel = -1
        self.souris_disc = [ 1 for i in range(10)]
        self.souris_do_not_update = 0

rs232_loc = c_rs232_loc()


# les autres ressources sont partagees
class c_rs232_in_out:
    def __init__(self):
        self.ser         = 0
        self.connected   = 0
    
        self.inp         = [ 0 for i in range(rs232_in_nb)]
        self.old_in      = [ 0 for i in range(rs232_in_nb)]

        self.an          = [ 0 for i in range(rs232_an_nb)]

        self.out         = [  0 for i in range(rs232_out_nb)]   
        self.old_out     = [ -1 for i in range(rs232_out_nb)]

        self.led         = [  0 for i in range(rs232_led_nb)]
        self.led_cli     = [  0 for i in range(rs232_led_nb)]
        self.led_pha     = [  0 for i in range(rs232_led_nb)]
        self.old_led     = [ -1 for i in range(rs232_led_nb)]
        self.old_led_cli = [ -1 for i in range(rs232_led_nb)]
        self.old_led_pha = [ -1 for i in range(rs232_led_nb)]

        self.fx          = [  0 for i in range(rs232_fx_nb)]   
        self.old_fx      = [ -1 for i in range(rs232_fx_nb)]

        self.neo         = [  0 for i in range(rs232_neo_nb)]   
        self.old_neo     = [ -1 for i in range(rs232_neo_nb)]
        self.pwm         = [  0 for i in range(rs232_pwm_nb)]   
        self.old_pwm     = [ -1 for i in range(rs232_pwm_nb)]
        self.srv         = [150 for i in range(rs232_srv_nb)]   
        self.old_srv     = [ -1 for i in range(rs232_srv_nb)]

        self.aig_cmd     = [ 0 for i in range(rs232_aig_nb)]
        self.aig_dir     = [ 0 for i in range(rs232_aig_nb)]
        self.aig_dev     = [ 0 for i in range(rs232_aig_nb)]
        self.aig_pulse   = [ 0 for i in range(rs232_aig_nb)] #lecture seule
    
        self.carte_nb_aig = -1
        self.carte_nb_8in = -1
    
        self.aig_num    = 0
        self.aig_wait   = 0
        self.aig_ok     = 0
        self.nb_aig_nok = -1
        self.aig_order  = 0   #cmd

rs232 = []
for c in range(MAX_BOARD_NB):
    rs232.append(c_rs232_in_out())
        
        
# autres variables rs232 uu: mettre dans des attributs de class a terme    
rs232_aig_ok     = 0     
rs232_nb_aig_nok = -1
rs232_update_cpt = 0
rs232_stats = ""


def serial_tx(ser_,val):
    #global ser
    global rs232_ok
    try:
        if(python_vers == 2):
            ser_.write(chr(val))     # pyhton 2 sends characters
        if(python_vers == 3):
            ser_.write(bytes([val])) # pyhton 3 sends bytes
        ##print("tx_=" + str(val))    
    except:
        rs232_ok = 0
        print("RS232: ERROR: Perte de la liaison serie")


def serial_rx(ser_):
    #global ser
    global rs232_ok
    ##print("RRRx " + str(ser.inWaiting()) )
    try:
        if(ser_.inWaiting() == 0):
            return -1
        rb = ser_.read(1)
        if(python_vers == 2):
            rx = ord(rb[0])      # pyhton 2 returns characters
        if(python_vers == 3):
            rx = rb[0]          # python 3 returns bytes
        ##print("rx_=" + str(rx))
    except:
        rs232_ok = 0
        print("RS232: ERROR: Perte de la liaison serie")
        rx = -1
    return int(rx);
    

def rs232_init():
    #global ser;
    global rs232_ok
    #global carte_nb_aig; global carte_nb_8in;
    global rs232
    list_ser  = [ 0 for i in range(MAX_BOARD_NB) ]
    list_open = [ 0 for i in range(MAX_BOARD_NB) ]

    # 1. Essai d'ouverture du port serie
    try:
        for i in range(MAX_BOARD_NB):
            list_ser[i] = serial.Serial(LIST_SERIAL_PORTS[i], 115200, timeout = 10, stopbits = serial.STOPBITS_TWO)
            list_ser[i].setDTR(False)  # (it is possible to deactivated DTR in serialwin32.py to not reset the arduino board at port opening)
            print("RS232: OK to open serial port " + LIST_SERIAL_PORTS[i])
            list_open[i] = 1
    except:
        rs232_ok = 0
        print("RS232: Cannot open serial port " + LIST_SERIAL_PORTS[i])    
        tkinter.messagebox.showerror(SZ_TITLE, "Impossible d'ouvrir le port serie " + LIST_SERIAL_PORTS[i] + "\n\nErreurs possibles:\n- La carte Arduino n'est pas connectee\n- La carte Arduino est connectee sur un autre port\n- Un autre programme utilise deja ce port\n- Il y a un probleme avec le driver de la carte Arduino\n\nFonctionnement en mode deconnecte !")
        for i in range(MAX_BOARD_NB):
            if(list_open[i]):
                list_ser[i].close()    
        return

    # opening the port, resets the board, wait few seconds after board startup
    if(MAX_BOARD_NB == 1):
        print("RS232: Attente demarrage de la carte Arduino (apres RESET)") 
    else:
        print("RS232: Attente demarrage des cartes Arduino (apres RESET)") 
    #time.sleep(2)

    # 2. Essai de detection au plus tot du demarrage des cartes puis vidange
    for i in range(MAX_BOARD_NB):
        list_ser[i].flushInput()
    
    for i in range(20):      
        for c in range(MAX_BOARD_NB):
            serial_tx(list_ser[c], RS232_CMD_PING)
        time.sleep(0.3)
        for c in range(MAX_BOARD_NB):
            rx = serial_rx(list_ser[c])
            if(rx == -1):
                break
        if(rx != -1):
            print("RS232: Temps de demarrage = " + str(300 * i) + "ms")
            break
               
    time.sleep(0.3)
    for i in range(MAX_BOARD_NB):
        list_ser[i].flushInput()

    # 3. Test du ping 0->0x83
    print("RS232: TEST PING")
    for i in range(MAX_BOARD_NB):
        serial_tx(list_ser[i], RS232_CMD_PING)
    time.sleep(0.3)
    for i in range(MAX_BOARD_NB):
        rx = serial_rx(list_ser[i])
        if(rx == -1):
            tkinter.messagebox.showerror(SZ_TITLE, "Aucune reponse a la demande de ping de la carte Arduino sur " + LIST_SERIAL_PORTS[i] + "\n\nFonctionnement en mode deconnecte !")
            rs232_ok = 0
            for i in range(MAX_BOARD_NB):
                if(list_open[i]):
                    list_ser[i].close() 
            return
        if(rx != 0x83):
            tkinter.messagebox.showerror(SZ_TITLE, "Mauvaise reponse a la demande d'authentification de la carte Arduino sur " + LIST_SERIAL_PORTS[i] + " > " + str(rx) +" au lieu de 131\n\nFonctionnement en mode deconnecte !")
            rs232_ok = 0
            for i in range(MAX_BOARD_NB):
                if(list_open[i]):
                    list_ser[i].close() 
    print("RS232: PING OK")
        
    # 4. Test de l'authentification 1->0x83 1 'A'/'B' 1
    print("RS232: TEST ID")
    for i in range(MAX_BOARD_NB):
        serial_tx(list_ser[i], 1)
    time.sleep(0.3)
    list_arx = []
    for i in range(MAX_BOARD_NB):
        arx = [ 0 for j in range(16)]
        for j in range(16):
            arx[j] = serial_rx(list_ser[i])
        list_arx.append(arx)
        print("RS232: ID: " + str(arx))
        ok = 1
        if(arx[0] != 0x84):
            ok = 0
        if(arx[1] != 1):
            ok = 0
        if(arx[2] != ord('A') and arx[2] != ord('B')):
            ok = 0
        if(arx[3] != 1):
            ok = 0    #centrale_num
            
        if(ok == 0):
            tkinter.messagebox.showerror(SZ_TITLE, "Mauvaise Identification de la carte Arduino sur " + LIST_SERIAL_PORTS[i] + "\n\nFonctionnement en mode deconnecte !")
            rs232_ok = 0
            for i in range(MAX_BOARD_NB):
                if(list_open[i]):
                    list_ser[i].close() 
            return    
            
    print("RS232: TEST ID OK")
    
    #toutes les cartes ont bien repondus
    
    print("RS232: LISTE DES CENTRALES:")
    for i in range(MAX_BOARD_NB):
        print("RS232: CENTRALE:")
        print("RS232: - PORT   = " + LIST_SERIAL_PORTS[i] + ":")
        #verification du numero des centrales (doit etre entre 0-3 et unique !)
        num_centrale = list_arx[i][4]
        print("RS232: - numero = " + str(num_centrale))
        if(num_centrale >= MAX_BOARD_NB):
            print("RS232: numero trop grand ! 0-" + str(MAX_BOARD_NB - 1))
            tkinter.messagebox.showerror(SZ_TITLE, "Numero de centrale invalide " + str(num_centrale) + " au lieu de 0-" + str(MAX_BOARD_NB - 1) + "\n\n centrale sur " + LIST_SERIAL_PORTS[i] + "\n\nFonctionnement en mode deconnecte !")
            rs232_ok = 0
            for i in range(MAX_BOARD_NB):
                if(list_open[i]):
                    list_ser[i].close() 
            return
        if(rs232[num_centrale].connected):
            print("RS232: numero deja utilise !")
            tkinter.messagebox.showerror(SZ_TITLE, "Numero de centrale en deja utilise " + str(num_centrale) + "\n\n centrale sur " + LIST_SERIAL_PORTS[i] + "\n\nFonctionnement en mode deconnecte !")
            rs232_ok = 0
            for i in range(MAX_BOARD_NB):
                if(list_open[i]):
                    list_ser[i].close() 
            return  
            
        rs232[num_centrale].connected = 1
        rs232[num_centrale].ser       = list_ser[i]
        if(list_arx[i][2] == ord('A')):
            print("RS232: - type   = A")
            rs232[num_centrale].centrale_type = 'A'
        elif(list_arx[i][2] == ord('B')):
            print("RS232: - type   = B")
            rs232[num_centrale].centrale_type = 'B'
        else:
            tkinter.messagebox.showerror(SZ_TITLE, "Type de centrale non supporte" + "\n\nFonctionnement en mode deconnecte !")
            print("RS232: - type = NON SUPPORTE")
            rs232_ok = 0
            for i in range(MAX_BOARD_NB):
                if(list_open[i]):
                    list_ser[i].close() 
            return  
        rs232[num_centrale].carte_nb_aig = list_arx[i][7]
        rs232[num_centrale].carte_nb_8in = list_arx[i][6]   
                    
    # 5. initialisations
    if(rs232[0].connected):
        serial_tx(rs232[0].ser, RS232_CMD_AU1)   # entre en AU pendant l'initialisation
    for c in range(MAX_BOARD_NB):
        if(rs232[c].connected):
            serial_tx(rs232[c].ser, RS232_CMD_CFG); serial_tx(rs232[c].ser, 2);    # desactivation de mise a jour des aiguillages
            #ser.write(bytes([RS232_CMD_INIT,0xff])) # INIT      0x78 + 0.0.CLRIN.AIG.PHA.CLI.LED.OUT

if(rs232_ok):
    rs232_init()
    
def rs232_get_pulse(c):
    global rs232
    
    print("AIG: CENTRALE " + str(c))
    for i in range(96):
        rs232[c].aig_pulse[i] = 0
        
    left = rs232[c].carte_nb_aig
    base = 0
    while(left):
        nb = left
        if(nb > 30):
            nb = 30
        for i in range(base, base + nb):    
            rs232[c].ser.write(bytes([RS232_CMD_AIGPOS, i]))   # // AIG-POS  0x21 + #aig -> T1.N6.N5.N4.N3.N2.N1.N0 + inv.pulsems(3bits).T0.pos1.pos0.cmd
        rb = rs232[c].ser.read(2 * nb)
        for i in range(nb):
            pulse = int(256 * rb[2 * i] + rb[2 * i + 1])
            sz = "AIG" + str(1000 * c + base + i) + " cfg=" + str(pulse)
            
            pulse = (pulse >> 4) & 0x7
            rs232[c].aig_pulse[base + i] = pulse
            
            if(pulse == 0):
                continue
        
            if(pulse == 0):
                sz += " inutilise"
            else:
                sz += "pulse="        
            if(pulse == 1):
                sz += "125ms"
            if(pulse == 2):
                sz += "250ms"
            if(pulse == 3):
                sz += "500ms"
            if(pulse == 4):
                sz += "1s"
            if(pulse == 5):
                sz += "2s"
            if(pulse == 6):
                sz += "5s"
            if(pulse == 7):
                sz += "10s"
            print(sz)
            
        base += 30
        left -= nb        
    return
    
if(rs232_ok):
    for c in range(MAX_BOARD_NB):
        if(rs232[c].connected):
            rs232_get_pulse(c)


def rs232_update():
    global rs232_ok
    global rs232_update_cpt  
    global rs232_stats
    global rs232_nb_aig_nok
    global rs232_update_cpt
    
    global rs232
    global rs232_loc
    
    if(rs232_ok == 0):
        return

    rs232_update_cpt += 1
    if(normal_dbg):    
        print("RS232: update (" + str(rs232_update_cpt) + ")")
    t1 = time.time()

    #while(ser.inWaiting()):   #ser.in_waiting from pyserial 3.0
    #    rx = serial_rx()
    #    print("rx="+str(rx))            #read in and sync (not checked at the moment)

    # A: creation des donnees a ECRIRE   

    awb = [0 for i in range(MAX_BOARD_NB)]
    for c in range(MAX_BOARD_NB):
        if(rs232[c].connected == 0):
            continue

        rs232[c].ser.flushInput()
            
        wb = []

        # 1. ping
        wb.append(0)
        
        # 1.5. gestion AU
        if(c == 0):    #AU, loco, souris only for centrale 0
            if(rs232_loc.au_set):
                rs232_loc.au_set = 0
                wb.append(RS232_CMD_AU1)
            if(rs232_loc.au_clr):
                rs232_loc.au_clr = 0
                wb.append(RS232_CMD_AU0)
            wb.append(RS232_CMD_AUGET)   #demande de l'etat d'arret d'urgence
    
        # 2. Lecture des entrees IN
        wb.append(RS232_CMD_INMCLR) #lecture des entree 0-255
    
        # 2.5. Lecture des AN
        wb.append(RS232_CMD_AN); wb.append(10);  #AN0
        wb.append(RS232_CMD_AN); wb.append(11);  #AN1
        nb_rd_an = 2
    
        # 3. loco   
        if(c == 0):    #AU, loco, souris only for centrale 0
            for i in range(1,51):
                new = 0
                vit = rs232_loc.vit[i]; old_vit = rs232_loc.old_vit[i];
                if(vit != old_vit):
                    new = 1
                sens = rs232_loc.sens[i]; old_sens = rs232_loc.old_sens[i];
                if(sens != old_sens):
                    new = 1
                fct0 = rs232_loc.fct0[i]; old_fct0 = rs232_loc.old_fct0[i];
                if(fct0 != old_fct0):
                    new = 1
                fct1 = rs232_loc.fct1[i]; old_fct1 = rs232_loc.old_fct1[i];
                if(fct1 != old_fct1):
                    new = 1
                fct2 = rs232_loc.fct2[i]; old_fct2 = rs232_loc.old_fct2[i];
                if(fct2 != old_fct2):
                    new = 1
                fct3 = rs232_loc.fct3[i]; old_fct3 = rs232_loc.old_fct3[i];
                if(fct3 != old_fct3):
                    new = 1
                fct4 = rs232_loc.fct4[i]; old_fct4 = rs232_loc.old_fct4[i];
                if(fct4 != old_fct4):
                    new = 1
                
                if(new == 0):
                    continue;
                        
                # 3) dcc command (adr + sens + vit + fct)
                wb.append(0x10)
                wb.append(i)
                cran = vit
                if(sens):
                    cran |= 0x20
                wb.append(cran)
                fct_43210 = 0
                if(fct0):
                    fct_43210 |= 1
                if(fct1):
                    fct_43210 |= 2
                if(fct2):
                    fct_43210 |= 4
                if(fct3):
                    fct_43210 |= 8
                if(fct4):
                    fct_43210 |= 16            
                wb.append(fct_43210)     

                rs232_loc.old_vit[i]  = vit;
                rs232_loc.old_sens[i] = sens;
                rs232_loc.old_fct0[i] = fct0
                rs232_loc.old_fct1[i] = fct1
                rs232_loc.old_fct2[i] = fct2
                rs232_loc.old_fct3[i] = fct3
                rs232_loc.old_fct4[i] = fct4

        if(c == 0):    #AU, loco, souris only for centrale 0
            # 3.5. Souris
            # changement des souris (si necessaire) 
            for i in range(10):
                if(rs232_loc.souris_adr[i] != rs232_loc.old_souris_adr[i]):
                    wb.append(RS232_CMD_MOUSE)    # MOUSE  0x60 + #mou(0-9) + adr     = 0x60
                    wb.append(i)
                    wb.append(rs232_loc.souris_adr[i])
                    rs232_loc.old_souris_adr[i] = rs232_loc.souris_adr[i]
            
            # lecture des souris (une par update)
            n = rs232_loc.souris_get
            for i in range(rs232_loc.souris_get_nb):
                wb.append(RS232_CMD_MOUSEIN2)    # MOUSE-IN2 0x62 + #mou(0-9) -> adr -> vit -> fct      
                wb.append(n)                     # MOUSE-IN  0x61 + #mou(0-9) -> adr
                n += 1
                if(n == 10):
                    n = 0
    
        # 4. Mise a jour des sorties OUT
        for i in range(rs232_out_nb):
            if(rs232[c].out[i] == rs232[c].old_out[i]):
                continue
            if(rs232[c].out[i] == 0):
                wb.append(RS232_CMD_OUT0)
            else:
                wb.append(RS232_CMD_OUT1)
            wb.append(i)
            rs232[c].old_out[i] = rs232[c].out[i]
        
        # 4.1. Mise a jour des sorties PWM
        for i in range(rs232_pwm_nb):
            val = rs232[c].pwm[i]
            if(val == rs232[c].old_pwm[i]):
                continue
            wb.append(RS232_CMD_PWM)
            wb.append(i)
            wb.append(val)
            rs232[c].old_pwm[i] = val   

        # 4.2. Mise a jour des sorties Servos
        for i in range(rs232_srv_nb):
            val = rs232[c].srv[i]
            if(val == rs232[c].old_srv[i]):
                continue
            wb.append(RS232_CMD_SRV)
            wb.append(i)
            wb.append(val)                 #uu: voir si on garde cote fdcc 50->250  (ou si on passe a 500->2500 pour etre en us)
            rs232[c].old_srv[i] = val   

        # 5. Mise a jour des leds
        for i in range(rs232_led_nb):
            if(rs232[c].led[i] == rs232[c].old_led[i]):
                continue
            if(rs232[c].led[i] == 0):
                wb.append(RS232_CMD_LEDOFF)
            else:
                wb.append(RS232_CMD_LEDON)
            wb.append(i)
            rs232[c].old_led[i] = rs232[c].led[i]

        for i in range(rs232_led_nb):
            if(rs232[c].led_cli[i] == rs232[c].old_led_cli[i]):
                continue
            if(rs232[c].led_cli[i] == 0):
                wb.append(RS232_CMD_LEDCLI0)
            else:
                wb.append(RS232_CMD_LEDCLI1)
            wb.append(i)
            rs232[c].old_led_cli[i] = rs232[c].led_cli[i]

        for i in range(rs232_led_nb):
            if(rs232[c].led_pha[i] == rs232[c].old_led_pha[i]):
                continue
            if(rs232[c].led_pha[i] == 0):
                wb.append(RS232_CMD_LEDPHA0)
            else:
                wb.append(RS232_CMD_LEDPHA1)
            wb.append(i)
            rs232[c].old_led_pha[i] = rs232[c].led_pha[i]

        # 5.1. Mise a jour des sorties NEO
        for i in range(rs232_neo_nb):
            rvb = rs232[c].neo[i]
            if(rvb == rs232[c].old_neo[i]):
                continue
            rs232[c].old_neo[i] = rvb
            wb.append(RS232_CMD_NEO)
            wb.append(i)
            rvb = rs232[c].neo[i]
            r = rvb >> 16
            v = (rvb >> 8) & 0xff
            b = rvb & 0xff
            wb.append(r)
            wb.append(v)
            wb.append(b)

        # 5.2. Mise a jour des effets speciaux
        for i in range(rs232_fx_nb):
            if(rs232[c].fx[i] == rs232[c].old_fx[i]):
                continue
            if(rs232[c].fx[i] == 0):
                wb.append(RS232_CMD_FX0)
            else:
                wb.append(RS232_CMD_FX1)
            wb.append(i)
            rs232[c].old_fx[i] = rs232[c].fx[i]
        
        # 6. Mise a jour des aiguillages (un par un et en attendant le feedback)
        if(rs232[c].aig_wait == 0):     #nouvel ordre uniquement s'il n'y a pas un aiguillage en cours de manoeuvre
            for i in range(rs232_aig_nb):
                if(rs232[c].aig_pulse[rs232[c].aig_num] == 0):   #aiguillage non supporte par la centrale
                    rs232[c].aig_num += 1
                    if(rs232[c].aig_num == rs232_aig_nb):
                         rs232[c].aig_num = 0
                    continue
                if(rs232[c].aig_dir[rs232[c].aig_num]):
                    pos = 0
                elif(rs232[c].aig_dev[rs232[c].aig_num]):
                    pos = 1
                else:
                    pos = 2
                if(rs232[c].aig_cmd[rs232[c].aig_num] == pos):
                    rs232[c].aig_num += 1
                    if(rs232[c].aig_num == rs232_aig_nb):
                        rs232[c].aig_num = 0
                    continue
                rs232[c].aig_wait = 1
                rs232[c].aig_dir[rs232[c].aig_num] = 0
                rs232[c].aig_dev[rs232[c].aig_num] = 0
                if(rs232[c].aig_cmd[rs232[c].aig_num] == 0):
                    wb.append(RS232_CMD_AIGDIR)  #direct
                    rs232[c].aig_order = 0
                else:
                    wb.append(RS232_CMD_AIGDEV)  #devie
                    rs232[c].aig_order = 1
                wb.append(rs232[c].aig_num)
                break
   
        # 7. Demande de feedbcak pour l'aiguillage en mouvement
        # fait meme si aucun aiguillage en mouvement afin d'avoir toujours le meme flux de reception
        wb.append(RS232_CMD_AIGPOS)
        wb.append(rs232[c].aig_num)
    
        # 8. Prog
        if(c == 0):    #AU, loco, souris only for centrale 0
            if(rs232_loc.cv_prog_go):
                wb.append(RS232_CMD_LOCOPGM)  # LOCOPGM   0x12 + 0x55 + 0xAA ++ CV(1-1024) + DAT(0-255)
                wb.append(0x55)
                wb.append(0xAA)
                wb.append(rs232_loc.cv_prog_adr_h)
                wb.append(rs232_loc.cv_prog_adr_l)
                wb.append(rs232_loc.cv_prog_dat)
                rs232_loc.cv_prog_go = 0
    
        # 9. ping
        wb.append(0)
        
        awb[c] = wb      
    
    
    # B. envoie des commandes (par lot de 62bytes). (62 car la taille du buffer de reception de l'arduino est de 64bytes)
    #    (Pas besoin pour les reponses car == 39 (1+32+1+3+1+1).
    if(normal_dbg):
        for c in range(MAX_BOARD_NB):
            if(rs232[c].connected):
                wb = awb[c]
                nb = len(wb)
                print("RS232: CENTRALE " + str(c) + " WR nb=" + str(nb) + ">" + str(wb))

    base = 0
    end = 0
    ntx = 0
    while(end == 0):
        end   = 1
        nbmax = 0
        for c in range(MAX_BOARD_NB):
            if(rs232[c].connected == 0):
                continue
            wb = awb[c]
            nb = len(wb) - base
            if(nb <= 0):
                continue
            if(nb > nbmax):
                nbmax = nb
            if(nb > 62):
                nb = 62
                end = 0
            for i in range(nb):
                serial_tx(rs232[c].ser, wb[base+i])
                ntx += nb
        if(base):
            print(str(nbmax // 62))   # aff pr les gros transfer (>62bytes)
        base += 62
        if(end == 0):
            time.sleep(0.2)   ##uu:essayer sans attendre car c le buffer windows qui doit faire tampon
            

    # C. reception des data
    #   [0] [1]  [2..33]        [34,35]     [36..50]        [51] [52] 
    for c in range(MAX_BOARD_NB):
        if(rs232[c].connected == 0):
            continue
    
        if(c == 0):
            nb = 1 + 1 + rs232[c].carte_nb_8in + nb_rd_an + 3 * rs232_loc.souris_get_nb + 2 + 1  
            # 0x83 + AU + 32*8in + 2*an + souris_get_nb*3*mouse feedback + aig feedbcak(2bytes) + 0x83 = 54bytes (32in8/5mouses/2an)
            # 0x83 + AU +  4*8in + 2*an + souris_get_nb*3*mouse feedback + aig feedbcak(2bytes) + 0x83 = 22bytes (04in8/5mouses/2an)
        else:
            nb = 1 + rs232[c].carte_nb_8in + nb_rd_an + 2 + 1  
        
        rb = []
        w = 0
        nrx = 0
        while(1):
            rx = serial_rx(rs232[c].ser)
            if(rx == -1):
                time.sleep(0.010)
                w += 1
                if(w < 20):   #wait 0.2s max
                    continue
                break
            rb.append(rx)
            nrx += 1
            if(nrx == nb):
                break
        ####
        if(normal_dbg):
            print("RS232: CENTRALE " +str(c) + " RD nb=" + str(len(rb)) + ">" + str(rb))

        #1. verification probleme du lien
        if(len(rb) == 0):
            rs232_ok = 0
            for c in range(MAX_BOARD_NB):
                if(rs232[c].connected):
                    rs232[c].ser.close()
            print("RS232: NO REPLAY")
            tkinter.messagebox.showerror(SZ_TITLE, "Aucune reponse de la carte Arduino\n\nFonctionnement en mode deconnecte !")
            return
        if(len(rb) != nb):
            rs232_ok = 0
            for c in range(MAX_BOARD_NB):
                if(rs232[c].connected):
                    rs232[c].ser.close()
            print("RS232: Mauvaise quantite de donnee recu " + str(len(rb)) + " a la place de " + str(nb))
            tkinter.messagebox.showerror(SZ_TITLE, "Mauvaise quantite de donnee recu " + str(len(rb)) + " a la place de " + str(nb) + "\n\nFonctionnement en mode deconnecte !")
            return
        if(int(rb[0]) != 0x83):
            rs232_ok = 0
            for c in range(MAX_BOARD_NB):
                if(rs232[c].connected):
                    rs232[c].ser.close()
            print("RS232: Mauvaise reponse a la 1ere demande d'authentification de la carte Arduino " + str(rb[0]) +" au lieu de 131")
            tkinter.messagebox.showerror(SZ_TITLE, "Mauvaise reponse a la 1ere demande d'authentification de la carte Arduino " + str(rb[0]) +" au lieu de 131\n\nFonctionnement en mode deconnecte !")
            return 
        if(int(rb[nb-1]) != 0x83):
            rs232_ok = 0
            for c in range(MAX_BOARD_NB):
                if(rs232[c].connected):
                    rs232[c].ser.close()
            print("RS232: Mauvaise reponse a la 2eme demande d'authentification de la carte Arduino " + str(rb[carte_nb_8in+6]) +" au lieu de 131")
            tkinter.messagebox.showerror(SZ_TITLE, "Mauvaise reponse a la 2eme demande d'authentification de la carte Arduino " + str(rb[carte_nb_8in+6]) +" au lieu de 131\n\nFonctionnement en mode deconnecte !")
            return
       
        # 1.5. mise a jour de l'AU
        if(c == 0):
            if(int(rb[1])):
                rs232_loc.au_state = 1
            else:       
                rs232_loc.au_state = 0    

        # 2. mise a jour des entrees
        if(c == 0):
            st = 2
        else:
            st = 1
        for i in range(rs232_in_nb):
            rs232[c].old_in[i] = rs232[c].inp[i]
            rs232[c].inp[i] = 0
            if(i<(8*rs232[c].carte_nb_8in)):
                if(int(rb[(i // 8) + st] & (1 << (i % 8)))):
                     rs232[c].inp[i] = 1
                 
        # 3. mise a jour des entrees analogiques
        for i in range(nb_rd_an):
            rs232[c].an[i] = int(rb[st+rs232[c].carte_nb_8in + i]);

        # 4. mise a jour feedback souris
        # MOUSE-IN2 0x62 + #mou(0-9) -> adr -> vit -> fct       
        # MOUSE-IN  0x61 + #mou(0-9) -> adr
        if(c == 0):
            for i in range(rs232_loc.souris_get_nb):
                adr = int(rb[2+rs232[c].carte_nb_8in+nb_rd_an + 3 * i])
                rs232_loc.souris_disc[rs232_loc.souris_get] = adr >> 7
                #if(adr&0x80):
                #    souris_disc[souris_get]=1
                #else:
                #    souris_disc[souris_get]=0   
                adr &= 0x7f #remove disconnect flag 
                vit = int(rb[2+rs232[c].carte_nb_8in+nb_rd_an + 3 * i + 1])
                fct = int(rb[2+rs232[c].carte_nb_8in+nb_rd_an + 3 * i + 2])
                # print("SOURIS_GET=" + str(souris_get) + " ADR=" + str(adr) + " VIT=" + str(vit) + " FCT=" + str(fct))    
                if(adr):           
                    rs232_loc.old_vit[adr]  = rs232_loc.vit[adr]  = vit & 0x1f
                    rs232_loc.old_sens[adr] = rs232_loc.sens[adr] = (vit>>5) & 1
                    rs232_loc.old_fct0[adr] = rs232_loc.fct0[adr] = fct & 1
                    rs232_loc.old_fct1[adr] = rs232_loc.fct1[adr] = (fct>>1) & 1
                    rs232_loc.old_fct2[adr] = rs232_loc.fct2[adr] = (fct>>2) & 1 
                    rs232_loc.old_fct3[adr] = rs232_loc.fct3[adr] = (fct>>3) & 1
                    rs232_loc.old_fct4[adr] = rs232_loc.fct4[adr] = (fct>>4) & 1
                if(rs232_loc.souris_do_not_update == 0):
                    if(rs232_loc.souris_get != rs232_loc.in_mouse_sel):
                        try: #car fct pas encore definit aux premiers appels
                            loco_move_souris_adr(rs232_loc.souris_get,adr)   #uu:check
                            souris_adr[rs232_loc.souris_get] = adr
                        except:
                            pass
                rs232_loc.souris_get += 1
                if(rs232_loc.souris_get == 10):
                    rs232_loc.souris_get = 0
            if(rs232_loc.souris_do_not_update):
                rs232_loc.souris_do_not_update -= 1
                 
        # 3. mise a jour feedback aiguillage
        aig_rep = int(rb[ nb - 2])    # AIG-POS  0x21 + #aig -> T1.N6.N5.N4.N3.N2.N1.N0 + inv.pulsems(3bits).T0.pos1.pos0.cmd  (rb[nb-2] donne le second byte, celui qui nous interesse)
        if(rs232[c].aig_wait):
            if(rs232[c].aig_order == 0 and aig_rep & 2):  #position 0 stable
                rs232[c].aig_dir[rs232[c].aig_num] = 1
                rs232[c].aig_dev[rs232[c].aig_num] = 0
                rs232[c].aig_wait = 0
                rs232[c].aig_num += 1
                if(rs232[c].aig_num == rs232_aig_nb):
                    rs232[c].aig_num = 0
            if(rs232[c].aig_order == 1 and aig_rep & 4):  #position 1 stable
                rs232[c].aig_dir[rs232[c].aig_num] = 0
                rs232[c].aig_dev[rs232[c].aig_num] = 1
                rs232[c].aig_wait = 0
                rs232[c].aig_num += 1
                if(rs232[c].aig_num == rs232_aig_nb):
                    rs232[c].aig_num = 0

        for i in range(rs232_aig_nb):
            if(rs232[c].aig_dir[i]):
                pos = 0
            elif(rs232[c].aig_dev[i]):
                pos = 1
            else:
                pos = 2
            if(rs232[c].aig_cmd[i] != pos):
                rs232[c].aig_dir[i] = 0
                rs232[c].aig_dev[i] = 0
            elif(rs232[c].aig_cmd[i] == 0):
                rs232[c].aig_dir[i] = 1
                rs232[c].aig_dev[i] = 0
            else:
                rs232[c].aig_dir[i] = 0
                rs232[c].aig_dev[i] = 1

        rs232[c].aig_ok     = 1
        rs232[c].nb_aig_nok = 0
        for i in range(rs232_aig_nb):
            if(rs232[c].aig_pulse[i] == 0):   #aiguillage non supporte par la centrale
                continue
            if(rs232[c].aig_dir[i]):
                pos = 0
            elif(rs232[c].aig_dev[i]):
                pos = 1
            else:
                pos = 2
            if(rs232[c].aig_cmd[i] != pos):
                rs232[c].aig_ok      = 0
                rs232[c].nb_aig_nok += 1
                    
    rs232_aig_ok     = 1
    rs232_nb_aig_nok = 0
    for c in range(MAX_BOARD_NB):
        if(rs232[c].connected):
            if(rs232[c].aig_ok == 0):
                rs232_aig_ok = 0    
            rs232_nb_aig_nok += rs232[c].nb_aig_nok
    t2 = time.time()
    ####print("RS232: cpt=" + str(rs232_update_cpt) + " TXbytes=" + str(ntx) + " RXbytes=" + str(nrx) +" time=" + str(t2-t1))
    rs232_stats = "TX=" + str(ntx) + " RX=" + str(nrx) + " AIGnok=" + str(rs232_nb_aig_nok)
        
    return
   
    
# simulation du deplacement des aiguillages lorsque la carte n'est pas connectee
def no_rs232_update():
    global rs232
    global rs232_loc
    global rs232_aig_ok
    global rs232_nb_aig_nok
    global rs232_stats
    
    for c in range(MAX_BOARD_NB):
        for i in range(rs232_aig_nb):
            if(rs232[c].aig_cmd[i] == 0):  #direct
                rs232[c].aig_dir[i] = 1
                rs232[c].aig_dev[i] = 0
            else:
                rs232[c].aig_dir[i] = 0
                rs232[c].aig_dev[i] = 1
    rs232_aig_ok     = 1
    rs232_nb_aig_nok = 0
    
    if(rs232_loc.au_clr):
        rs232_loc.au_clr   = 0
        rs232_loc.au_state = 0
    if(rs232_loc.au_set):
        rs232_loc.au_set   = 0
        rs232_loc.au_state = 1
        
    for c in range(MAX_BOARD_NB):
        for i in range(rs232_in_nb):
            rs232[c].old_in[i] = rs232[c].inp[i]
    
    rs232_stats = "TX=0 RX=0 AIGnok=0"
    return

    
if(rs232_ok):
    print("RS232: Initialisation des loco/entrees/sorties/led... de la carte Arduino")   
    rs232_update()
    rs232_update()

if(rs232_ok):
    print("RS232: Electronique initialisee, Vous pouvez activer la puissance !")
    if(rs232[0].connected):
        serial_tx(rs232[0].ser, RS232_CMD_AU0)   # on enleve l'AU
    tkinter.messagebox.showinfo(SZ_TITLE, "Electronique initialisee\n\nVous pouvez activer la puissance !")
    for c in range(MAX_BOARD_NB):
        if(rs232[c].connected):
            serial_tx(rs232[c].ser, RS232_CMD_CFG); serial_tx(rs232[c].ser, 3); #activation des aiguillages
    
  
#---------------------------------------------------------------------------------------
# Entrees/Sorties
#---------------------------------------------------------------------------------------

esb = 0  # selected e/s board

led_nb = 256  #uu: essayer d'utiliser les variables rs232
out_nb = 64
fx_nb  = 64
aig_nb = 96
pwm_nb = rs232_pwm_nb
srv_nb = rs232_srv_nb
neo_nb = 60
in_nb  = 256
an_nb  = 2

class c_es():
    def __init__(self):
        self.c_var_in_mem = 0
        self.b_var_in_clr = 0

        self.led_on   = [ 0 for i in range(led_nb)]
        self.led_lock = [ 1 for i in range(led_nb)]
        self.out_on   = [ 0 for i in range(out_nb)]
        self.out_lock = [ 1 for i in range(out_nb)]
        self.fx_on    = [ 0 for i in range(fx_nb )]
        self.fx_lock  = [ 1 for i in range(fx_nb )]
        self.aig_on   = [ 0 for i in range(aig_nb)]
        self.aig_lock = [ 1 for i in range(aig_nb)]
        self.pwm      = [ 0 for i in range(pwm_nb)]
        self.pwm_lock = [ 1 for i in range(pwm_nb)]
        self.srv      = [ 0 for i in range(srv_nb)]
        self.srv_lock = [ 1 for i in range(srv_nb)]
        self.neo_rvb  = [ 0 for i in range(neo_nb)]
        self.neo_lock = [ 1 for i in range(neo_nb)]
        self.in_on    = [ 0 for i in range(in_nb )]
        
es = []
for c in range(MAX_BOARD_NB):
    es.append(c_es())
        
        
def in_clr_cb():
    global es
    es[esb].b_var_in_clr = 1
    return

def in_mem_cb():
    global es
    if(es[esb].c_var_in_mem == 0):
        es[esb].c_var_in_mem = 1; c_in_mem.select()
    else:
        es[esb].c_var_in_mem = 0; c_in_mem.deselect()
    return

l_in = tkinter.Label( top, text = "in (rouge=off, vert=on)")
l_led = tkinter.Label(top, text = "led (rouge=off, vert=on, jaune=cli(phase0), bleu=cli(phase1))")
l_out = tkinter.Label(top, text = "out (rouge=off, vert=on)")
l_aig = tkinter.Label(top, text = "aig (petit rect=cmd, gros=position, rouge=direct, vert=devie, gris=maneuvre)")
b_in_clr = tkinter.Button(top, text = "clr", command = in_clr_cb)
c_in_mem = tkinter.Checkbutton(top, text = "mem", command = in_mem_cb)
l_pwm = tkinter.Label(top, text = "pwm")
l_srv = tkinter.Label(top, text = "srv")
l_neo = tkinter.Label(top, text = "neo")
l_an0 = tkinter.Label(top, text = "AN0 = .../255 = ....mV", font = ("Helvetica", 16), fg = "#880000")
l_an1 = tkinter.Label(top, text = "AN1 = .../255 = ....mV", font = ("Helvetica", 16), fg = "#880000")

def pot_r_cb(val):
    return
def pot_v_cb(val):
    return
def pot_b_cb(val):
    return

sel_pwm = -1        
def pot_pwm_cb(val):
    if(sel_pwm == -1):    
        return
    es[esb].pwm[sel_pwm] = pot_pwm.get()
    debug_pwm_redraw(sel_pwm, 0)
    
sel_srv = -1            
def pot_srv_cb(val):
    es[esb].srv[sel_srv] = pot_srv.get()
    debug_srv_redraw(sel_srv)
    return
    
l_r = tkinter.Label(top, text = "R"); pot_r = tkinter.Scale(top, from_ = 255, to = 0, orient = tkinter.VERTICAL, command = pot_r_cb, length = 200); pot_r.set(0) #tickinterval=255
l_v = tkinter.Label(top, text = "V"); pot_v = tkinter.Scale(top, from_ = 255, to = 0, orient = tkinter.VERTICAL, command = pot_v_cb, length = 200); pot_v.set(0)
l_b = tkinter.Label(top, text = "B"); pot_b = tkinter.Scale(top, from_ = 255, to = 0, orient = tkinter.VERTICAL, command = pot_b_cb, length = 200); pot_b.set(0)

pot_pwm = tkinter.Scale(top, from_ = 255, to = 0 , orient = tkinter.VERTICAL, command = pot_pwm_cb, length = 200); pot_pwm.set(0)   #uu: passer a 255->100%
pot_srv = tkinter.Scale(top, from_ = 250, to = 50, orient = tkinter.VERTICAL, command = pot_srv_cb, length = 200); pot_srv.set(0)   #uu: passer a 250->2500us

my_canvas_in  = tkinter.Canvas(top, bg = "blue"   , height = 16 * 20, width = 16 * 30)
my_canvas_led = tkinter.Canvas(top, bg = "blue"   , height = 16 * 20, width = 16 * 30)
my_canvas_out = tkinter.Canvas(top, bg = "blue"   , height =  4 * 20, width = 16 * 30)
my_canvas_aig = tkinter.Canvas(top, bg = "blue"   , height =  6 * 20, width = 16 * 30)
my_canvas_pwm = tkinter.Canvas(top, bg = "#888888", height =  7 * 30, width =  8 * 30)
my_canvas_srv = tkinter.Canvas(top, bg = "#888888", height =  6 * 30, width =  8 * 30)
my_canvas_neo = tkinter.Canvas(top, bg = "blue"   , height =  6 * 30, width = 10 * 30)
my_canvas_fx  = tkinter.Canvas(top, bg = "blue"   , height =  4 * 20, width = 16 * 30)

screen_esin_on  = 0
screen_esout_on = 0
screen_esled_on = 0
screen_esaig_on = 0
screen_espwm_on = 0
screen_essrv_on = 0
screen_esneo_on = 0
screen_esan_on  = 0
screen_esfx_on  = 0
debug_force_redraw = 1

def esin_on():
    global debug_force_redraw
    l_in.grid(row = 0, column = 0, sticky = tkinter.W)
    c_in_mem.grid(row = 0, column = 1, sticky = tkinter.W)
    b_in_clr.grid(row = 0, column = 2, sticky = tkinter.W)
    my_canvas_in.grid(row = 1, column = 0, columnspan = 3, rowspan = 1, sticky = tkinter.N)
    debug_force_redraw = 1
def esin_off():
    l_in.grid_forget()
    c_in_mem.grid_forget()
    b_in_clr.grid_forget()
    my_canvas_in.grid_forget()
   
def esout_on():
    global debug_force_redraw
    l_out.grid(row = 0, column = 0, sticky = tkinter.W)
    my_canvas_out.grid(row = 1, column = 0, sticky = tkinter.N)
    debug_force_redraw = 1
def esout_off():
    l_out.grid_forget()
    my_canvas_out.grid_forget()
    
def esled_on():        
    global debug_force_redraw
    l_led.grid(row = 0, column = 0, sticky = tkinter.W)
    my_canvas_led.grid(row = 1, column = 0)
    debug_force_redraw = 1
def esled_off():        
    l_led.grid_forget()
    my_canvas_led.grid_forget()

def esfx_on():
    global debug_force_redraw
    my_canvas_fx.grid(row = 0, column = 0, sticky = tkinter.N)
    debug_force_redraw = 1
def esfx_off():
    my_canvas_fx.grid_forget()
    
def esaig_on():
    global debug_force_redraw
    l_aig.grid(row = 0, column = 0, sticky = tkinter.W)
    my_canvas_aig.grid(row = 1,column = 0)
    debug_force_redraw = 1
def esaig_off():
    l_aig.grid_forget()
    my_canvas_aig.grid_forget()

def espwm_on():
    global debug_force_redraw
    l_pwm.grid(row = 0, column = 0, sticky = tkinter.W)
    my_canvas_pwm.grid(row = 1,column = 0)
    pot_pwm.grid(row = 1, column = 1)
    debug_force_redraw = 1
def espwm_off():
    l_pwm.grid_forget()
    my_canvas_pwm.grid_forget()    
    pot_pwm.grid_forget()

def essrv_on():
    global debug_force_redraw
    l_srv.grid(row = 0, column = 0, sticky = tkinter.W)
    my_canvas_srv.grid(row = 1, column = 0)
    pot_srv.grid(row = 1, column = 1)
    debug_force_redraw = 1
def essrv_off():
    l_srv.grid_forget()
    my_canvas_srv.grid_forget()    
    pot_srv.grid_forget()

def esneo_on():
    global debug_force_redraw
    l_neo.grid(row = 0, column = 0, sticky = tkinter.W)
    my_canvas_neo.grid(row = 1, column = 0)
    #l_r.grid(row = 0, column = 1); pot_r.grid(row = 1, column = 1)
    #l_v.grid(row = 0, column = 2); pot_v.grid(row = 1, column = 2)
    #l_b.grid(row = 0, column = 3); pot_b.grid(row = 1, column = 3)
    debug_force_redraw = 1
def esneo_off():
    l_neo.grid_forget()
    my_canvas_neo.grid_forget()    
    #l_r.grid_forget(); pot_r.grid_forget()    
    #l_v.grid_forget(); pot_v.grid_forget()    
    #l_b.grid_forget(); pot_b.grid_forget()    

def esan_on():
    global debug_force_redraw
    l_an0.grid(row = 0, column = 0, sticky = tkinter.W)
    l_an1.grid(row = 1, column = 0, sticky = tkinter.W)
    debug_force_redraw = 1
def esan_off():
    l_an0.grid_forget()
    l_an1.grid_forget()

    
#---------------------------------------------------------------------------------------

led_size_x = 30
led_size_y = 20
led_nb_x   = 16
led_nb_y   = 16

led_rect_widget = [ 0 for i in range(led_nb)]
led_line1_widget = [ 0 for i in range(led_nb)]
led_line2_widget = [ 0 for i in range(led_nb)]
led_draw_optim_last_v = [ -1 for i in range(led_nb)]
led_draw_optim_last_lock = [ -1 for i in range(led_nb)]
for j in range(0, led_nb_y):
    for i in range(0, led_nb_x):
        led_rect_widget[led_nb_x * j + i] = my_canvas_led.create_rectangle(led_size_x * i + 1, led_size_y * j + 1, led_size_x * i + led_size_x - 1 + 1, led_size_y * j + led_size_y - 1 + 1, fill = 'red', outline = 'black')
        my_canvas_led.create_text(led_size_x * i + int(led_size_x / 2), led_size_y * j + int(led_size_y / 2), text = str(led_nb_x * j + i))
        led_line1_widget[led_nb_x * j + i] = my_canvas_led.create_line(led_size_x * i + 1, led_size_y * j  + 1, led_size_x * (i + 1), led_size_y * (j + 1), fill = "black")
        led_line2_widget[led_nb_x * j + i] = my_canvas_led.create_line(led_size_x * i + 1, led_size_y * (j + 1), led_size_x * (i + 1), led_size_y * j + 1, fill = "black")

def debug_led_redraw(num, redraw):
    global es
    global led_draw_optim_last_v
    global led_draw_optim_last_lock
    
    if(es[esb].led_lock[num]):
        if(var[esb].led[num] == 0):
            v = 0
        elif(var[esb].led_cli[num] == 0):
            v = 1
        elif(var[esb].led_pha[num] == 0):
            v = 2
        else:
            v = 3
    else:
        v = es[esb].led_on[num]
        
    if(v != led_draw_optim_last_v[num] or redraw):
        led_draw_optim_last_v[num] = v
        if(  v == 0):
            my_canvas_led.itemconfigure(led_rect_widget[num], fill = 'red')
        elif(v == 1):
            my_canvas_led.itemconfigure(led_rect_widget[num], fill = '#00FF00')  #or green
        elif(v == 2):
            my_canvas_led.itemconfigure(led_rect_widget[num], fill = 'yellow')
        elif(v == 3):
            my_canvas_led.itemconfigure(led_rect_widget[num], fill = 'blue')

    if(es[esb].led_lock[num] != led_draw_optim_last_lock[num] or redraw):
        led_draw_optim_last_lock[num] = es[esb].led_lock[num]
        if(es[esb].led_lock[num] == 0):
            my_canvas_led.itemconfigure(led_line1_widget[num], state = 'hidden')
            my_canvas_led.itemconfigure(led_line2_widget[num], state = 'hidden')
        else:
            my_canvas_led.itemconfigure(led_line1_widget[num], state = 'normal')
            my_canvas_led.itemconfigure(led_line2_widget[num], state = 'normal')

def debug_led_redraw_all():
    global debug_force_redraw
    
    redraw = debug_force_redraw
    for i in range(rs232_led_nb):
        debug_led_redraw(i, redraw)
    if(redraw):
        debug_force_redraw = 0
        
def callback_canvas_led_mouse(event):
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)

    i = int(my_x / led_size_x);
    if(i >= led_nb_x):
        return
    j = int(my_y / led_size_y);
    if(j >= led_nb_y):
        return

    num = led_nb_x * j + i
    es[esb].led_on[num] = es[esb].led_on[num] + 1
    if(es[esb].led_on[num] == 4):
        es[esb].led_on[num] = 0

    debug_led_redraw(num, 0)  #uu: voir si vraiment necessaire (idem en 3)
    

def callback_canvas_led_mouse3(event):
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)

    i = int(my_x / led_size_x);
    if(i >= led_nb_x):
        return
    j = int(my_y / led_size_y);
    if(j >= led_nb_y):
        return
        
    num = led_nb_x * j + i
    
    es[esb].led_lock[num] ^= 1

    debug_led_redraw(num, 0)

    
my_canvas_led.bind("<Button-1>", callback_canvas_led_mouse)
my_canvas_led.bind("<Button-3>", callback_canvas_led_mouse3)


#---------------------------------------------------------------------------------------

out_size_x = 30
out_size_y = 20
out_nb_x   = 16
out_nb_y   = 4

out_rect_widget = [ 0 for i in range(out_nb)]
out_line1_widget = [ 0 for i in range(out_nb)]
out_line2_widget = [ 0 for i in range(out_nb)]
out_draw_optim_last_v = [ -1 for i in range(out_nb)]
out_draw_optim_last_lock = [ -1 for i in range(out_nb)]
for j in range(0, out_nb_y):
    for i in range(0, out_nb_x):
        out_rect_widget[out_nb_x * j + i] = my_canvas_out.create_rectangle(out_size_x * i + 1, out_size_y * j  + 1, out_size_x * i + out_size_x - 1 + 1, out_size_y * j + out_size_y - 1 + 1, fill = 'red', outline = 'black')
        my_canvas_out.create_text(out_size_x * i + int(out_size_x / 2),out_size_y * j + int(out_size_y / 2), text = str(out_nb_x * j + i))
        out_line1_widget[out_nb_x * j + i] = my_canvas_out.create_line(out_size_x * i + 1, out_size_y *  j + 1, out_size_x * (i + 1), out_size_y * (j + 1), fill = "black")
        out_line2_widget[out_nb_x * j + i] = my_canvas_out.create_line(out_size_x * i + 1, out_size_y * (j + 1), out_size_x * (i + 1), out_size_y * j + 1, fill = "black")

def debug_out_redraw(num, redraw):
    global out_draw_optim_last_v
    global out_draw_optim_last_lock
    
    if(es[esb].out_lock[num]):
        v = var[esb].out[num]
    else:
        v = es[esb].out_on[num]
        
    if(v != out_draw_optim_last_v[num] or redraw):
        out_draw_optim_last_v[num] = v
        if(v == 0):
            my_canvas_out.itemconfigure(out_rect_widget[num],fill = 'red')
        else:
            my_canvas_out.itemconfigure(out_rect_widget[num],fill = '#00FF00')  #or green

    if(es[esb].out_lock[num] != out_draw_optim_last_lock[num] or redraw):
        out_draw_optim_last_lock[num] = es[esb].out_lock[num]
        if(es[esb].out_lock[num] == 0):
            my_canvas_out.itemconfigure(out_line1_widget[num], state = 'hidden')
            my_canvas_out.itemconfigure(out_line2_widget[num], state = 'hidden')
        else:
            my_canvas_out.itemconfigure(out_line1_widget[num], state = 'normal')
            my_canvas_out.itemconfigure(out_line2_widget[num], state = 'normal')

def debug_out_redraw_all():
    global debug_force_redraw  
    
    redraw = debug_force_redraw
    for i in range(rs232_out_nb):
        debug_out_redraw(i, redraw)
    if(redraw):
        debug_force_redraw = 0
        
        
def callback_canvas_out_mouse(event):
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)

    i = int(my_x / out_size_x);
    if(i >= out_nb_x):
        return
    j = int(my_y / out_size_y);
    if(j >= out_nb_y):
        return
    
    num = out_nb_x * j + i
    es[esb].out_on[num] ^= 1
        
    debug_out_redraw(num, 0)

def callback_canvas_out_mouse3(event):
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)

    i = int(my_x / out_size_x);
    if(i >= out_nb_x):
        return
    j = int(my_y / out_size_y);
    if(j >= out_nb_y):
        return
    
    num = out_nb_x * j + i
    es[esb].out_lock[num] ^= 1

    debug_out_redraw(num, 0)

   
my_canvas_out.bind("<Button-1>", callback_canvas_out_mouse)
my_canvas_out.bind("<Button-3>", callback_canvas_out_mouse3)


#---------------------------------------------------------------------------------------

fx_size_x = 30
fx_size_y = 20
fx_nb_x   = 16
fx_nb_y   = 4

fx_rect_widget = [ 0 for i in range(fx_nb)]
fx_line1_widget = [ 0 for i in range(fx_nb)]
fx_line2_widget = [ 0 for i in range(fx_nb)]
fx_draw_optim_last_v = [ -1 for i in range(fx_nb)]
fx_draw_optim_last_lock = [ -1 for i in range(fx_nb)]
for j in range(0, fx_nb_y):
    for i in range(0, fx_nb_x):
        fx_rect_widget[fx_nb_x * j + i] = my_canvas_fx.create_rectangle(fx_size_x * i + 1, fx_size_y * j + 1, fx_size_x * i + fx_size_x - 1 + 1, fx_size_y * j + fx_size_y - 1 + 1, fill = 'red', outline = 'black')
        my_canvas_fx.create_text(fx_size_x * i + int(fx_size_x / 2), fx_size_y * j + int(fx_size_y / 2), text = str(fx_nb_x * j + i))
        fx_line1_widget[fx_nb_x * j + i] = my_canvas_fx.create_line(fx_size_x * i + 1, fx_size_y * j + 1, fx_size_x * (i + 1), fx_size_y * (j + 1), fill = "black")
        fx_line2_widget[fx_nb_x * j + i] = my_canvas_fx.create_line(fx_size_x * i + 1, fx_size_y * (j + 1), fx_size_x * (i + 1),fx_size_y * j + 1, fill = "black")

def debug_fx_redraw(num, redraw):
    global fx_draw_optim_last_v
    global fx_draw_optim_last_lock
    
    if(es[esb].fx_lock[num]):
        v = var[esb].fx[num]
    else:
        v = es[esb].fx_on[num]
        
    if(v != fx_draw_optim_last_v[num] or redraw):
        fx_draw_optim_last_v[num] = v
        if(v == 0):
            my_canvas_fx.itemconfigure(fx_rect_widget[num], fill = 'red')
        else:
            my_canvas_fx.itemconfigure(fx_rect_widget[num], fill = '#00FF00')  #or green

    if(es[esb].fx_lock[num] != fx_draw_optim_last_lock[num] or redraw):
        fx_draw_optim_last_lock[num] = es[esb].fx_lock[num]
        if(es[esb].fx_lock[num] == 0):
            my_canvas_fx.itemconfigure(fx_line1_widget[num], state = 'hidden')
            my_canvas_fx.itemconfigure(fx_line2_widget[num], state = 'hidden')
        else:
            my_canvas_fx.itemconfigure(fx_line1_widget[num], state = 'normal')
            my_canvas_fx.itemconfigure(fx_line2_widget[num], state = 'normal')

def debug_fx_redraw_all():
    global debug_force_redraw  
    redraw = debug_force_redraw
    for i in range(rs232_fx_nb):
        debug_fx_redraw(i, redraw)
    if(redraw):
        debug_force_redraw = 0
        
def callback_canvas_fx_mouse(event):
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)

    i = int(my_x / fx_size_x);
    if(i >= fx_nb_x):
        return
    j = int(my_y / fx_size_y);
    if(j >= fx_nb_y):
        return
    
    num = fx_nb_x * j + i
    es[esb].fx_on[num] ^= 1
        
    debug_fx_redraw(num, 0)

def callback_canvas_fx_mouse3(event):
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)
    
    i = int(my_x / fx_size_x);
    if(i >= fx_nb_x):
        return
    j = int(my_y / fx_size_y);
    if(j >= fx_nb_y):
        return
    
    num = fx_nb_x * j + i
    es[esb].fx_lock[num] ^= 1

    debug_fx_redraw(num, 0)

   
my_canvas_fx.bind("<Button-1>", callback_canvas_fx_mouse)
my_canvas_fx.bind("<Button-3>", callback_canvas_fx_mouse3)


#---------------------------------------------------------------------------------------

aig_size_x = 30
aig_size_y = 20
aig_nb_x   = 16
aig_nb_y   = 6

aig_rect_widget          = [ 0 for i in range(aig_nb)]
aig_rect_cmd_widget      = [ 0 for i in range(aig_nb)]
aig_line1_widget         = [ 0 for i in range(aig_nb)]
aig_line2_widget         = [ 0 for i in range(aig_nb)]
aig_draw_optim_last_v    = [ -1 for i in range(aig_nb)]
aig_draw_optim_last_v2   = [ -1 for i in range(aig_nb)]
aig_draw_optim_last_lock = [ -1 for i in range(aig_nb)]

for j in range(0, aig_nb_y):
    for i in range(0, aig_nb_x):
        aig_num = j * aig_nb_x + i
        aig_rect_widget[aig_num] = my_canvas_aig.create_rectangle(aig_size_x * i + 1, aig_size_y * j + 1, aig_size_x * i + aig_size_x - 1 + 1, aig_size_y * j + aig_size_y - 1 + 1, fill = 'red', outline = 'black')
        aig_rect_cmd_widget[aig_num] = my_canvas_aig.create_rectangle(aig_size_x * i + 1, aig_size_y * j + 1, aig_size_x * i + aig_size_x / 3, aig_size_y * j + aig_size_y / 3, fill = 'red', outline = 'black')
        my_canvas_aig.create_text(aig_size_x * i + int(aig_size_x / 2), aig_size_y * j + int(aig_size_y / 2), text = str(aig_nb_x * j + i))
        aig_line1_widget[aig_num] = my_canvas_aig.create_line(aig_size_x * i + 1, aig_size_y * j + 1, aig_size_x * (i + 1), aig_size_y * (j + 1), fill = "black")
        aig_line2_widget[aig_num] = my_canvas_aig.create_line(aig_size_x * i + 1, aig_size_y * (j + 1), aig_size_x * (i + 1), aig_size_y * j + 1, fill = "black")

def debug_aig_redraw(num, redraw):
    global aig_draw_optim_last_v
    global aig_draw_optim_last_v2
    global aig_draw_optim_last_lock
    
    #print("aig_redraw_"+str(num))
    if(es[esb].aig_lock[num]):
        v = var[esb].aig_cmd[num]
    else:
        v = es[esb].aig_on[num]
        
    if(v != aig_draw_optim_last_v[num] or redraw):
        aig_draw_optim_last_v[num] = v
        if(v == 0):
            my_canvas_aig.itemconfigure(aig_rect_cmd_widget[num],fill = 'red')
        else:
            my_canvas_aig.itemconfigure(aig_rect_cmd_widget[num],fill = '#00FF00')  #or green

    if(rs232[esb].aig_dir[num]):
        v = 0
    elif(rs232[esb].aig_dev[num]):
        v = 1
    else:
        v = 2
        
    if(v != aig_draw_optim_last_v2[num] or redraw):
        aig_draw_optim_last_v2[num] = v
        if(v == 0):
            my_canvas_aig.itemconfigure(aig_rect_widget[num],fill = 'red')
        elif(v == 1):
            my_canvas_aig.itemconfigure(aig_rect_widget[num],fill = '#00FF00')
        else:
            my_canvas_aig.itemconfigure(aig_rect_widget[num],fill = '#cccccc')

    if(es[esb].aig_lock[num] != aig_draw_optim_last_lock[num] or redraw):
        aig_draw_optim_last_lock[num] = es[esb].aig_lock[num]
        if(es[esb].aig_lock[num] == 0):
            my_canvas_aig.itemconfigure(aig_line1_widget[num],state = 'hidden')
            my_canvas_aig.itemconfigure(aig_line2_widget[num],state = 'hidden')
        else:
            my_canvas_aig.itemconfigure(aig_line1_widget[num],state = 'normal')
            my_canvas_aig.itemconfigure(aig_line2_widget[num],state = 'normal')

    
def debug_aig_redraw_all():
    global debug_force_redraw  
    redraw = debug_force_redraw
    for i in range(rs232_aig_nb):
        debug_aig_redraw(i, redraw)
    if(redraw):
        debug_force_redraw = 0

        
def callback_canvas_aig_mouse(event):
    global es
    global aig_rect_widget
    
    my_x = int(event.x)
    my_y = int(event.y)
    
    i = int(my_x / aig_size_x);
    if(i >= aig_nb_x):
        return
    j = int(my_y / aig_size_y);
    if(j >= aig_nb_y):
        return
    num = aig_nb_x * j + i
    
    es[esb].aig_on[num] ^= 1

    debug_aig_redraw(num, 0)


def callback_canvas_aig_mouse3(event):
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)

    i = int(my_x / aig_size_x);
    if(i >= aig_nb_x):
        return
    j = int(my_y / aig_size_y);
    if(j >= aig_nb_y):
        return
    
    num = aig_nb_x * j + i
    es[esb].aig_lock[num] ^= 1

    debug_aig_redraw(num, 0)


my_canvas_aig.bind("<Button-1>", callback_canvas_aig_mouse)
my_canvas_aig.bind("<Button-3>", callback_canvas_aig_mouse3)


#---------------------------------------------------------------------------------------

pwm_rect_widget = [ 0 for i in range(pwm_nb)]
pwm_line1_widget = [ 0 for i in range(pwm_nb)]
pwm_line2_widget = [ 0 for i in range(pwm_nb)]
pwm_draw_optim_last_val = [ -1 for i in range(pwm_nb)]
pwm_draw_optim_last_lock = [ -1 for i in range(pwm_nb)]
pwm_txt = [ 0 for i in range(pwm_nb)]

for n in range(0, pwm_nb):
    j = n // 8
    i = n & 7
    pwm_rect_widget[n] = my_canvas_pwm.create_rectangle(30 * i + 1, 30 * j + 1, 30 * i + 30 + 1, 30 * j + 30 + 1, fill = 'red', outline = 'black')
    my_canvas_pwm.create_text(30 * i + 15, 30 * j + 15 - 8, text = str(n), fill = "black")
    pwm_txt[n] = my_canvas_pwm.create_text(30 * i + 15, 30 * j + 15 + 8, text = str(0), fill = "black")
    pwm_line1_widget[n] = my_canvas_pwm.create_line(30 * i + 1, 30 * j      + 1, 30 * i + 30 + 1, 30 * j + 30 + 1, fill = "black")
    pwm_line2_widget[n] = my_canvas_pwm.create_line(30 * i + 1, 30 * j + 30 + 1, 30 * i + 30 + 1, 30 * j      + 1, fill = "black")
    
def debug_pwm_redraw(num, redraw):
    global pwm_draw_optim_last_val
    global pwm_draw_optim_last_lock
    global pot_pwm
    
    if(es[esb].pwm_lock[num]):
        val = var[esb].pwm[num];
    else:
        val = es[esb].pwm[num];
        
    if(num == sel_pwm):
        if(es[esb].pwm_lock[num]):     
            pot_pwm.set(val)
        
    #if(val!=pwm_draw_optim_last_val[num]):
    #    pwm_draw_optim_last_val[num] = val
    #    r = val  #0-255
    #    tk_rgb = "#%02x%02x%02x" % (r, 0, 0)
    #    my_canvas_pwm.itemconfigure(pwm_rect_widget[num] , fill = tk_rgb)
    #uu: mettre la valeur
    
    my_canvas_pwm.itemconfigure(pwm_txt[num], text = str(val))

    if(es[esb].pwm_lock[num] != pwm_draw_optim_last_lock[num] or redraw):
        pwm_draw_optim_last_lock[num] = es[esb].pwm_lock[num]
        if(es[esb].pwm_lock[num] == 0):
            my_canvas_pwm.itemconfigure(pwm_line1_widget[num],state = 'hidden')
            my_canvas_pwm.itemconfigure(pwm_line2_widget[num],state = 'hidden')
        else:
            my_canvas_pwm.itemconfigure(pwm_line1_widget[num],state = 'normal')
            my_canvas_pwm.itemconfigure(pwm_line2_widget[num],state = 'normal')

def debug_pwm_redraw_all():
    global debug_force_redraw  
    redraw = debug_force_redraw
    for i in range(pwm_nb):
        debug_pwm_redraw(i, redraw)
    if(redraw):
        debug_force_redraw = 0

def get_num_pwm_mouse(x, y):
    i = int(x / 30);
    if(i >= 8):
        return -1
    j = int(y / 30);
    num = 8 * j + i
    if(num > pwm_nb):
       return -1
    return num
        
def callback_canvas_pwm_mouse(event):
    global sel_pwm; global pot_pwm;
    my_x = int(event.x)
    my_y = int(event.y)
    
    if(sel_pwm != -1):
        my_canvas_pwm.itemconfigure(pwm_rect_widget[sel_pwm], fill = "#FF0000")
    sel_pwm = -1        
    
    sel_pwm = get_num_pwm_mouse(my_x, my_y)
    if(sel_pwm != -1):    
        my_canvas_pwm.itemconfigure(pwm_rect_widget[sel_pwm], fill = "#00FF00")
        pot_pwm.set(es[esb].pwm[sel_pwm]) 

def callback_canvas_pwm_mouse3(event):
    global sel_pwm; global debug_pwm_lock; global pot_pwm
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)
    
    if(sel_pwm != -1):
        my_canvas_pwm.itemconfigure(pwm_rect_widget[sel_pwm], fill = "#FF0000")
    sel_pwm = -1        
    
    sel_pwm = get_num_pwm_mouse(my_x, my_y)
    if(sel_pwm != -1):    
        my_canvas_pwm.itemconfigure(pwm_rect_widget[sel_pwm], fill = "#00FF00")    
        es[esb].pwm_lock[sel_pwm] ^= 1
        if(es[esb].pwm_lock[sel_pwm]):
            pot_pwm.set(0)
        else:
            es[esb].pwm[sel_pwm] = var[esb].pwm[sel_pwm];
        pot_pwm.set(es[esb].pwm[sel_pwm])
        debug_pwm_redraw(sel_pwm, 0)
        
my_canvas_pwm.bind("<Button-1>", callback_canvas_pwm_mouse)
my_canvas_pwm.bind("<Button-3>", callback_canvas_pwm_mouse3)    

        
#---------------------------------------------------------------------------------------

srv_rect_widget = [ 0 for i in range(srv_nb)]
srv_line1_widget = [ 0 for i in range(srv_nb)]
srv_line2_widget = [ 0 for i in range(srv_nb)]
srv_draw_optim_last_val = [ -1 for i in range(srv_nb)]
srv_draw_optim_last_lock = [ -1 for i in range(srv_nb)]
srv_txt = [ 0 for i in range(srv_nb)]

for n in range(0, srv_nb):
    j = n // 8
    i = n & 7
    srv_rect_widget[n] = my_canvas_srv.create_rectangle(30 * i + 1, 30 * j + 1, 30 * i + 30 + 1, 30 * j + 30 + 1, fill = 'red', outline = 'black')
    my_canvas_srv.create_text(30 * i + 15, 30 * j + 15 - 8, text = str(n), fill = "black")
    srv_txt[n] = my_canvas_srv.create_text(30 * i + 15, 30 * j + 15 + 8, text = str(0), fill = "black")
    srv_line1_widget[n] = my_canvas_srv.create_line(30 * i + 1, 30 * j      + 1, 30 * i + 30 + 1, 30 * j + 30 + 1, fill = "black")
    srv_line2_widget[n] = my_canvas_srv.create_line(30 * i + 1, 30 * j + 30 + 1, 30 * i + 30 + 1, 30 * j       +1, fill = "black")
    
def debug_srv_redraw(num, redraw):
    global srv_draw_optim_last_val
    global srv_draw_optim_last_lock
    global pot_srv
    
    if(es[esb].srv_lock[num]):
        val = var[esb].srv[num];
    else:
        val = es[esb].srv[num];
        
    if(num == sel_srv):
        if(es[esb].srv_lock[num]):     
            pot_srv.set(val)
            
    my_canvas_srv.itemconfigure(srv_txt[num], text = str(val))

    if(es[esb].srv_lock[num] != srv_draw_optim_last_lock[num] or redraw):
        srv_draw_optim_last_lock[num] = es[esb].srv_lock[num]
        if(es[esb].srv_lock[num] == 0):
            my_canvas_srv.itemconfigure(srv_line1_widget[num], state = 'hidden')
            my_canvas_srv.itemconfigure(srv_line2_widget[num], state = 'hidden')
        else:
            my_canvas_srv.itemconfigure(srv_line1_widget[num], state = 'normal')
            my_canvas_srv.itemconfigure(srv_line2_widget[num], state = 'normal')

def debug_srv_redraw_all():
    global debug_force_redraw  
    redraw = debug_force_redraw
    for i in range(srv_nb):
        debug_srv_redraw(i, redraw)
    if(redraw):
        debug_force_redraw = 0

def get_num_srv_mouse(x, y):
    i = int(x / 30);
    if(i >= 8):
        return -1
    j = int(y / 30);
    num = 8 * j + i
    if(num > srv_nb):
       return -1
    return num
        
def callback_canvas_srv_mouse(event):
    global sel_srv; global pot_srv;
    
    my_x = int(event.x)
    my_y = int(event.y)
    
    if(sel_srv != -1):
        my_canvas_srv.itemconfigure(srv_rect_widget[sel_srv], fill = "#FF0000")
    sel_srv = -1        
    
    sel_srv = get_num_srv_mouse(my_x, my_y)
    if(sel_srv != -1):    
        my_canvas_srv.itemconfigure(srv_rect_widget[sel_srv],fill = "#00FF00")
        pot_srv.set(es[esb].srv[sel_srv]) 

def callback_canvas_srv_mouse3(event):
    global sel_srv; global debug_srv_lock; global pot_srv; global debug_srv
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)
    
    if(sel_srv != -1):
        my_canvas_srv.itemconfigure(srv_rect_widget[sel_srv],fill = "#FF0000")
    sel_srv = -1        
    
    sel_srv = get_num_srv_mouse(my_x, my_y)
    if(sel_srv != -1):    
        my_canvas_srv.itemconfigure(srv_rect_widget[sel_srv],fill = "#00FF00")    
        es[esb].srv_lock[sel_srv] ^= 1
        if(des[esb].srv_lock[sel_srv]):
            pot_srv.set(0)
        else:
            es[esb].srv[sel_srv] = var[esb].srv[sel_srv];
        pot_srv.set(es[esb].srv[sel_srv])
        debug_srv_redraw(sel_srv, 0)
        
my_canvas_srv.bind("<Button-1>", callback_canvas_srv_mouse)
my_canvas_srv.bind("<Button-3>", callback_canvas_srv_mouse3)    


#---------------------------------------------------------------------------------------

neo_rect_widget = [ 0 for i in range(neo_nb)]
neo_line1_widget = [ 0 for i in range(neo_nb)]
neo_line2_widget = [ 0 for i in range(neo_nb)]
neo_draw_optim_last_rvb = [ -1 for i in range(neo_nb)]
neo_draw_optim_last_lock = [ -1 for i in range(neo_nb)]

for j in range(0, 6):
    for i in range(0, 10):
        neo_rect_widget[10 * j + i] = my_canvas_neo.create_rectangle(30 * i + 1, 30 * j + 1, 30 * i + 30 + 1, 30 * j + 30 + 1, fill = 'black', outline = 'white')
        my_canvas_neo.create_text(30 * i + 15, 30 * j + 15 - 8, text = str(10 * j + i), fill = "black")
        my_canvas_neo.create_text(30 * i + 15, 30 * j + 15 + 8, text = str(10 * j + i), fill = "white")
        neo_line1_widget[10 * j + i] = my_canvas_neo.create_line(30 * i + 1, 30 * j      + 1, 30 * i + 30 + 1, 30 * j + 30 + 1, fill = "black")
        neo_line2_widget[10 * j + i] = my_canvas_neo.create_line(30 * i + 1, 30 * j + 30 + 1, 30 * i + 30 + 1, 30 * j       +1, fill = "white")
            
def debug_neo_redraw(num, redraw):
    global es
    global neo_draw_optim_last_rvb
    global neo_draw_optim_last_lock
    
    if(es[esb].neo_lock[num]):
        rvb = var[esb].neo_rvb[num];
    else:
        rvb = es[esb].neo_rvb[num];
        
    if(rvb != neo_draw_optim_last_rvb[num] or redraw):
        neo_draw_optim_last_rvb[num] = rvb
        r = rvb >> 16
        v = (rvb >> 8) & 0xff
        b = rvb & 0xff
        tk_rgb = "#%02x%02x%02x" % (r, v, b)
        my_canvas_neo.itemconfigure(neo_rect_widget[num], fill = tk_rgb)

    if(es[esb].neo_lock[num] != neo_draw_optim_last_lock[num] or redraw):
        neo_draw_optim_last_lock[num] = es[esb].neo_lock[num]
        if(es[esb].neo_lock[num] == 0):
            my_canvas_neo.itemconfigure(neo_line1_widget[num], state = 'hidden')
            my_canvas_neo.itemconfigure(neo_line2_widget[num], state = 'hidden')
        else:
            my_canvas_neo.itemconfigure(neo_line1_widget[num], state = 'normal')
            my_canvas_neo.itemconfigure(neo_line2_widget[num], state = 'normal')

def debug_neo_redraw_all():
    global debug_force_redraw  
    redraw = debug_force_redraw
    for i in range(neo_nb):
        debug_neo_redraw(i, redraw)
    if(redraw):
        debug_force_redraw = 0
        
def callback_canvas_neo_mouse(event):
    global debug_neo_r; global debug_neo_v; global debug_neo_b  #pot pas encore utilise
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)
    num = -1

    ##print("clicked at canvas neo " + str(event.x) + " " + str (event.y))
    i = int(my_x / 30);
    if(i >= 10):
        return
    j = int(my_y / 30);
    if(j >= 6):
        return    
    num = 10 * j + i
    
    color = askcolor()
    es[esb].neo_rvb[num] = 256 * 256 * int(color[0][0]) + 256 * int(color[0][1]) + int(color[0][2])
        
    debug_neo_redraw(num, 0)

def callback_canvas_neo_mouse3(event):
    global es
    
    my_x = int(event.x)
    my_y = int(event.y)
    num = -1
    
    ##print("clicked3 at canvas neo " + str(event.x) + " " + str (event.y))
    i = int(my_x / 30);
    if(i >= 10):
        return
    j = int(my_y / 30);
    if(j >= 6):
        return
    num = 10 * j + i
    
    es[esb].neo_lock[num] ^= 1

    debug_neo_redraw(num, 0)

   
my_canvas_neo.bind("<Button-1>", callback_canvas_neo_mouse)
my_canvas_neo.bind("<Button-3>", callback_canvas_neo_mouse3)    

#---------------------------------------------------------------------------------------

in_size_x = 30
in_size_y = 20
in_nb_x   = 16
in_nb_y   = 16

in_rect_widget = [ 0 for i in range(in_nb)]
in_draw_optim_last_v = [ -1 for i in range(in_nb)]
for j in range(0, in_nb_y):
    for i in range(0, in_nb_x):
        in_rect_widget[in_nb_x * j + i] = my_canvas_in.create_rectangle(in_size_x * i + 1, in_size_y * j + 1, in_size_x * i + in_size_x - 1 + 1, in_size_y * j + in_size_y - 1 + 1, fill = 'red', outline = 'black')
        my_canvas_in.create_text(in_size_x * i + int(in_size_x / 2), in_size_y * j + int(in_size_y / 2), text = str(in_nb_x * j + i))

def debug_in_redraw_all():
    global in_draw_optim_last_v
    global debug_force_redraw
    
    redraw = debug_force_redraw
    
    for j in range(0, in_nb_y):
        for i in range(0, in_nb_x):
            num = in_nb_x*j+i

            if((es[esb].in_on[num] != in_draw_optim_last_v[num]) or redraw):
                in_draw_optim_last_v[num] = es[esb].in_on[num]
                if(es[esb].in_on[num] == 0):
                    my_canvas_in.itemconfigure(in_rect_widget[num], fill = 'red')
                else:
                    my_canvas_in.itemconfigure(in_rect_widget[num], fill = '#00FF00')
                    
    if(redraw):
        debug_force_redraw = 0

#---------------------------------------------------------------------------------------

def debug_an_redraw_all():
    val = rs232[esb].an[0]; mv = val * 5000 // 255
    l_an0.config(text = "AN0 = " + str(val) + "/255 = " + str(mv) + "mV")
    val = rs232[esb].an[1]; mv = val * 5000 // 255
    l_an1.config(text = "AN1 = " + str(val) + "/255 = " + str(mv) + "mV")

                        
#---------------------------------------------------------------------------------------
# fdcc IO
#---------------------------------------------------------------------------------------

var8_nb = 1000
var_nb  = 1000
res_nb  = 128
zone_nb = 200
cf_nb   = 200

class c_var():
    def __init__(self):
        self.out     = [  0 for i in range(rs232_out_nb)]   #initialise avec la valeur specifiee au debut, ici 0
        self.led     = [  0 for i in range(rs232_led_nb)]
        self.led_cli = [  0 for i in range(rs232_led_nb)]
        self.led_pha = [  0 for i in range(rs232_led_nb)]

        self.aig_cmd = [  0 for i in range(rs232_aig_nb)]

        self.neo_rvb = [  0 for i in range(rs232_neo_nb)]
        self.pwm     = [  0 for i in range(rs232_pwm_nb)]
        self.srv     = [150 for i in range(rs232_srv_nb)]
     
        self.fx      = [ 0 for i in range(rs232_fx_nb)] 
        self.res     = [ 0 for i in range(res_nb)     ]

var = []
for c in range(MAX_BOARD_NB):
    var.append(c_var())
    
class c_var2():   # ces variables ne dependent pas de la board
    def __init__(self):
        self.var8    = [ 0 for i in range(var8_nb)]  #variables entieres 0-2000000000
        self.var     = [ 0 for i in range(var_nb)]   #variables binaires 0/1
        self.zone    = [ 0 for i in range(zone_nb)]
        self.iti_cmd = [ 0 for i in range(1000)]
        self.cf      = [ 0 for i in range(cf_nb)]    

var2 = c_var2() 
        

def fdcc_and_debug_to_rs232():
    global rs232

    for c in range(MAX_BOARD_NB):
    
        # out
        for i in range(rs232_out_nb):
             if(es[c].out_lock[i] == 1):   # verrouille -> valeur normale
                 rs232[c].out[i] = var[c].out[i]
             else:                       # non verouille -> valeur de debug
                 rs232[c].out[i] = es[c].out_on[i]

        # fx
        for i in range(rs232_fx_nb):
            if(es[c].fx_lock[i] == 1):
                rs232[c].fx[i] = var[c].fx[i]
            else:
                rs232[c].fx[i] = es[c].fx_on[i]
             
        # neo
        for i in range(rs232_neo_nb):
            if(es[c].neo_lock[i]):
                rs232[c].neo[i] = var[c].neo_rvb[i]
            else:
                rs232[c].neo[i] = es[c].neo_rvb[i]
             
        # pwm
        for i in range(rs232_pwm_nb):
            if(es[c].pwm_lock[i]):
                rs232[c].pwm[i] = var[c].pwm[i]
            else:
                rs232[c].pwm[i] = es[c].pwm[i]
    
        # servos
        for i in range(rs232_srv_nb):
            if(es[c].srv_lock[i]):
                rs232[c].srv[i] = var[c].srv[i]
            else:
                rs232[c].srv[i] = es[c].srv[i]
             
        # led
        for i in range(rs232_led_nb):
            if(es[c].led_lock[i]):
                rs232[c].led[i]     = var[c].led[i];
                rs232[c].led_cli[i] = var[c].led_cli[i];
                rs232[c].led_pha[i] = var[c].led_pha[i]
            else:
                if(es[c].led_on[i] == 0):
                    rs232[c].led[i]     = 0;
                    rs232[c].led_cli[i] = 0;
                    rs232[c].led_pha[i] = 0
                if(es[c].led_on[i] == 1):
                    rs232[c].led[i]     = 1;
                    rs232[c].led_cli[i] = 0
                    rs232[c].led_pha[i] = 0
                if(es[c].led_on[i] == 2):
                    rs232[c].led[i]     = 1
                    rs232[c].led_cli[i] = 1
                    rs232[c].led_pha[i] = 0
                if(es[c].led_on[i] == 3):
                    rs232[c].led[i]     = 1
                    rs232[c].led_cli[i] = 1
                    rs232[c].led_pha[i] = 1
        # aig_cmd
            for i in range(rs232_aig_nb):
                if(es[c].aig_lock[i]):
                    rs232[c].aig_cmd[i] = var[c].aig_cmd[i]
                else:
                    rs232[c].aig_cmd[i] = es[c].aig_on[i]
    return

                 
def rs232_to_fdcc_and_debug():
    global es
    
    for c in range(MAX_BOARD_NB):
        for i in range(rs232_in_nb):
            if(es[c].c_var_in_mem == 0):
                es[c].in_on[i] = rs232[c].inp[i]
            else:
                if(es[c].b_var_in_clr == 1):
                    es[c].in_on[i] = 0
                es[c].in_on[i] |= rs232[c].inp[i]
        if(es[c].b_var_in_clr == 1):
            es[c].b_var_in_clr = 0
    return

    
    
#---------------------------------------------------------------------------------------
# Horo
#---------------------------------------------------------------------------------------

horo_on = 1 #activation de l'horloge
horo_j_dep = 0  #jour   0-6  au demarrage
horo_h_dep = 6  #heure  0-23 au demarrage
horo_m_dep = 0  #minute 0-59 au demarrage

horo_j  = horo_j_dep #jour   0-6
horo_h  = horo_h_dep #heure  0-23
horo_m  = horo_m_dep #minute 0-59
# 4  = 1min /   sec      -> 1h / min
# 40 = 1min / 10sec      -> 1h / 10min
horo_tic_by_m = 4   

horo_t = 0

def horo_tic():
    global horo_t
    global horo_j; global horo_h; global horo_m
    
    if(horo_on == 0):
        return
        
    horo_t -= 1
    if(horo_t <= 0):    
        horo_t = horo_tic_by_m
        horo_m += 1
        if(horo_m == 60): 
            horo_m = 0
            horo_h += 1
            if(horo_h == 24):     
                horo_h = 0
                horo_j += 1
                if(horo_j == 7):  
                    horo_j = 0
        #decommenter ces lignes, si vous voulez sauter la nuit      
        #if(horo_h == 22 and horo_m == 31):   # @ 22:31, il sera 06:00
        #    horo_h == 6
        #    horo_m == 0
        #    horo_j += 1
        #       if(horo_j == 7):  
        #            horo_j = 0
                                                        
    return                  


    
#---------------------------------------------------------------------------------------
# Regu
#---------------------------------------------------------------------------------------

# widgets des regulateurs detachables (index suivant l'ordre de lancement des regus)
a_top_used = [ 0 for i in range(rs232_loco_nb+1)]    #0 not used 1-50
a_top_regu = [ 0 for i in range(rs232_loco_nb+1)]    #contient les top des regulateurs
a_mobilelococanvas = [ 0 for i in range(rs232_loco_nb+1)]

a_mobilelocorectslide = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectslidecursor = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocotextsens = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectsens = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectau = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectf0 = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectf1 = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectf2 = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectf3 = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectf4 = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocoadrvit = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocorectcac = [ 0 for i in range(rs232_loco_nb+1)]
a_mobilelocotxtcac = [ 0 for i in range(rs232_loco_nb+1)]


old_red = -1
def independant_regu_redraw_au():
    global old_red
    if((rs232_loc.au_state) and (tic_cli)):
        red = 1
    else:
        red = 0    
    if(old_red == red): #ne redesine que lorsque cela est necessaire
        return
    old_red = red
    
    for i in range(1, 51):
        if(a_top_used[i]):
            try:
                if(red):
                     a_mobilelococanvas[i].itemconfigure(a_mobilelocorectau[i], fill = 'red')
                else:
                     a_mobilelococanvas[i].itemconfigure(a_mobilelocorectau[i], fill = '#cccccc')
            except:
                pass
    return

def independant_regu_redraw(adr):
    global a_mobilelococanvas
    global a_mobilelocorectslide
    global a_mobilelocorectslidecursor
    global a_mobilelocorectsens
    global a_mobilelocotextsens
    global a_mobilelocorectau
    global a_mobilelocorectf0
    global a_mobilelocorectf1
    global a_mobilelocorectf2
    global a_mobilelocorectf3
    global a_mobilelocorectf4
    global a_mobilelocoadrvit
    global a_mobilelocorectcac
    global a_mobilelocotxtcac

    sens = rs232_loc.sens[adr]
    v_abs  = rs232_loc.vit[adr]

    if(a_top_used[adr] == 0):
        return

    # indication adresse et vitesse
    if(sens):
        adrvit = "$" + str(adr) + " +" + str(v_abs)
    else:
        adrvit = "$" + str(adr) + " -" + str(v_abs)
    a_mobilelococanvas[adr].itemconfigure(a_mobilelocoadrvit[adr], text = adrvit)

    # vit
    a_mobilelococanvas[adr].coords(a_mobilelocorectslidecursor[adr], 10, 35 + 4 * (28 - v_abs), 10 + 10 + 20 + 10, 35 + 4 * (28 - v_abs) + 10)

    # sens
    if(sens == 0):
         a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectsens[adr], fill = '#FF8888')
         a_mobilelococanvas[adr].itemconfigure(a_mobilelocotextsens[adr], text = "-")
    else:
         a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectsens[adr], fill = '#8888FF')
         a_mobilelococanvas[adr].itemconfigure(a_mobilelocotextsens[adr], text = "+")

    # fct
    if(rs232_loc.fct0[adr]):
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf0[adr], fill = 'yellow')
    else:
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf0[adr], fill = '#888888')
    if(rs232_loc.fct1[adr]):
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf1[adr], fill = 'yellow')
    else:
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf1[adr], fill = '#888888')
    if(rs232_loc.fct2[adr]):
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf2[adr], fill = 'yellow')
    else:
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf2[adr], fill = '#888888')
    if(rs232_loc.fct3[adr]):
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf3[adr], fill = 'yellow')
    else:
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf3[adr], fill = '#888888')
    if(rs232_loc.fct4[adr]):
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf4[adr], fill = 'yellow')
    else:
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectf4[adr], fill = '#888888')
        
    # cac       
    if(cac_on[adr]):
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectcac[adr], fill = 'red')
    else:
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocorectcac[adr], fill = '#888888')
    if(cac_side[adr] == "+"):
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocotxtcac[adr], text = "CAC+")       
    else:
        a_mobilelococanvas[adr].itemconfigure(a_mobilelocotxtcac[adr], text = "CAC-")       

    # au
    # independant_regu_redraw_au()    #appele par la bcl principale maintenant
                                               
    return


def independant_regu_redraw_all():
    for i in range(51):
        if(a_top_used[i]):
            independant_regu_redraw(i)
    return

    
in_regu = 0    
last_regu_used = -1

def independant_regu_callback_canvas_mouse(event):
    global rs232_loc
    global in_regu
    global last_regu_used
    global cac_on

    ##print("canvas regu " + str(event.widget) + "clicked at:" + str(event.x) + "," + str (event.y))
    
    adr = 0
    for i in range(51):
        if(a_top_used[i]):
            if(str(event.widget) == str(a_mobilelococanvas[i])):
               adr = i
               break
    ##print("adr=" + str(adr))
    last_regu_used = adr

    # sens
    x = 15; y = 180
    if((event.x > x) and (event.x < (x + 30)) and (event.y > y) and (event.y < (y + 30))):
        rs232_loc.sens[adr] ^= 1

    # vit
    elif(event.x < 40):
        v = (35 + 28 * 4 - event.y) // 4
        ##print("v="+str(v))
        if(v < 0):
            v = 0
        if(v > 28):
            v = 28
        if((v >= 0) and (v <= 28)):
            rs232_loc.vit[adr] = v
            in_regu = adr

    elif((event.x >= 40) and (event.x <= 75)):
        if((event.y >= 35) and (event.y <= 85)):
            if(rs232_loc.vit[adr] < 28):
                rs232_loc.vit[adr] += 1
        if((event.y >= 120) and (event.y <= 170)):
            if(rs232_loc.vit[adr] > 0):
                rs232_loc.vit[adr] -= 1

    # fct
    if((event.x >= 75) and (event.x <= 105)):
        y = 20
        if((event.y >= y) and (event.y <= y + 30)):
            rs232_loc.fct0[adr] ^= 1
        y += 30
        if((event.y >= y) and (event.y <= y + 30)):
            rs232_loc.fct1[adr] ^= 1
        y += 30
        if((event.y >= y) and (event.y <= y + 30)):
            rs232_loc.fct2[adr] ^= 1
        y += 30
        if((event.y >= y) and (event.y <= y + 30)):
            rs232_loc.fct3[adr] ^= 1
        y += 30
        if((event.y >= y) and (event.y <= y + 30)):
            rs232_loc.fct4[adr] ^= 1
            
    # cac
    if((event.x >= 75) and (event.x <= 105)):
        y = 230
        if((event.y >= y) and (event.y <= y + 30)):
            cac_on[adr] ^= 1
    
    # au
    y = 180
    if((event.x >= 75) and (event.x <= 105) and (event.y >= y) and (event.y <= y + 30)):
        if(rs232_loc.au_state == 0):
            rs232_loc.au_set = 1
        else:       
            rs232_loc.au_clr = 1  # a commenter pour ne pas arreter l'AU par les regus

    # redraw
    independant_regu_redraw(adr)
                                               
    return
    
    
def independant_regu_callback_canvas_mouse_3(event):
    global cac_side
    
    adr = 0
    for i in range(51):
        if(a_top_used[i]):
            if(str(event.widget) == str(a_mobilelococanvas[i])):
               adr = i
               break
    #print("canvas regu " + str(event.widget) + "clicked 3 at:" + str(event.x) + "," + str (event.y) + "adr= " + str(adr))
     
    # cac
    if((event.x >= 75) and (event.x <= 105)):
        y = 230
        if((event.y >= y) and (event.y <= y + 30)):
            if(cac_on[adr]):
                if(cac_side[adr] == "+"):
                    cac_side[adr] = "-"
                else:
                    cac_side[adr] = "+"

    # redraw
    independant_regu_redraw(adr)
                                               
    return
    
    
def independant_regu_callback_canvas_mouse_move(event):
    global rs232_loc
    global in_regu
    
    ##print("move " + str(event.widget) + "clicked at:" + str(event.x) + "," + str (event.y))
    if(in_regu == 0):
        return
    adr = in_regu
    v = (35 + 28 * 4 - event.y) // 4
    if(v < 0):
        v = 0
    if(v > 28):
        v = 28
    if((v >= 0) and (v <= 28)):
        ##print("v="+str(v))
        rs232_loc.vit[adr] = v
        independant_regu_redraw(adr)
    
    
def independant_regu_callback_canvas_mouse_release(event):
    global in_regu
    
    ##print("release " + str(event.widget) + "clicked at:" + str(event.x) + "," + str (event.y))
    in_regu = 0
    return
    
    
def independant_regu_callback_canvas_key(event):
    k = repr(event.char)
    print("KEY=" + k[1])
    #if(k[1]=="9" or k[1]=="+"):
    return
    

def regu_close_callback(adr):
    global a_top_used
    global a_top_regu
    ##print("regu " + str(adr) + " closed, top= " + str(a_top_regu[adr]))
    a_top_used[adr] = 0
    #time.sleep(1)
    a_top_regu[adr].destroy()
    return

    
def regu_close_callback_01():    #il doit y avoir plus simple !
    regu_close_callback(1)
def regu_close_callback_02():
    regu_close_callback(2)
def regu_close_callback_03():
    regu_close_callback(3)
def regu_close_callback_04():
    regu_close_callback(4)
def regu_close_callback_05():
    regu_close_callback(5)
def regu_close_callback_06():
    regu_close_callback(6)
def regu_close_callback_07():
    regu_close_callback(7)
def regu_close_callback_08():
    regu_close_callback(8)
def regu_close_callback_09():
    regu_close_callback(9)
def regu_close_callback_10():
    regu_close_callback(10)
def regu_close_callback_11():
    regu_close_callback(11)
def regu_close_callback_12():
    regu_close_callback(12)
def regu_close_callback_13():
    regu_close_callback(13)
def regu_close_callback_14():
    regu_close_callback(14)
def regu_close_callback_15():
    regu_close_callback(15)
def regu_close_callback_16():
    regu_close_callback(16)
def regu_close_callback_17():
    regu_close_callback(17)
def regu_close_callback_18():
    regu_close_callback(18)
def regu_close_callback_19():
    regu_close_callback(19)
def regu_close_callback_20():
    regu_close_callback(20)
def regu_close_callback_21():
    regu_close_callback(21)
def regu_close_callback_22():
    regu_close_callback(22)
def regu_close_callback_23():
    regu_close_callback(23)
def regu_close_callback_24():
    regu_close_callback(24)
def regu_close_callback_25():
    regu_close_callback(25)
def regu_close_callback_26():
    regu_close_callback(26)
def regu_close_callback_27():
    regu_close_callback(27)
def regu_close_callback_28():
    regu_close_callback(28)
def regu_close_callback_29():
    regu_close_callback(29)
def regu_close_callback_30():
    regu_close_callback(30)
def regu_close_callback_31():
    regu_close_callback(31)
def regu_close_callback_32():
    regu_close_callback(32)
def regu_close_callback_33():
    regu_close_callback(33)
def regu_close_callback_34():
    regu_close_callback(34)
def regu_close_callback_35():
    regu_close_callback(35)
def regu_close_callback_36():
    regu_close_callback(36)
def regu_close_callback_37():
    regu_close_callback(37)
def regu_close_callback_38():
    regu_close_callback(38)
def regu_close_callback_39():
    regu_close_callback(39)
def regu_close_callback_40():
    regu_close_callback(40)
def regu_close_callback_41():
    regu_close_callback(41)
def regu_close_callback_42():
    regu_close_callback(42)
def regu_close_callback_43():
    regu_close_callback(43)
def regu_close_callback_44():
    regu_close_callback(44)
def regu_close_callback_45():
    regu_close_callback(45)
def regu_close_callback_46():
    regu_close_callback(46)
def regu_close_callback_47():
    regu_close_callback(47)
def regu_close_callback_48():
    regu_close_callback(48)
def regu_close_callback_49():
    regu_close_callback(49)
def regu_close_callback_50():
    regu_close_callback(50)


# code des regulateurs detachables
def independant_regu(adr):
    global a_top_used
    global a_top_regu
    global a_mobilelococanvas
    global a_mobilelocorectsens
    global a_mobilelocorectau
    global a_mobilelocorectf0
    global a_mobilelocorectf1
    global a_mobilelocorectf2
    global a_mobilelocorectf3
    global a_mobilelocorectf4
    global a_mobilelocorectslide
    global a_mobilelocorectslidecursor
    global a_mobilelocorectcac
    #global zetop

    if(a_top_used[adr]):    #exit if already one
        return
    a_top_regu[adr] = top = tkinter.Toplevel()
    a_top_used[adr] = 1
    ##print("top="+str(top))
    top.wm_attributes("-topmost", 1)    #top.lift(aboveThis=gtop) #not working

    if(adr == 1):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_01)
    if(adr == 2):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_02)
    if(adr == 3):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_03)
    if(adr == 4):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_04)
    if(adr == 5):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_05)
    if(adr == 6):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_06)
    if(adr == 7):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_07)
    if(adr == 8):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_08)
    if(adr == 9):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_09)
    if(adr == 10):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_10)
    if(adr == 11):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_11)
    if(adr == 12):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_12)
    if(adr == 13):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_13)
    if(adr == 14):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_14)
    if(adr == 15):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_15)
    if(adr == 16):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_16)
    if(adr == 17):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_17)
    if(adr == 18):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_18)
    if(adr == 19):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_19)
    if(adr == 20):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_20)
    if(adr == 21):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_21)
    if(adr == 22):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_22)
    if(adr == 23):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_23)
    if(adr == 24):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_24)
    if(adr == 25):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_25)
    if(adr == 26):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_26)
    if(adr == 27):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_27)
    if(adr == 28):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_28)
    if(adr == 29):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_29)
    if(adr == 30):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_30)
    if(adr == 31):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_31)
    if(adr == 32):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_32)
    if(adr == 33):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_33)
    if(adr == 34):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_34)
    if(adr == 35):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_35)
    if(adr == 36):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_36)
    if(adr == 37):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_37)
    if(adr == 38):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_38)
    if(adr == 39):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_39)
    if(adr == 40):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_40)
    if(adr == 41):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_41)
    if(adr == 42):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_42)
    if(adr == 43):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_43)
    if(adr == 44):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_44)
    if(adr == 45):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_45)
    if(adr == 46):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_46)
    if(adr == 47):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_47)
    if(adr == 48):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_48)
    if(adr == 49):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_49)
    if(adr == 50):
        top.protocol("WM_DELETE_WINDOW", regu_close_callback_50)
        
       
    a_mobilelococanvas[adr] = my_canvas = tkinter.Canvas(top, bg = "#5B9BD5" , height = 290, width = 160)   #bg="green" sous windows la fenetre faisait plus de 120 !!!
    my_canvas.grid(row = 0,column = 0)
    ##print("canvas="+str(my_canvas))
    
    #bindings
    my_canvas.bind("<Button-1>", independant_regu_callback_canvas_mouse)
    my_canvas.bind("<B1-Motion>", independant_regu_callback_canvas_mouse_move)
    my_canvas.bind("<ButtonRelease-1>", independant_regu_callback_canvas_mouse_release)
    my_canvas.bind("<Button-3>", independant_regu_callback_canvas_mouse_3)
    my_canvas.bind("<Key>", independant_regu_callback_canvas_key)    #ne recoit pas les keys
    ###top.bind("<Key>", callback_canvas_loco_key)  #uu:voir cela !!!    

    #adr-vit
    x = 30; y = 20;    a_mobilelocoadrvit[adr] = my_canvas.create_text(x, y, text = "$03 +31" )

    #slide
    x = 20; y = 35;       a_mobilelocorectslide[adr] = my_canvas.create_rectangle(x, y, x + 20, y + 112 + 10, fill = '#cccccc', outline = 'black')    #0-31: 32 position -1 courrante = 28 * 4 pixels = 112
    x = 10; y = 35 + 112; a_mobilelocorectslidecursor[adr] = my_canvas.create_rectangle(x, y, x + 40, y + 10, fill = 'red', outline = 'black')
    x = 15; y = 35 + 124 + 10 + 10;

    #sens
    y = 180
    a_mobilelocorectsens[adr] = my_canvas.create_rectangle(x, y, x + 30, y + 30, fill = 'blue', outline = 'black')
    a_mobilelocotextsens[adr] = my_canvas.create_text(x + 15, y + 15, text = "+")

    #F0-4
    x = 75;
    y = 20;    a_mobilelocorectf0[adr] = my_canvas.create_rectangle(x, y, x + 30, y + 30, fill = 'yellow', outline = 'black')
    my_canvas.create_text(x + 15, y + 15, text = "FL")
    y += 30;   a_mobilelocorectf1[adr] = my_canvas.create_rectangle(x, y, x + 30, y + 30, fill = 'yellow', outline = 'black')
    my_canvas.create_text(x + 15, y + 15, text = "F1")
    y += 30;   a_mobilelocorectf2[adr] = my_canvas.create_rectangle(x, y, x + 30, y + 30, fill = 'yellow', outline = 'black')
    my_canvas.create_text(x + 15, y + 15, text = "F2")
    y += 30;   a_mobilelocorectf3[adr] = my_canvas.create_rectangle(x, y, x + 30, y + 30, fill = 'yellow', outline = 'black')
    my_canvas.create_text(x + 15, y + 15, text = "F3")
    y += 30;   a_mobilelocorectf4[adr] = my_canvas.create_rectangle(x, y, x + 30, y + 30, fill = 'yellow', outline = 'black')
    my_canvas.create_text(x + 15, y + 15, text = "F4")
    
    #AU LOCO
    y += 40; a_mobilelocorectau[adr] = my_canvas.create_rectangle(x, y, x + 30, y + 30, fill = '#cccccc', outline = 'black')
    my_canvas.create_text(x + 15, y + 15, text = "AU")

    #CAC
    x = 75; y = 230
    a_mobilelocorectcac[adr] = my_canvas.create_rectangle(x, y, x + 30, y + 30, fill = 'yellow', outline = 'black')
    a_mobilelocotxtcac[adr] = my_canvas.create_text(x + 15, y + 15, text = "CAC+")      
    
    #image
    try:
        my_canvas.create_image(10, 220, image = a_locophotoimage[adr], anchor = "nw")
    except:
        pass

    #redraw
    independant_regu_redraw(adr)
    return


    
def callback_key(event):
    k = repr(event.char)
    print("KEY=" + k[1])
    #if(k[1]=="9" or k[1]=="+"):
    return
    
    
    
    
#---------------------------------------------------------------------------------------
# Loco
#---------------------------------------------------------------------------------------

# widgets de la fenetre principale (index loco suivant le fichier de config)
a_locorect       = [ 0 for i in range(rs232_loco_nb+1)]   #index 0 not used 1-50
a_locovalid      = [ 0 for i in range(rs232_loco_nb+1)]
a_loconame       = [ 0 for i in range(rs232_loco_nb+1)]
a_locophotoimage = [ 0 for i in range(rs232_loco_nb+1)]
a_locoimage      = [ 0 for i in range(rs232_loco_nb+1)]

a_locotext1      = [ 0 for i in range(rs232_loco_nb+1)]
a_locotext2      = [ 0 for i in range(rs232_loco_nb+1)]
a_locotext3      = [ 0 for i in range(rs232_loco_nb+1)]

rs232_souris_nb = 10
a_sourisoval     = [ 0 for i in range(rs232_souris_nb)]
a_souristxt      = [ 0 for i in range(rs232_souris_nb)]
locorect_au      = 0
locotxt_au       = 0

my_canvas_loco = tkinter.Canvas(top, bg = "#5B9BD5", width = 5 * (150 + 10) +10 + 30 + 10, height = 10 * (50 + 10) + 10)

screen_loco_on = 0

def loco_on():
    my_canvas_loco.grid(row = 0, column = 0)
    return

    
def loco_off():
    my_canvas_loco.grid_forget()
    return


with open("config_loco.txt", mode = "r", encoding = "utf-8", errors = 'ignore') as my_file:   #ignore remove non utf-8 chars
    adr = 0; name = ""; imgfile = ""
    
    print("LOCO: loading file")
    i = 0; j = 0;
    for adr in range(1, rs232_loco_nb + 1):   #1--50
        x = (150 +10) * i + 10
        y = (50  +10) * j + 10
        j += 1
        if(j == 10):
            j = 0
            i += 1            
        # frame & image
        a_locorect[adr]  = my_canvas_loco.create_rectangle(x, y - 1, x + 150, y + 50, fill = "#5B9BD5", outline = "#5B9BD5")        
        my_canvas_loco.itemconfigure(a_locorect[adr], fill = '#7BABFF')
    
    i = 0; j = 0; cpt = 0   
    for line in my_file:
        line2 = line.rstrip("\n")
        if(line2 == ""):
            continue
        if(line2[0] == "#"):
            continue
        #print("cpt=" + str(cpt) + " " + line2 + " .")
        if(cpt == 0):
            adr = int(line2)
            cpt += 1
        elif(cpt == 1):
            name = line2
            cpt += 1
        else:
            cpt = 0
            if(adr >= 1 and adr <= rs232_loco_nb):
                a_loconame[adr] = name
                imgfile = line2 
                print(" adr=" + str(adr) + " name=" + name + " imgfile=" + imgfile + ".")           
                a_locophotoimage[adr] = tkinter.PhotoImage(file = imgfile)
                #a_locowidth[adr] = a_locophotoimage[adr].width()
                #a_locoheight[adr] = a_locophotoimage[adr].height()          
                i = (adr - 1) // 10
                j = (adr - 1) - 10 * i
                x = (150 + 10) * i + 10 + 10
                y = (50  + 10) * j + 10
                a_locoimage[adr] = my_canvas_loco.create_image(x, y + 1, image = a_locophotoimage[adr], anchor = "nw")
                a_locovalid[adr] = 1
                #my_canvas_loco.itemconfigure(a_locorect[adr], fill='#7BABFF')

    i = 0; j = 0;
    for adr in range(1, 51):   #1--50
        x = (150 + 10) * i + 10
        y = (50  + 10) * j + 10
        j += 1
        if(j == 10):
            j = 0
            i += 1            
        # frame & image
        sz = ""
        if(adr < 10):
            sz += "0"
            sz += str(adr)
        else:     
            sz += str(adr)
        sz += ":"
        if(a_locovalid[adr]):
            sz += a_loconame[adr]        
        a_locotext1[adr] = my_canvas_loco.create_text(x + 60, y     , text = sz          , fill = 'white', anchor = "nw")  #"01:bb63000"
        a_locotext2[adr] = my_canvas_loco.create_text(x + 60, y + 15, text = "+00 F-----", fill = 'white', anchor = "nw")  #"+25 F01234"
        a_locotext3[adr] = my_canvas_loco.create_text(x + 60, y + 30, text = ""          , fill = 'white', anchor = "nw")  #"RANS1"

print("LOCO: file loaded")
        
x = 5 * 160 + 10 + 20; y = 10 + 15
locorect_au = my_canvas_loco.create_rectangle(x - 15, y - 15, x + 15, y + 15, fill = "#00cc00", outline = "#000000")
locotxt_au  = my_canvas_loco.create_text(x, y, text = "AU")
    
old_red2 = -1
def loco_redraw_au():
    global old_red2
    if((rs232_loc.au_state) and (tic_cli)):
        red = 1
    else:
        red = 0    
    if(old_red2 == red): #ne redesine que lorsque cela est necessaire
        return
    old_red2 = red
    
    if(red):
        my_canvas_loco.itemconfigure(locorect_au, fill = 'red')
    else:
        my_canvas_loco.itemconfigure(locorect_au, fill = '#cccccc')
    return

    
#souris_adr  = [ 0 for i in range(10)]
x = 5 * 160 + 10 + 20; y = 70
for i in range(0, rs232_souris_nb):
    a_sourisoval[i] = my_canvas_loco.create_oval(x - 12, y - 12, x + 12, y + 12, fill = "#5B9BE5", outline = "#000000")
    a_souristxt[i]  = my_canvas_loco.create_text(x, y, text = "S" + str(i) , fill = 'white', anchor = "center")
    y += 30
    
def loco_move_souris_adr(m,adr):
    if(adr == 0):
        x = 5 * 160 + 10 + 20 - 12
        y = 70 + 30 * m - 12
    else:
        c = (adr - 1) // 10
        l = (adr - 1) % 10
        x = 10 + 160 * c + 125  # - m  # - m est un leger decalage si on a plusieurs souris sur la meme loco
        y = 10 + 60  * l +  25
    my_canvas_loco.coords(a_sourisoval[m], x, y, x + 24, y + 24)    
    my_canvas_loco.coords(a_souristxt[m] , x + 12, y + 12) 
    
a_locorect_fill = [ "" for i in range(51) ]   
a_locorect_txt  = [ "" for i in range(51) ]   
def loco_update():
    global my_canvas_loco
    global a_locorect_fill
    global a_locorect_txt
    
    for adr in range(1, rs232_loco_nb + 1):   #1--50
        if(rs232_loc.sens[adr]):
            t = "+"
        else:
            t = "-"
        v = rs232_loc.vit[adr]
        if(v < 10):
            t += "0"
        t += str(v)
        t += " F"
        if(rs232_loc.fct0[adr]):
            t += "0"
        else:
            t += "-"
        if(rs232_loc.fct1[adr]):
            t += "1"
        else:
            t += "-"
        if(rs232_loc.fct2[adr]):
            t += "2"
        else:
            t += "-"
        if(rs232_loc.fct3[adr]):
            t += "3"
        else:
            t += "-"
        if(rs232_loc.fct4[adr]):
            t += "4"
        else:
            t += "-"
        if(a_locorect_txt[adr] != t):
            a_locorect_txt[adr] = t
            my_canvas_loco.itemconfigure(a_locotext2[adr], text = t)
        
        if(v == 0):
            new_fill = '#7BABFF'  #bleue par defaut
        else:       
            new_fill = '#008000'  #vert fonce
        if(a_locorect_fill[adr] != new_fill):
            a_locorect_fill[adr] = new_fill
        try:
            my_canvas_loco.itemconfigure(a_locorect[adr], fill = new_fill)  # peut planter (chez Francois)
        except:
            pass
            
    for i in range(0, 10):
        if(rs232_loc.souris_disc[i]):
            my_canvas_loco.itemconfigure(a_sourisoval[i], fill = '#cccccc') 
        else:
            my_canvas_loco.itemconfigure(a_sourisoval[i], fill = '#5B9BE5')   # ou rosee '#E55B9B'
    return

        
def callback_canvas_loco_mouse(event):
    global rs232_loc
    
    ##print("canvas loco clicked at:" + str(event.x) + "," + str (event.y))

    i = (event.x - 10) // 160
    j = (event.y - 10) // 60
    adr = 10 * i + j + 1
    
    ii = event.x - (10 + 160 * i)

    # AU ou selection souris libre    
    if(event.x > (10 + 5 * 160 + 10)):
        if(event.y < 40):
            if(rs232_loc.au_state == 0):
                rs232_loc.au_set = 1
            else:       
                rs232_loc.au_clr = 1
        else:
            m = (event.y - (70-15)) // 30
            if(m >= 0 and m <= 10 and rs232_loc.souris_adr[m] == 0):
                ##print("mouse " + str(m))
                rs232_loc.in_mouse_sel = m
        return      
    
    # selection souris utilisee    
    if(ii > 120):
        for m in range(10):
            if(rs232_loc.souris_adr[m] == adr):
                ##print("mouse2 " + str(m))
                rs232_loc.in_mouse_sel = m        
        return

    # selection regu
    if(adr >= 1 and adr <= 50):
        ##print("adr:" + str(adr))
        independant_regu(adr)
    
    
def callback_canvas_loco_mouse_release(event):
    global rs232_loc
    
    ##print("canvas loco released at:" + str(event.x) + "," + str (event.y))
    
    i = (event.x - 10) // 160
    j = (event.y - 10) // 60
    adr = 10 * i + j + 1
    
    if(rs232_loc.in_mouse_sel != -1):
        if(adr <= 0 or adr >= 51):
            adr = 0
        print("final adr = " + str(adr))
        loco_move_souris_adr(rs232_loc.in_mouse_sel, adr)
        rs232_loc.souris_do_not_update = 4
        rs232_loc.souris_adr[rs232_loc.in_mouse_sel] = adr
        rs232_loc.in_mouse_sel = -1


def callback_canvas_loco_mouse_move(event):
    if(rs232_loc.in_mouse_sel == -1):       
        return
    x = event.x - 12
    y = event.y - 12
    m = rs232_loc.in_mouse_sel
    my_canvas_loco.coords(a_sourisoval[m], x, y, x + 24, y + 24)    
    my_canvas_loco.coords(a_souristxt[m] , x + 12, y + 12)  

    
my_canvas_loco.bind("<Button-1>", callback_canvas_loco_mouse)
my_canvas_loco.bind("<B1-Motion>", callback_canvas_loco_mouse_move)
my_canvas_loco.bind("<ButtonRelease-1>", callback_canvas_loco_mouse_release)
#my_canvas_loco.bind("<Button-3>", callback_canvas_loco_mouse3)

'''
def callback_canvas_loco_key(event):
    print("canvas loco pressed:" + repr(event.char))

#seul les top et toplevel recoivent les touches !
#my_canvas_loco.bind("<Key>", callback_canvas_loco_key)    #marche pas !
top.bind("<Key>", callback_canvas_loco_key)
#sans passer a une autre fenetre ca ne marche pas !
top.iconify()
top.deiconify()
'''



#---------------------------------------------------------------------------------------
# PROG
#---------------------------------------------------------------------------------------

def cv_prog_cb():
    global rs232_loc
             
    try:
        adr = int(e_cv_adr.get())
        dat = int(e_cv_dat.get())
    except:
        tkinter.messagebox.showerror(SZ_TITLE, "Erreur ADR=1-1024 DAT=0-255")
        return
        
    if(adr < 1 or adr > 1024 or dat < 0 or dat > 255):
        tkinter.messagebox.showerror(SZ_TITLE, "Erreur ADR=1-1024 DAT=0-255")
        return

    rs232_loc.cv_prog_adr_h = adr >> 8
    rs232_loc.cv_prog_adr_l = adr & 255
    rs232_loc.cv_prog_dat   = dat
    rs232_loc.cv_prog_go    = 1
    
    print("PROG CV " + str(adr) + " = " + str(dat))
    
    return

l_cv_adr  = tkinter.Label(top, text = "CV ADR 1-1024")
e_cv_adr  = tkinter.Entry(top)
l_cv_dat  = tkinter.Label(top, text = "DAT 0-255")
e_cv_dat  = tkinter.Entry(top)
b_cv_prog = tkinter.Button(top, text = "PROG", command = cv_prog_cb)
e_cv_adr.delete(0, 1000); e_cv_adr.insert(0, "1")
e_cv_dat.delete(0, 1000); e_cv_dat.insert(0, "3")

screen_cvprog_on = 0

def cvprog_on():
    l_cv_adr.grid(row = 0, column = 0)
    e_cv_adr.grid(row = 0, column = 1)
    l_cv_dat.grid(row = 0, column = 2)
    e_cv_dat.grid(row = 0, column = 3)
    b_cv_prog.grid(row = 0, column = 4)
    tkinter.messagebox.showwarning("ATTENTION", "ATTENTION, TOUTES LES LOCOMOTIVES ALIMENTEES PAR LE BOOSTER SERONT PROGRAMEES !")
    return

def cvprog_off():
    l_cv_adr.grid_forget()
    e_cv_adr.grid_forget()
    l_cv_dat.grid_forget()
    e_cv_dat.grid_forget()
    b_cv_prog.grid_forget()
    return



#---------------------------------------------------------------------------------------
# TDV (Equations)
#---------------------------------------------------------------------------------------

#
# fichier des equations
#
# operations
#  l <var>  / ln <var> : lecture       acc = var          / acc = ~var
#  a <var>  / an <var> : ET            acc = acc ET   var / acc = acc ET   ~var
#  o <var>  / on <var> : OU            acc = acc OU   var / acc = acc OU   ~var
#  x <var>  / xn <var> : OU EXCLUSIF   acc = acc OUEX var / acc = acc OUEX ~var
#  = <var>  / =n <var> : affectation   var = acc          / var = ~acc
#  =s <var> / =c <var> : mise a 1      si(acc==1) var=1   / mise a 0 si(acc==1) var=0
#
# variables binaires (0/1): (L=Lecture seule)
#  i<0-255> L: etat courant de l'entree
#  y<0-255> L: ancien  etat de l'entree
#  o<0-63>   : sortie
#  l<0-255>  : led
#  c<0-255>  : clignotement de led
#  p<0-255>  : phase de led
#  a<0-95 >  : commande aiguillage
#  d<0-95 > L: position directe aiguillage
#  e<0-95 > L: position evitement aiguillage
#  t<0-999>  : itineraire cmd
#  k<0-999> L: itineraire ok
#  w<0-999> L: itineraire voyant
#  u<0-999> L: itineraire ouverture
#  r<0-95 >  : reservation d'aiguillage
#  z<0-199> L: reservation de zone
#  v<0-999> L: variable
#  f<0-199>  : commutateur de fermeture
#  s<0-199>  : effets speciaux
#  j<0-99>  L: occupation du canton
#
# variables entieres (0-255):
#  ana<0-1  > L: entree analogique (0=0v, 255=5V)
#  pwm<0-51 >  : sortie pwm (0=0%, 255=100%)
#  srv<0-47 >  : servomoteur (50=0.5ms, 150=1.5ms, 250=2.5ms)
#  var<0-999>  : variable (les variables var et v ne sont pas communes)
# - pour un test, remplacer <var> par <var255>==<val> (ou <>, >=, <=, <, >)
# - pour une affectation, remplacer <var> par <var255>=val
#    (l'affectation est effectuee si acc==1)
#
# variables des couleurs (#000000-#FFFFFF):
#  neo<0-59>  : led neopixel
#
# ex:
# l i1 a ana1>=200 = o3 = srv4=28 =n srv4=0
#

'''
a.   AIG
b   ????? peut etre pour le bmvu
c.   LED-CLI
d.   AIG_DIR (rd)
e.   AIG-EVI (rd)
f    commutateurs de fermeture
g
h
i.   IN     (rd)
j    OCC-CANTON (rd)
k    ITI ok (rd)
l.   LED
m
n
o.   OUT
p.   LED-PHA
q
r.   AIG-RES (rd)
s.   fonctions speciales
t    ITI cmd
u    ITI ouv (rd)
v    VAR
w    ITI voy (rd)
x
y.   OLD IN  (rd)
z    REZ-Z   (rd)
.=un jeu par board
'''

tdv_file_instr = []
tdv_file_var = []

with open("config_eq.txt", mode = "r", encoding = "utf-8", errors = 'ignore') as my_file:   #ignore remove non utf-8 chars
    print("TDV: loading file")    
    for line in my_file:
        line2 = line.rstrip("\n")
        #print(line2)
        if(line2 == ""):
            continue
        if(line2[0] == "#"):
            continue
        astr = line2.split()    #separateur blanc(s) ou tab
        l = len(astr)
        if(l % 2):
            print("ERROR tdv: " + line2)
            quit()
        for i in range(l // 2):
            tdv_file_instr.append(astr[2 * i])
            tdv_file_var.append(astr[2 * i + 1])

    print("TDV: file loaded with " + str(len(tdv_file_instr)) + " operations")
    #print(str(tdv_file_instr))
    #print(str(tdv_file_var))

    
# pour les variables binaires, tdv_set assigne val a la variable
# pour les autres, tdv_set evalue l'expression si val==1 (pwm18=125, 125 sera affecte a pwm18 si val==1)    
def tdv_set(szVar,val):    # uu: voir si tdv_inv(szVar) a du sens
    global rs232
    global var
    global var2
    
    if(szVar == ""):
        print("ERROR tdv, empty variable")
        return
        
    if(szVar[1] >= "0" and szVar[1] <= "9"):
    
        c = szVar[0]
        num = int(szVar[1:]); board = num // 1000; num = num % 1000
        if(board > MAX_BOARD_NB):
            print("ERROR tdv, invalid variable " + szVar)
            return
    
        #cannot set i, y, d, e, k, u, w, r, z
        if(c == "o"):
            if(num < rs232_out_nb):
                var[board].out[num] = val
                return
        if(c == "l"):
            if(num < rs232_led_nb):
                var[board].led[num] = val
                return
        if(c == "c"):
            if(num < rs232_led_nb):
                var[board].led_cli[num] = val
                return
        if(c == "p"):
            if(num < rs232_led_nb):
                var[board].led_pha[num] = val
                return
        if(c == "a"):
            if(num < rs232_aig_nb):
                if(var[board].res[num] == 0):   #faire seulement si l'aiguillage n'est pas reserve
                    var[board].aig_cmd[num] = val
                return
        if(c == "v"):
            if(num < var_nb):
                var2.var[num] = val
                return
        if(c == "t"):
            if(num < nb_iti):
                var2.iti_cmd[num] = val
                return
        if(c == "i" and rs232_ok == 0):  #possibilite d'ecrire les entrees, uniquement en mode deconnecte
                rs232[board].inp[num] = val
                return
        if(c == "f"):
            if(num < cf_nb):
                var2.cf[num] = val
        if(c == "s"):
            if(num < fx_nb):
                var[board].fx[num] = val
            
    else:
        if(val != 1):
            return
            
        dec = szVar.split("=")  #pwm38=124 -> pwm38 124
        if(len(dec) != 2):
            return
        num = int(dec[0][3:]); board = num // 1000; num = num % 1000
        c   = dec[0][:3]
        if(c == "var"):    #var38=124
            if(num < var8_nb):
                val = int(dec[1])
                var2.var8[num] = val
                return
        if(c == "pwm"):   #pwm38=124
            if(num < rs232_pwm_nb):
                val = int(dec[1])
                var[board].pwm[num] = val
                return
        if(c == "srv"):   #srv38=124
            if(num < rs232_srv_nb):
                val = int(dec[1])
                var[board].srv[num] = val
                return
        if(c == "neo"):    #neo28=#ff0000
                val = int(dec[1][1:],16)
                var[board].neo_rvb[num] = val
                return
    
    print("ERROR tdv, invalid variable " + szVar)
    #quit()


#helper fct    
def get_val_from_var(name, num): 
    if(name == "ana"):
        if(num < rs232_an_nb):
            board = num // 1000; num = num % 1000 
            return rs232[board].an[num]
    if(name == "var"):
        if(num < var8_nb):
            return var2.var8[num]
    return 0

    
# pour les variables binaires, tdv_get retourne l'etat de la variable
# pour les autres, tdv_get retourne la reponse a l'evaluation (var12==1000 retourne 0 sir var12 different de 1000, sinon retourne 1)    
def tdv_get(szVar):
    if(szVar == ""):
        print("ERROR tdv, empty variable")
        return 0

    if(szVar[1] >= "0" and szVar[1] <= "9"):
        c = szVar[0]
        num = int(szVar[1:]); board = num // 1000; num = num % 1000
        if(board > MAX_BOARD_NB):
            print("ERROR tdv, invalid variable " + szVar)
            return 0
        if(c == "i"):
            if(num < rs232_in_nb):
                return rs232[board].inp[num]    
        if(c == "j"):
            if(num < 100):
                return cantons[num].occ
        if(c == "y"):
            if(num < rs232_in_nb):
                return rs232[board].old_in[num]    
        if(c == "o"):
            if(num < rs232_out_nb):
                return var[board].out[num]    
        if(c == "l"):
            if(num < rs232_led_nb):
                return var[board].led[num] 
        if(c == "c"):
            if(num < rs232_led_nb):
                return var[board].led_cli[num]
        if(c == "p"):
            if(num < rs232_led_nb):
                return var[board].led_pha[num] 
        if(c == "a"):
            if(num < rs232_aig_nb):
                return var[board].aig_cmd[num] 
        if(c == "d"):
            if(num < rs232_aig_nb):
                return rs232[board].aig_dir[num] 
        if(c == "e"):
            if(num < rs232_aig_nb):
                return rs232[board].aig_dev[num]
        if(c == "v"):
            if(num < var_nb):
                if(var2.var[num]):
                    return 1
                else:
                    return 0
        if(c == "t"):
            if(num < nb_iti):
                #return is_iti_ok(num)        
                return var2.iti_cmd[num]               
        if(c == "k"):  #iti_ok
            if(num < nb_iti):
                if(iti_step[num] == ITI_STEP_ON_TP):
                    return 1
                elif(iti_step[num] == ITI_STEP_PROX):
                    return 1
                elif(iti_step[num] == ITI_STEP_DET_DEB):
                    return 1
                elif(iti_step[num] == ITI_STEP_DET_FIN):
                    return 1
                elif(iti_step[num] == ITI_STEP_T_FIN):
                    return 1
                else:
                    return 0                
        if(c == "u"):  #iti_ouv
            if(num < nb_iti):
                if(iti_step[num] == ITI_STEP_ON_TP):
                    return 1
                if(iti_step[num] == ITI_STEP_DET_DEB):
                    return 1
                else:
                    return 0                
        if(c == "w"):  #iti_voy
            if(num < nb_iti):
                if(iti_step[num] == ITI_STEP_MEM and iti_cli):
                    return 1
                elif(iti_step[num] == ITI_STEP_PREP and iti_cli):
                    return 1
                elif(iti_step[num] == ITI_STEP_ON_TP):
                    return 1
                elif(iti_step[num] == ITI_STEP_PROX):
                    return 1
                elif(iti_step[num] == ITI_STEP_DET_DEB):
                    return 1
                elif(iti_step[num] == ITI_STEP_DET_FIN):
                    return 1
                elif(iti_step[num] == ITI_STEP_T_FIN):
                    return 1
                else:
                    return 0
        if(c == "z"):
            if(num < zone_nb):
                return var2.zone[num]
        if(c == "r"):
            if(num < res_nb):
                return var[board].res[num]
        if(c == "f"):
            if(num < cf_nb):
                return var2.cf[num]
        if(c == "s"):
            if(num < fx_nb):
                return var[board].fx[num]
                    
    else:               
        c = szVar[:3]       #var3==54 --> var
        
        dec = szVar.split("==")   #var3==54 --> var3 54
        if(len(dec) == 2):
            num = int(dec[0][3:])
            val = int(dec[1])
            if(get_val_from_var(c, num) == val):
                return 1
            else:
                return 0
        dec = szVar.split("<>")
        if(len(dec) == 2):
            num = int(dec[0][3:])
            val = int(dec[1])
            if(get_val_from_var(c, num) != val):
                return 1
            else:
                return 0
        dec = szVar.split(">=")
        if(len(dec) == 2):
            num = int(dec[0][3:])
            val = int(dec[1])
            if(get_val_from_var(c, num) >= val):
                return 1
            else:
                return 0
        dec = szVar.split("<=")
        if(len(dec) == 2):
            num = int(dec[0][3:])
            val = int(dec[1])
            if(get_val_from_var(c, num) <= val):
                return 1
            else:
                return 0
        dec = szVar.split(">")
        if(len(dec) == 2):
            num = int(dec[0][3:])
            val = int(dec[1])
            if(get_val_from_var(c, num) > val):
                return 1
            else:
                return 0
        dec = szVar.split("<")
        if(len(dec) == 2):
            num = int(dec[0][3:])
            val = int(dec[1])
            if(get_val_from_var(c, num) < val):
                return 1
            else:
                return 0
            
    print("ERROR tdv, invalid variable " + szVar)
    #quit()
    return 0

    
    
def tdv_tic():
    acc = 0
    for cpt in range(len(tdv_file_instr)):
        order = tdv_file_instr[cpt]
        szVar = tdv_file_var[cpt]
        if(order == "l"):
            acc = tdv_get(szVar)
        if(order == "ln"):
            acc = 1 - tdv_get(szVar)
        if(order == "a"):
            acc &= tdv_get(szVar)
        if(order == "an"):
            acc &= (1 - tdv_get(szVar))
        if(order == "o"):
            acc |= tdv_get(szVar)
        if(order == "on"):
            acc |= (1 - tdv_get(szVar))
        if(order == "x"):
            acc ^= tdv_get(szVar)
        if(order == "xn"):
            acc ^= (1 - tdv_get(szVar))
        if(order == "="):
            tdv_set(szVar,acc)
        if(order == "=n"):
            tdv_set(szVar,1-acc)
        if(order == "=s" and acc):
            tdv_set(szVar,1)
        if(order == "=c" and acc):
            tdv_set(szVar,0)
        ##print("TDV: order=" + order + " var=" + szVar + " val=" + str(acc))

        

#---------------------------------------------------------------------------------------
# SIG
#---------------------------------------------------------------------------------------
        
# fichier de configuration de la signalisation
#
# Il est possible de gerer la signalisation avec les equations mais cela peut vite devenir 
# fastidieux. Ce fichier permet de gerer des signalisations complexes facilement
#
# Chaque nouveau signal se definit par s<0-199>:  (ne pas oublier les :)
# Pour le systeme, chaque signal doit être dans un des etats suivants:
# - C = Carre         = franchissement interdit (aucun itineraire ou aiguillages mal positiones
#                                                ou commutateur de fermeture ou bmvu occupe)
# - S = Semaphore     = canton suivant occupe(indique par les entrees i ou l'etat des cantons j)
# - A = Avertissement = le signal suivant est C ou S
# - L = voie Libre    = le signal suivant n'est pas C ou S
# - M = Manoeuvre     = un itinéraire de manœuvre est actif 
#
# Il faut donc indiquer pour chaque "chemin", l'itineraire ou la position des aiguillages,
# les detecteurs ou le canton et le signal suivant ou le bmvu
# Pour un heurtoir, remplacer le signal suivant par stop
# Si le signal, ne gere pas l'avertissement, ex S de BAPR, omettre le signal suivant
# S’il n'y a pas d'itineraire, mettre itix dans tous les cas (meme avec des aiguillages ou bmvu)
# Il est possible de rajouter des aiguillage apres un iti<0-999> ou un itix
# Pour un itineraire de manœuvre, rajouter m, il n'y aura donc que les valeurs C(pour le Cv) ou M
#
# Il faudra definir pour chaque etat C/S/A/L/M, l'etat de toutes les leds en signalisation
# lumineuse ou la position des servomoteurs en signalisation mecanique.
# Si vous souhaitez juste specifier les etats actifs des leds, alors rajouter l'etat D (defaut) 
# en premier pour eteindre toutes les leds avant d’activer les actives dans les autres etats.
# Les signaux peuvent être reels (implantes sur le reseau) ou virtuels (non presents)
# Les signaux peuvent servir a la conduite automatique des trains.
#
# Pour un signal qui accepte le v200 (vert clignotant pour reduire a 160km/h), rajouter v200
# l'etat LCLI est alors ajoute
#
# Si vous souhaitez utilsier le rappel de ralentissement RR et RRCLI pour les aiguilles en pointes
# rajouter rr ou rrcli dans chaque chemin
# Si vous souhaiter avertir un RR ou RRCLI par un avertissement, ajouter ra dans le chemin
# Si vous souhaiter avertir un RR ou RRCLI par un ralentissement R ou RCLI, ajouter r dans le chemin
# Les etats suivant apparaissent RR0,RR,RRCLI,R,RCLI (et peuvent etre en // de l'etat A)
# Il faudra definir les leds a allumer et la position des servos pour ces etats 
#
# Vous pouvez rajouter un indicateur de direction en rajoutant dir<0-4>, 0-4 indique le nb de lampes a activer
#
# syntax:
# s<0-199>:
# iti<0-999>/itix a<0-95*>[-,/] i<0-255*>/j<0-99> s<0-199>/stop m rr/rrcli ra/r dir<0-4> v200     / ?? bmvu<0-9>[a,b]
# D/C/S/A/L/LCLI: l/c<0-255*>[+/-] srv<0-47*>=<50,255>
# R0/RR/RRCLI/R/RCLI: "
# D0/D1/D2/D3/D4:     "
# * = ajouter 0 pr la board 0, 1000 pr la board 1 ... 
#
# ex:
# # C101
# s1:
# iti1 j1 s11
# iti2 j2 s12
# C: l0+ l1+ l2- l3-
# S: l0- l1+ l2- l3-
# A: l0- l1- l2+ l3-
# L: l0- l1- l2- l3+
#
# meme ex:
# # C101
# s1:
# iti1 j1 s11
# iti2 j2 s12
# D: l0- l1- l2- l3-
# C: l0+ l1+
# S: l1+
# A: l2+
# L: l3+

sig_nb = 200  # s0 - s199

sig_file_lines = []

sig_state = [ 0 for i in range(sig_nb)]     # attention c'est un mask !
SIG_C      = 1
SIG_S      = 2
SIG_A      = 4
SIG_L      = 8
SIG_LCLI   = 16   # si v200
SIG_M      = 32   # si m
SIG_RR     = 64   # si rr
SIG_RRCLI  = 128  #
SIG_R      = 256  # si r
SIG_RCLI   = 512  #  

sig_dir = [ 0 for i in range(sig_nb)]   # 0=0lampe, 1=1lampe=gauche, ...


with open("config_signalisation.txt", mode = "r", encoding = "utf-8", errors = 'ignore') as my_file:   #ignore remove non utf-8 chars
    print("SIG: loading file")    
    for line in my_file:
        line2 = line.rstrip("\n")
        #print(line2)
        if(line2 == ""):
            continue
        if(line2[0] == "#"):
            continue
        elems = []
        astr = line2.split()    #separateur blanc(s) ou tab
        l = len(astr)
        for i in range(l):
            if(astr[i][0] == '#'):
                break
            elems.append(astr[i])
        if(len(elems)):
             sig_file_lines.append(elems)    

    print("SIG: file loaded")
    print(sig_file_lines)   #uu: enlever

    
def sig_tic():
    global sig_state
    global sig_dir

    sig_active_lines = [ 0 for i in range(sig_nb)]

    # Try to find active line for each signals
    sig = 0
    actif = 0
    nb = len(sig_file_lines)
    for i in range(nb):
        actif = 0
        elems = sig_file_lines[i]
        for j in range(len(elems)):
            sz = elems[j]
            if(sz[0]=="s" and sz[-1] == ":"):
                sig = int(sz[1:-1])
            elif(sz[0:4] == "itix"):
                 actif = 1
            elif(sz[0:3] == "iti"):
                num = int(sz[3:])
                if(iti_step[num] == ITI_STEP_ON_TP):
                    actif = 1
                if(iti_step[num] == ITI_STEP_DET_DEB):
                    actif = 1
            elif(sz[0] == "a"):
                num = int(sz[1:-1]); board = num // 1000; num = num % 1000
                if(sz[-1] == "-" and rs232[board].aig_dir[num] == 0):
                    actif = 0    
                if(sz[-1] == "/" and rs232[board].aig_dev[num] == 0):
                    actif = 0    
            elif(sz[0:3] == "bmvu"):
                pass
                # si bmvu pas actif, alors
                #   actif = 0    
            
        if(actif):
            sig_active_lines[sig] = elems
    #print("active line = " + str(sig_active_lines))   #uu: enlever

    # Initialisations
    # et calcul de l'etat C
    for sig in range(sig_nb):
        # defaut
        sig_state[sig]    = 0
        sig_dir[sig]      = 0
        
        # etat C: commutateur de fermeture ou aucune ligne active
        if(var2.cf[sig] or sig_active_lines[sig] == 0):
            sig_state[sig] |= SIG_C
        
    # 1ere analyse des lignes actives
    # + evaluation des etats S, RR, RRCLI, M
    # + evaluation de la direction
    for sig in range(sig_nb):
        if(sig_state[sig] == 0):
            elems = sig_active_lines[sig]
            for j in range(len(elems)):
                sz = elems[j]
                #print("sz = " + sz)
                # etat S
                if(sz[0] == "j" or sz[0] == "i"):
                    if(sz[1] >= "0" and sz[1] <= "9"):
                        occ = tdv_get(sz)
                        #print("ask for " + sz + " result " + str(occ))                      
                        if(occ):
                            sig_state[sig] |= SIG_S
                            
                # RR et RRCLI                      
                if(sz == "rr"):
                    sig_state[sig] |= SIG_RR
                if(sz == "rrcli"):
                    sig_state[sig] |= SIG_RRCLI

                # etat M (manoeuvre autorisee)          
                if(sz == "m"):
                    sig_state[sig] |= SIG_M
                    
                # indicateur de direction geographique
                if(sz[0:3] == "dir"):
                    sig_dir[sig] = int(sz[3])
                    
            if(sig_state[sig] & SIG_S):
                sig_state[sig] = SIG_S   #efface M, RR, RCLI
                sig_dir[sig]   = 0
                                    
    # 2eme analyse des lignes actives
    # + evaluation des etats A,R,RCLI (maintenant que l'on connait C,S,M,RR,RRCLI)
    for sig in range(sig_nb):
        if((sig_state[sig] & (SIG_C | SIG_S | SIG_M)) == 0):
            ra = 0; r = 0
            elems = sig_active_lines[sig]
            for j in range(len(elems)):
                sz = elems[j]
                if(sz == "stop"):
                    sig_state[sig] |= SIG_A
                elif(sz == "ra"):
                    ra = 1
                elif(sz == "r"):
                    r = 1
                elif(sz[0] == "s"):
                    s_next = int(sz[1:])
                    #print("snext=" + str(s_next) + " state= " + str(sig_state[s_next]))
                    
                    if(sig_state[s_next] & (SIG_C | SIG_S | SIG_M)):
                        sig_state[sig] |= SIG_A
                        
                    elif(sig_state[s_next] & (SIG_RR | SIG_RRCLI)): #si on a RR(CLI) le feu n'est pas en C,S,M 
                        if(ra):
                            sig_state[sig] |= SIG_A
                        if(r):
                            if((sig_state[sig] & (SIG_RR | SIG_RRCLI)) == 0):
                                if(sig_state[s_next] & SIG_RR):
                                    sig_state[sig] |= SIG_R
                                if(sig_state[s_next] & SIG_RRCLI):
                                    sig_state[sig] |= SIG_RCLI
                            else:
                                sig_state[sig] |= SIG_A
                                       
    # + evaluation etat LCLI                     
    for sig in range(sig_nb):
        if(sig_state[sig] == 0):
            v200 = 0
            elems = sig_active_lines[sig]
            for j in range(len(elems)):
                sz = elems[j]
                if(sz == "v200"):
                    v200 = 1
                elif(v200 and sz[0] == "s"):
                    s_next = int(sz[1:])
                    if(sig_state[s_next]):
                        sig_state[sig] |= SIG_LCLI

    # + evaluation etat L (si pas C,S,A)                     
    for sig in range(sig_nb):
        if(sig_state[sig] == 0):
            sig_state[sig] |= SIG_L
                                    
    dbg = "sig_state "
    for sig in range(10):     
        dbg += str(sig_state[sig]) + " "
    print(dbg)
        
    # mise a jour des leds et servos 
    sig = 0
    for i in range(len(sig_file_lines)):
        elems = sig_file_lines[i]
        actif = 0
        for j in range(len(elems)):
            sz = elems[j]
            if(sz[0]=="s" and sz[-1] == ":"):
                sig = int(sz[1:-1])
                break
            
            if(sz == "D:"):
                actif = 1; continue
            elif(sz == "C:"):
                if(sig_state[sig] & SIG_C):
                    actif = 1; continue
            elif(sz == "S:"):
                if(sig_state[sig] & SIG_S):
                    actif = 1; continue
            elif(sz == "A:"):
                if(sig_state[sig] & SIG_A):
                    actif = 1; continue
            elif(sz == "L:"):
                if(sig_state[sig] & SIG_L):
                    actif = 1; continue             
            elif(sz == "LCLI:"):
                if(sig_state[sig] & SIG_LCLI):
                    actif = 1; continue             
            elif(sz == "M:"):
                if(sig_state[sig] & SIG_M):
                    actif = 1; continue             
                    
            elif(sz == "R0:"):
                if(sig_state[sig] & (SIG_RR | SIG_RRCLI | SIG_R | SIG_RCLI) == 0):
                    actif = 1; continue             
            elif(sz == "RR:"):
                if(sig_state[sig] & SIG_RR):
                    actif = 1; continue             
            elif(sz == "RRCLI:"):
                if(sig_state[sig] & SIG_RRCLI):
                    actif = 1; continue             
            elif(sz == "R:"):
                if(sig_state[sig] & SIG_R):
                    actif = 1; continue             
            elif(sz == "RCLI:"):
                if(sig_state[sig] & SIG_RCLI):
                    actif = 1; continue             

            elif(sz == "D0:"):
                if(sig_dir[sig] == 0):
                    actif = 1; continue             
            elif(sz == "D1:"):
                if(sig_dir[sig] == 1):
                    actif = 1; continue             
            elif(sz == "D2:"):
                if(sig_dir[sig] == 2):
                    actif = 1; continue             
            elif(sz == "D3:"):
                if(sig_dir[sig] == 3):
                    actif = 1; continue             
            elif(sz == "D4:"):
                if(sig_dir[sig] == 4):
                    actif = 1; continue             
                    
            if(actif):
                if(sz[0:3]=="srv"):
                    tdv_set(sz, 1)
                if(sz[-1] == "+" or sz[-1] == "-"): 
                    num = int(sz[1:-1])
                    if(sz[-1] == "+"):
                        val = 1
                    else:
                        val = 0             
                    tdv_set(sz[0:-1], val)
                    #print("set " + sz[0:-1] + " to " + str(val))

    
        
#---------------------------------------------------------------------------------------
# ITI
#---------------------------------------------------------------------------------------            

ITI_STEP_OFF     = 0
ITI_STEP_MEM     = 1
ITI_STEP_PREP    = 2
ITI_STEP_ON_TP   = 3
ITI_STEP_PROX    = 4
ITI_STEP_DET_DEB = 5
ITI_STEP_DET_FIN = 6
ITI_STEP_T_FIN   = 7
ITI_STEP_DE_RES  = 8 #etat transitoire (ne devrait pas etre vu en dehors de iti_tic)

nb_iti = 1000
nb_iti_loaded = 0
iti_step = [ 0 for i in range(nb_iti)]
iti_tempo = [ 0 for i in range(nb_iti)]
iti_db = [ 0 for i in range(nb_iti)]
iti_cli = 0
iti_cpt = 0
for i in range(nb_iti):
    sl = []
    iti_db[i] = sl
    sl = 0

with open("config_iti.txt", mode = "r", encoding = "utf-8", errors = 'ignore') as my_file:   #ignore remove non utf-8 chars
    print("ITI: loading file")    
    for line in my_file:
        line2 = line.rstrip("\n")
        if(line2 == ""):
            continue
        if(line2[0] == "#"):
            continue
        astr = line2.split()    #separateur blanc(s) ou tab
        l = len(astr)
        if(astr[0][:3] != "iti"):
            print("ITI: Error " + line2)
            quit()
        iti_num = int(astr[0][3:])
        for i in range(1, l):
            if(astr[i][0] == "#"):
                break
            iti_db[iti_num].append(astr[i])
        nb_iti_loaded += 1
    print("ITI: file loaded with " + str(nb_iti_loaded) + " itineraires")

def iti_tic():
    global iti_step
    global var
    global var2
    global rs232
    global iti_cpt
    global iti_cli
    
    #print("iti_step[20]=" + str(iti_step[20]))
    
    #gestion du clignotement pour les voyants
    iti_cpt += 1
    if(iti_cpt == 2):
        iti_cpt = 0
        iti_cli ^= 1
    
    #state machine des itineraires
    for iti in range(nb_iti):
    
        if(iti_step[iti] == ITI_STEP_OFF):
            if(var2.iti_cmd[iti] == 0):  #pour gagner du temps car cas le plus probable !
                continue 
            iti_step[iti] = ITI_STEP_MEM          
                
        if(iti_step[iti] == ITI_STEP_MEM):
            if(var2.iti_cmd[iti] == 0):
                iti_step[iti] = ITI_STEP_OFF
                continue
            #check aig et zones libres (+ detection deb et fin pour DA)
            ok = 1
            l = len(iti_db[iti])
            for i in range(1, l):
                s = iti_db[iti][i]
                if(s[0] == "a"):
                    num = int(s[1:-1]); board = num // 1000; num = num % 1000
                    if(var[board].res[num]):
                        ok = 0
                        break
                if(s[0] == "z"):
                    num = int(s[1:])
                    if(var2.zone[num]):
                        ok = 0
                        break 
                if(s[0:4] == "deb:"):
                    variable = s[4:]
                    if(tdv_get(variable)):                   
                        ok = 0
                        break 
                if(s[0:4] == "fin:"):
                    variable = s[4:]
                    if(tdv_get(variable)):                   
                        ok = 0
                        break 
            if(ok == 0):
                continue
            #reservation et positionement des aiguillages
            iti_step[iti] = ITI_STEP_PREP
            for i in range(1,l):
                s = iti_db[iti][i]
                if(s[0] == "a"):
                    num = int(s[1:-1]); board = num // 1000; num = num % 1000
                    var[board].res[num] = 1
                    pos = s[-1]
                    if(pos == "-"):
                        var[board].aig_cmd[num] = 0
                        #print("***** set aig" + str(num) + " to direct")
                    if(pos == "/"):
                        var[board].aig_cmd[num] = 1
                        #print("***** set aig" + str(num) + " to devie")
                if(s[0] == "z"):
                    num = int(s[1:])
                    var2.zone[num] = 1           
                
        if(iti_step[iti] == ITI_STEP_PREP):
            #check aig in good position
            ok = 1          
            l = len(iti_db[iti])
            for i in range(1, l):
                s = iti_db[iti][i]
                if(s[0] == "a"):
                    num = int(s[1:-1]); board = num // 1000; num = num % 1000
                    pos = s[-1]
                    if(pos == "-" and rs232[board].aig_dir[num] == 0):
                        ok = 0
                        break
                    if(pos == "/" and rs232[board].aig_dev[num] == 0):
                        ok = 0
                        break
            if(ok == 0):
                continue
            if(iti_db[iti][0] == "tp"):
                iti_step[iti] = ITI_STEP_ON_TP
            if(iti_db[iti][0] == "da"):
                iti_step[iti] = ITI_STEP_PROX

        if(iti_step[iti] == ITI_STEP_ON_TP):
            if(var2.iti_cmd[iti] == 1):
                continue
            iti_step[iti] = ITI_STEP_DE_RES
                        
        if(iti_step[iti] == ITI_STEP_PROX):
            #search prox
            find = 0
            l = len(iti_db[iti])
            for i in range(1, l):
                s = iti_db[iti][i]
                if(s[0:5] == "prox:"):
                    find = 1
                    variable = s[5:]
                    if(tdv_get(variable)):
                        iti_step[iti] = ITI_STEP_DET_DEB
            if(find == 0):
                 iti_step[iti] = ITI_STEP_DET_DEB
            if(var2.iti_cmd[iti] == 0):
                iti_step[iti] = ITI_STEP_DE_RES

        if(iti_step[iti] == ITI_STEP_DET_DEB):
            #search det deb
            find = 0
            l = len(iti_db[iti])
            for i in range(1, l):
                s = iti_db[iti][i]
                if(s[0:4] == "deb:"):
                    find = 1
                    variable = s[4:]
                    if(tdv_get(variable)):
                        iti_step[iti] = ITI_STEP_DET_FIN
            if(find == 0):
                 iti_step[iti] = ITI_STEP_DET_FIN          
            if(var2.iti_cmd[iti] == 0):
                iti_step[iti] = ITI_STEP_DE_RES

        if(iti_step[iti] == ITI_STEP_DET_FIN):
            #search det fin
            find = 0
            l = len(iti_db[iti])
            for i in range(1, l):
                s = iti_db[iti][i]
                if(s[0:4] == "fin:"):
                    find = 1
                    variable = s[4:]
                    if(tdv_get(variable)):
                        iti_step[iti] = ITI_STEP_T_FIN
                        iti_tempo[iti] = -1
            if(find == 0):
                 iti_step[iti] = ITI_STEP_T_FIN            
                 iti_tempo[iti] = -1

        if(iti_step[iti] == ITI_STEP_T_FIN):
            if(iti_tempo[iti] == -1):
                #search lib
                find = 0
                l = len(iti_db[iti])
                for i in range(1, l):
                    s = iti_db[iti][i]
                    if(s[0:5] == "tlib:"):
                        find = 1
                        iti_tempo[iti] = 4 * int(s[5:-1])   #tlib:10s
                        find = 1
                if(find == 0):        
                    iti_tempo[iti] = 0
            if(iti_tempo[iti] == 0):
                iti_step[iti] = ITI_STEP_DE_RES
                var2.iti_cmd[iti] = 0
            if(iti_tempo[iti] > 0):
                iti_tempo[iti] -= 1
                                
        if(iti_step[iti] == ITI_STEP_DE_RES):
            iti_step[iti] = ITI_STEP_OFF
            #dereservation et positionement des aiguillages
            l = len(iti_db[iti])
            for i in range(1, l):
                s = iti_db[iti][i]
                if(s[0] == "a"):
                    num = int(s[1:-1]) ; board = num // 1000; num = num % 1000
                    var[board].res[num] = 0
                if(s[0] == "z"):
                    num = int(s[1:])
                    var2.zone[num] = 0

    return



#---------------------------------------------------------------------------------------
# Cantons (suivi des trains)
#---------------------------------------------------------------------------------------      

# config_cantons.txt:
#
#   Ce fichier decrit les cantons afin de suivre les trains et gerer l'occupation des cantons
#   
#   Syntax:
#   c<0-99>[+/-]: canton destination (avec :) 
#                     +/- indique le sens du canton destination pour cette ligne
#                         (si pas specifie, sens + par defaut)
#                     cette information est utilisee en cac (conduite automatique en mode cantonnement)              
#
#   c<0-99>       canton source (sans :)
#
#   a<0-_95><-/>  position d'un aiguillage (on peut creer un itineraire avec plusieurs aiguillages)
#
#   i<0-_255>     detecteurs
#                 - le permier indique le detecteur d'entree
#                 - le second  indique le detecteur de fin
#                      s’il n’y en a qu’un seul, il sert de début et de fin
#
#   t<0-59>s      tempo qui retarde la fin apres l’activation du detecteur de fin s’il existe
#                 ou qui simule une fin apres l’activation de l'unique capteur d’entrée
#
#   l<1-50>       assigner la loco au demarrage
#
#   s<0-199>      numero du signal du cote +/- du canton specifie precedement
#
# exemple d’une boucle a 3 cantons a un seul sens
# c0: c2 i10 i0
# c1: c0 i11 i1
# c2: c1 i12 i2
# 
# exemple avec loco50 sur c0 a l'init, canton fin apres 5s sur canton1, un seul capteur de fin pr canton2
# c0: c2 i10 i0 l50
# c1: c0 i11 i1 t5s
# c2: c1 i2
#
# exemple avec sens et signaux pour fonctionnement en conduite automatique en mode cantonement (cac)
# c0+: c2 i10 i0 s0
# c1+: c0 i11 i1 s1
# c2+: c1 i12 i2 s2

cantons_nb = 100

CANTON_FREE    = 0
CANTON_DEB_OK  = 1
CANTON_END_OK  = 2
CANTON_WAIT_OK = 4
CANTON_ERR     = 10   

class c_canton:
    def __init__(self):
        self.db      = []
        self.det     = []
        
        self.state   = CANTON_FREE
        self.train   = 0   # 0 = pas de train
        self.occ     = 0
        self.dcpt    = 0
        self.deb_det = ""
        
        self.s_p     = -1  # num signal cote +
        self.s_m     = -1  # num signal cote -
        self.sens    = "+"
        
        #self.lock    = 0
        #self.res     = 0
cantons = []
for i in range(cantons_nb):
    cantons.append(c_canton())

def cantons_make_txt(c):  #uu: rajouter t en variable menbre et mettre a jour pendant les transitions
    t = "???"
    if(cantons[c].state == CANTON_FREE):
        t = "---"
    if(cantons[c].state == CANTON_ERR):
        t = "ERR"
    if(cantons[c].state == CANTON_DEB_OK):
        t = ">" + str(cantons[c].train)
    if(cantons[c].state == CANTON_END_OK):
        t = "T" + str(cantons[c].train)
    if(cantons[c].state == CANTON_WAIT_OK):
        t = str(cantons[c].train)
    return "c" + str(c) + ":" + t

def cantons_make_report():
    t = ""
    for c in range(cantons_nb):
        if(cantons[c].state == CANTON_FREE):
            continue
        t += cantons_make_txt(c) + "\n"
    return t
    
def cantons_free(num):
    cantons[num].state = CANTON_FREE
    cantons[num].train = 0
    cantons[num].occ   = 0

def cantons_assign_train(num, train):
    cantons[num].state = CANTON_WAIT_OK
    cantons[num].train = train
    cantons[num].occ   = 1
    
def cantons_is_train(num, train):
    if(cantons[num].train == train):
        if(cantons[num].state == CANTON_WAIT_OK):
            return 1
    return 0
    
#cantons_assign_train(2, 28)    
# c2: c1 a1- a2/ i1 i21 t10s    
with open("config_cantons.txt", mode = "r", encoding = "utf-8", errors = 'ignore') as my_file: #ignore remove non utf-8 chars

    print("CANTONS: loading file")    
    
    c = 0
    nb_cantons_loaded = 0    
    c_src_ok = 0
    
    for line in my_file:
    
        line2 = line.rstrip("\n")
        if(line2 == ""):
            continue
            
        if(line2[0] == "#"):
            continue
            
        astr = line2.split() #separateur blanc(s) ou tab
        l = len(astr)
        for i in range(0, l):
            s = astr[i]
            
            if(s[0] == "#"):
                break
                
            if(s[-1] == ":"):
                if(s[-2] == "+"):
                    sens = "+"
                    c = int(s[1:-2])
                elif(s[-2] == "-"):
                    sens = "-"
                    c = int(s[1:-2])
                else:
                    sens = "+"
                    c = int(s[1:-1])                
                if(c < 0 or c > cantons_nb):            
                    print("CANTONS: Error (c<0-99>:) " + line2)
                    quit()
                nb_cantons_loaded += 1
                c_src_ok = 0
                continue
                    
            if(s[0] == "l"):
                l = int(s[1:])
                cantons_assign_train(c, l)
                continue #ps besoin de stoquer cette info
                
            if(s[0] == "s"):
                if(sens == "+"):
                    cantons[c].s_p = int(s[1:])
                if(sens == "-"):
                    cantons[c].s_m = int(s[1:])                   
                continue #ps besoin de stoquer cette info
                
            cantons[c].db.append(s)  # on stoque toutes les autres infos
    
            if(s[0] == "c"):   # on rajoute le sens du canton pour ce chemin
                 cantons[c].db.append(sens)
    
            if(s[0] == "i"): # on stoque les detecteurs en plus a part pour optimisation
                already = 0
                d = len(cantons[c].det)
                for j in range(d):
                    if(cantons[c].det[j] == s):
                        already = 1
                if(already == 0):
                    cantons[c].det.append(s)
                    
    print("CANTONS: file loaded with " + str(nb_cantons_loaded) + " cantons")
    
    for c in range(cantons_nb):
        if(len(cantons[c].db)):
            print("canton" + str(c))
            print("- det=" + str(cantons[c].det))
            print("-  db=" + str(cantons[c].db))
            print("-sig+=" + str(cantons[c].s_p))
            print("-sig-=" + str(cantons[c].s_m))

            
def cantons_find_previous(c_curr, det):
    ok = 0
    c_prev = -1
    nb_det = 0
    tempo = 0
    sens = '+'
    
    l = len(cantons[c_curr].db)
    
    for i in range(l):
    
        s = cantons[c_curr].db[i]
        
        if(s[0] == 'c'):
            if(ok):
                break
            ok = 1
            c_prev = int(s[1:])
            first_det = 1
            nb_det = 0
            tempo = 0
            sens = "+"
            
        if(s[0] == 'a'):
            aig = int(s[1:-1]);  board = aig // 1000; aig = aig % 1000
            pos = s[-1]
            if(pos == "-" and rs232[board].aig_dir[aig] == 0):
                ok = 0        
            if(pos == "/" and rs232[board].aig_dev[aig] == 0):
                ok = 0        
                
        if(s[0] == 'i'):
            nb_det +=1
            if(first_det):
                first_det = 0
                if(det != s):
                    ok=0

        if(s[0] == 't'):
            tempo = int(s[1:-1])

        if(s == '+'):
            sens = '+'
        if(s == '-'):
            sens = '-'
            
    if(ok):
        return c_prev, nb_det, tempo, sens
    else:
        return -1, 0, 0, '+'    
 
def cantons_maj():
    for c in range(cantons_nb):
    
        # FREE -> DEB (occupation du canton courant)    
        if(cantons[c].state == CANTON_FREE):
            for d in range(len(cantons[c].det)):
            
                sz_det = cantons[c].det[d]
                if(tdv_get(sz_det) == 0): # ou utiliser rs232[board].inp[num] 
                    continue              
                    
                print("CANTONS: c" + str(c) + " det_deb=" + sz_det)
                cantons[c].deb_det = sz_det   # si un capteur est actif, c'est forcement le premier car on est en FREE
                
                prev, nb_det, tempo, sens = cantons_find_previous(c, sz_det)
                
                if(prev == -1):
                    cantons[c].state = CANTON_ERR
                    cantons[c].train = 0
                    cantons[c].occ   = 1
                    print("CANTONS: c" + str(c) + " ERR = NO PREV")
                    
                elif(cantons[prev].train == 0):
                    cantons[c].state = CANTON_ERR
                    cantons[c].train = 0
                    cantons[c].occ   = 1
                    print("CANTONS: c" + str(c) + " ERR = NO TR")
                    
                else:
                    cantons[c].state = CANTON_DEB_OK
                    cantons[c].train = cantons[prev].train
                    cantons[c].occ   = 1
                    cantons[c].dcpt = tempo * 4
                    cantons[c].sens = sens                    
                    print("CANTONS: c" + str(c) + " STATE = DEB_OK, TRAIN =" + str(cantons[c].train))
                    
                    if(nb_det == 1):  #Pas d'autre detecteur, on va a end directement
                        cantons[c].state = CANTON_END_OK
                        
    # DEB -> END
    for c in range(cantons_nb):                   
        if(cantons[c].state == CANTON_DEB_OK):
            for d in range(len(cantons[c].det)):
                sz_det = cantons[c].det[d]
                
                if(sz_det == cantons[c].deb_det):
                    continue
                    
                if(tdv_get(sz_det)==0):
                    continue
                    
                cantons[c].state = CANTON_END_OK
                print("CANTONS: c" + str(c) + " det_end=" + sz_det)
                print("CANTONS: c" + str(c) + " STATE = END_OK")
                
    # END -> WAIT_OK (+liberation du canton precedent)              
    for c in range(cantons_nb):
        if(cantons[c].state == CANTON_END_OK):
        
            if(cantons[c].dcpt):
                cantons[c].dcpt -= 1
                continue
                
            cantons[c].state = CANTON_WAIT_OK
            print("CANTONS: c" + str(c) + " STATE = WAIT_OK")
           
            # WAIT_OK -> FREE          
            for c2 in range(cantons_nb):
                if(c == c2):
                    continue  #liberation de tous les autres cantons avec le meme train
                    
                if(cantons[c2].train == cantons[c].train):
                    cantons[c2].state = CANTON_FREE
                    cantons[c2].train = 0
                    cantons[c2].occ   = 0
                    print("CANTONS: c" + str(c2) + " STATE = FREE")
                        
    return

    

#---------------------------------------------------------------------------------------
# CAC : Conduite Automatique en mode Cantonement
#---------------------------------------------------------------------------------------      

cac_on     = [ 0 for i in range(51) ]
cac_on_old = [ 0 for i in range(51) ]
cac_side   = [ "+" for i in range(51) ]  # cote du canton indique sur le bouton

#CAC_MAX  = 28   #definis au debut maintenant
#CAC_RAL  = 8
#CAC_STOP = 0

def cac_maj():
    global rs232_loc
    global cantons
    global cac_on; global cac_on_old
    
    for t in range(1,51):    # loco 1-50
        if(cac_on[t] == 0):
            cac_on_old[t] = 0
            continue
        # chercher le canton de la tete du train
        c = -1
        for ct in range(cantons_nb):
            if(cantons[ct].train == t):
                c = ct
                if(cantons[c].state != CANTON_WAIT_OK):   # la tete de train est sur DEB_OK/END_OK/WAIT_OK et la queue eventuellement sur WAIT_OK du canton precedent
                    break
        if(c == -1):
            print("CAC: ERR pas de canton pour le train" + str(t))
            rs232_loc.vit[t] = 0
            cac_on[t] = 0; cac_on_old[t] = 0;
            continue
            
        # a l'initialisation on doit indiquer de quel cote se dirige le train   
        if(cac_on_old[t] == 0):
            cac_on_old[t] = 1
            cantons[c].sens = cac_side[t]
            
        # on a le train t sous conduite cac et le canton c
        # on sait aussi si le train est en "enter" DEB_OK/END_OK ou "in" WAIT_OK
        
        # trouver le signal en fin de canton et du bon cote pour adapter la vitesst
        if(cantons[c].sens):
            s = cantons[c].s_p
        else:
            s = cantons[c].s_m
        
        if(s == -1):
            state = SIG_C            # carre si aucun signal
        else:
            state = sig_state[s]     # attention c'est un mask !
            
        if(state & (SIG_C | SIG_S | SIG_M)):  # ralentissement puis arret au signal
            v_deb = CAC_RAL
            v_fin = CAC_STOP        
        elif(state & (SIG_RR | SIG_RRCLI)):   # ralentissement puis passage au ralentit au signal
            v_deb = CAC_RAL
            v_fin = CAC_RAL     
        else:   # SIG_A | SIG_L | SIG_LCLI | SIG_R | SIG_RCLI   # aucun ralentissement
            v_deb = CAC_MAX
            v_fin = CAC_MAX
        if(cantons[c].state == CANTON_WAIT_OK):
            v = v_fin
        else:
            v = v_deb
        rs232_loc.vit[t] = v  

        print("t" + str(t) + " c" + str(c) + " s" + str(s) + " v" + str(v))     
            
    return


    
#---------------------------------------------------------------------------------------
# Scripts
#---------------------------------------------------------------------------------------      

# ------------------------------------------------------------------------------------------------------------
# Syntaxe:
# ------------------------------------------------------------------------------------------------------------
#
# ----commentaires--------------------------------------------------------------------------------------------
#
# # commentaire
#
# ----programmes----------------------------------------------------------------------------------------------
#
# <label>:                              // un label est un repere ds le programme, on peut y sauter
# instructions
#
#    init:                              // au demerrage, le programme commence a s'executer a partir du label init:
#    loco 2
#    bcl:                               // ce label est utilise par la boucle
#    vit +10
#    tempo 1s
#    vit +0
#    tempo 1s
#    goto bcl                           // on saute au label bcl:
#    ...
#
#    Le programme commence a s'executer automatiquement a partir du label init lors du chargement d'un script
#
# run <label>                            // cette instruction demande d'executer le programme indique sans attendre sa fin
#                                        // pour attendre sa fin avant de continuer, faire suivre de wait_end_run
#
#    run depl22:                         // lance le programme depl22 puis continue en //
#
# wait_fin <label>                       // cette instruction attend la fin du programme indique
#
#    wait_fin depl22                     // attend que le programme de debut depl22 se termine
#
# ----loco----------------------------------------------------------------------------------------------------
#
# loco <adr loco>
#
#    loco 23              // selectionne la loco d'adresse 23 (si deja utilisee, attend qu'elle soit libre)
#    loco 0               // 0 libere la loco
#
# vit <+/-><0-31>
#
#    vit +31              // vitesse avant max (cran 31/31) pour la locomotive selectionnee
#    vit -0               // arret (cran 0/31), sens arriere
# 
# fct f<0-4><+/->
#
#    fct f0+ f4-          // active la fonction 0 de la locomotive selectionnee (souvent les phares), desactive la fonction 4
#
# ----aiguillages---------------------------------------------------------------------------------------------
#
# aig a<0-127><-,/>
# 
#     aig a3/ a23- a2-    // positione a2 et a23 en direct et a3 en devie
#                         // si un aiguillage est reserve par un itineraire alors le script attends que l'aiguillage soit libre
#                         // le script continue seulement lorsque les aiguillages ont ete mis dans la bonne position
#                         // il n'est pas (encore?) possible de reserver ou liberer un aiguillage
#                         // uu: voir si on gere les aiguillages comme des variables !
#                         // aig a3/ est equivalent a: wait !r3; set a3; wait e3
#
# ----itineraires---------------------------------------------------------------------------------------------
#
# les itineraires s'utilisent comme des es avec la variable t (de iTineraire)
#
# set t<0-999><+/->
# wait [!]t<0-999>
#
# set t31+  // demande l'etablissement de l'itineraire 31
# wait t31  // attend l'itineraire 31
#
# ----es-variables--------------------------------------------------------------------------------------------
#
# wait [!]i<0-255> [!]d<0-127> [!]e<0-127> [!]v<0-999>  // attend que toutes les conditions soient vraies
#      ana<0-1><==...><0-255> var<0-999><==...><0-255>
#
#     wait i1 !i2 v1      // attend i1 actif, i2 inactif, v1 active
#     wait an0>=127 a var3<>22 a i1
#
# set o<0-63><+/-> l<0-255><+/-> c<0-255><+/-> p<0-255><+/-> v<0-999><+/->  // positione les varaibles
#     pwm<0-51>=<0-255>  srv<0-47>=<50-255> neo<0-59>=#RRVVBB
#
#     set o2+ l3- l4+ l5+ c4+ c5+ p4- p5+ // active out2, eteint led3, fait clignoter led4 et 5 en alternance
#     set pwm3=45 neo28=#FF0000 srv28=124 o5+ //met pwm3 a 45(45/255=18%), le servo28 a 124 (1.24ms),
#                                             //le neopixel 8 au rouge a fond, active la sortie 5
#
# ----temps---------------------------------------------------------------------------------------------------
#
# tempo <temps><t,s> ...  // attend pendant une duree
#
#     tempo 1s            // attend une seconde
#     tempo 2t            // attend 2 tics (un tic=250ms, 2=500ms ...)
#     tempo 1s 10s        // attend aleatoirement entre 1 et 10 secondes
#
# waithm <hh:mm>          // attend l'heure specifiee (sur l'horloge de modelisme)
#
#     waithm 22:30        // attend 22h30 
#
# sethm <hh:mm>           // regle l'heure de l"horloge de modelisme
#
#     sethm 06:00         // regle l'heure a 6h 
#
# ----sauts---------------------------------------------------------------------------------------------------
#
# goto <label:>           // saute au label indique (ne pas oublie les :)
#
#     la_boucle:
#         ...
#         goto la_boucle:
#
# goto <label:> <pourcentage%>  // saute au label indique aleatoirement suivant le pourcentage sinon continue
#
#     la_boucle:
#         ...
#         goto la_boucle: 80%
#         ...
#         goto la_boucle:
#
# ----communication-utilisateur-------------------------------------------------------------------------------
#
# msg "message"           // affiche un message
#
#    msg "deplacement_1"  // affiche ce message ds le champ message du programme dans la fenetre des scripts
#                         // il ne doit pas y avoir de blanc ds le message
#                         // la chaine ne doit jamais etre vide
# 
# nomsg                   // efface le message
#
# sound "fichier.wav"     // joue un son (le script continue sans attendre la fin du son)
#
#    sound "sound/annonce_depart_25.wav"
#
# ----conduite-automatique-en-mode-cantonnement---------------------------------------------------------------
#
# cac_init <loco> <canton>
#    ex: cac_init 12 3
#    demande au systeme d'affecter la loco indiquee sur le canton specifie
#    si la loco est 0 alors le canton est libere
#
# cac_start <loco> <+/->
#    ex: cac_on 12 +
#    demarre la conduite automatique de la loco specifiee en mode cac dans le sens spécifie
#
# cac_stop <loco>
#    ex: cac_stop 12
#    arrete la conduite automatique de la loco specifiee
#    met egalement sa vitesse a 0
#
# cac_wait <loco> <canton>
#    ex: cac_wait 12 8
#    attend que la loco soit completement sur le canton spécifié
#
# ex
#    loco 12
#    vit +0
#    set t28+
#    cac_init 12 3
#    cac_start 12 +
#    cac_wait 12 8
#    cac_stop 12
#
# ------------------------------------------------------------------------------------------------------------

screen_script_on = 0

script_load_allowed = 1
script_filename = ""

nb_prog = 10      #UU: mettre une classe, ne pas limiter le nb, mettre ds un append
script_line = []
script_db   = []

prog_index = [ -1 for i in range(nb_prog)]  #-1 == inactif
prog_timer = [  0 for i in range(nb_prog)]  #0 aucun timer
prog_loco  = [  0 for i in range(nb_prog)]  #0 aucune loco
prog_name  = [ "" for i in range(nb_prog)]
prog_msg   = [ "" for i in range(nb_prog)]  # aucun message

prog_label_name = []
prog_label_internal_index = []


def script_load_cb():
    global script_filename
    global l_script_filename
    global script_load_allowed
    
    if(script_load_allowed):
        script_filename = tkinter.filedialog.askopenfilename(filetypes = [("alle Formate", "*")])
        print("filename=" + script_filename)
        #top.itemconfigure(l_script_filename, text = script_filename)
        l_script_filename.config(text = script_filename)
        b_script_load_off.config(text = "OFF")
        script_load(script_filename)
        script_load_allowed = 0
    else:
        l_script_filename.config(text = "Aucun fichier")
        b_script_load_off.config(text = "LOAD")
        script_unload()
        script_load_allowed = 1
    return
    
   
b_script_load_off = tkinter.Button(top, text = "LOAD" , command = script_load_cb)
l_script_filename = tkinter.Label(top, text = "Aucun fichier")
my_canvas_script = tkinter.Canvas(top, bg = "#5B9BD5", width = 800, height = 600)
l_prog_info = [ 0 for i in range(nb_prog)]
l_prog_msg = [ 0 for i in range(nb_prog)]

#zefont_script = ("Times New Roman", 10, "bold")
zefont_script = ("Helvetica", 10)
for i in range(nb_prog):
    x = 10
    y = 25 + 50 * i
    l_prog_info[i] = my_canvas_script.create_text(x, y, text = "", font = zefont_script, fill = "#000000", anchor = tkinter.W)
    x += 50
    y += 20
    l_prog_msg[i]  = my_canvas_script.create_text(x, y, text = "", font = zefont_script, fill = "#FF0000", anchor = tkinter.W)

def script_on():
    b_script_load_off.grid(row = 0, column = 0, sticky = tkinter.W)
    l_script_filename.grid(row = 0, column = 1, sticky = tkinter.W)
    my_canvas_script.grid(row = 1, column = 0, columnspan = 2, sticky = tkinter.W)
    return

def script_off():
    b_script_load_off.grid_forget()
    l_script_filename.grid_forget()
    my_canvas_script.grid_forget()
    return

script_error = 0
error_msg = ""

def script_load(filename):
    global script_line; global script_db
    global prog_label_internal_index; global prog_label_name
    global prog_index; global prog_timer; global prog_loco; global prog_name; global prog_msg
    global script_error

    script_line = 0; script_line = [];
    script_db = 0; script_db = []    
    prog_label_name = 0; prog_label_name = []
    prog_label_internal_index = 0; prog_label_internal_index = []
    
    with open(filename, mode = "r", encoding = "utf-8", errors = 'ignore') as my_file:   #ignore remove non utf-8 chars
        print("SCRIPT: loading file " + filename)
        internal_index = 0
        num_line = 0
        for line in my_file:
            num_line += 1
            line2 = line.rstrip("\n")
            #print(line2)
            if(line2 == ""):
                continue
            if(line2[0] == "#"):
                continue
            #uu: isoler les commentaire de fin de ligne
            script_db.append(line2)
            script_line.append(num_line)
            #if(line2[:4]=="prog" and line2[-1]==":"):
            #num=int(line2[4:-1])
            if(line2[-1] == ":"):    #termine par :, c'est un label
                prog_label_name.append(line2)
                prog_label_internal_index.append(internal_index)                
            internal_index += 1
    print("SCRIPT: file loaded")
    print("db:" + str(script_db))
    print("line:" + str(script_line))
    print("label_name:" + str(prog_label_name))
    print("label_internal_index:" + str(prog_label_internal_index))
    script_db.append("end"); script_line.append(-1)

    #init sur le label init:
    index = script_get_label_index("init:")
    if(index == -1):
        print("SCRIPT: ERROR: Pas de init:")
        return

    for ch in range(nb_prog):
        prog_index[ch] = -1
        prog_timer[ch] = 0
        prog_loco[ch]  = 0
        prog_name[ch]  = ""
        prog_msg[ch]   = ""

    prog_index[0] = index  #apres l'initialisation, le ch 0 est forcement libre
    prog_name[0]  = "init:"
    
    script_error = 0
    return

def script_unload():
    global prog_index; global prog_timer; global prog_loco; global prog_name; global prog_msg
    global script_error
    for ch in range(nb_prog):
        prog_index[ch] = -1
        prog_timer[ch] = 0
        prog_loco[ch]  = 0
        prog_name[ch]  = ""
        prog_msg[ch]   = ""
    script_error = 0
    return
    #uu: penser a de-reserver
    
def script_get_label_index(sz):
    for i in range(len(prog_label_name)):
        if(prog_label_name[i] == sz):
            return prog_label_internal_index[i]
    return -1


#script_load("default_script.txt")

def update_script_windows():
    i2 = 0
    for ch in range(nb_prog):
        index = prog_index[ch]
        if(index == -1):
            continue
        cmd = script_db[index]
        line = script_line[index]
        info = "CH" + str(ch) + " " + prog_name[ch] + " ligne:" + str(line) + " instr:" + cmd
        my_canvas_script.itemconfigure(l_prog_info[i2], text = info)
        my_canvas_script.itemconfigure(l_prog_msg[i2] , text = prog_msg[ch])
        i2 += 1
    if(script_error == 0):
        my_canvas_script.configure(background = '#5B9BD5')
    else:       
        my_canvas_script.configure(background = '#ff0000')
        if(i2 < nb_prog):
            my_canvas_script.itemconfigure(l_prog_info[i2], text = error_msg)
            i2 += 1
    while(i2<nb_prog):       
        my_canvas_script.itemconfigure(l_prog_info[i2], text = "")
        my_canvas_script.itemconfigure(l_prog_msg[i2] , text = "")
        i2 += 1
    

def script_tic():
    global prog_timer
    global prog_index
    global horo_h; global horo_m
    global script_error
    global error_msg
    global rs232_loc
    global cac_on; global cac_side
    
    if(script_error):
        return
    
    error_msg = ""    
    for ch in range(nb_prog):
        if(prog_index[ch] == -1):
            continue
        if(prog_timer[ch] > 0):
            prog_timer[ch] -= 1
            if(prog_timer[ch] == 0):
                prog_index[ch] += 1
            else:
                continue
        while(1):
            err = 0
            cmd = script_db[prog_index[ch]]
            line = script_line[prog_index[ch]]
                        
            print("SCRIPT: CH"+str(ch)+" PROG:"+prog_name[ch]+" ligne:"+str(line)+" instr:"+cmd)

            acmd = cmd.split()    #separateur blanc(s) ou tab
            lcmd = len(acmd)
            print("SCRIPT: acmd=" + str(acmd) + " len=" + str(lcmd))
                
            print("SCRIPT: cmd=" + acmd[0])
            
            if(acmd[0][-1] == ":"): #c'est un label
                prog_index[ch] += 1;
                
            elif(acmd[0] == "end"):
                #uu: penser aux liberations
                print("SCRIPT: end")
                prog_index[ch] = -1
                break
                
            elif(acmd[0] == "loco"):
                #uu: penser a reserver et liberer
                loc = int(acmd[1])
                if(loc > rs232_loco_nb):
                    error_msg = "loc > 50"; script_error = 1; break;
                print("SCRIPT: sel loc = " + str(loc))
                prog_loco[ch] = loc
                prog_index[ch] += 1;
                
            elif(acmd[0] == "vit"):
                loc = prog_loco[ch]
                sens = 1                
                if(acmd[1][0] == '-'):
                    sens = 0
                rs232_loc.sens[loc] = sens
                vit = abs(int(acmd[1]))
                if(vit > 28):
                    error_msg = "vit > 28"; script_error = 1; break;
                rs232_loc.vit[loc] = vit
                print("SCRIPT: set loc = " + str(loc) + " sens = " + str(sens) + " vit = " + str(vit))
                prog_index[ch] += 1;
                
            elif(acmd[0] == "fct"):
                loc = prog_loco[ch]
                i = 1
                while(i < lcmd):
                    s = acmd[i]; i += 1
                    if(s[-1] == '+'):
                        v = 1
                    elif(s[-1] == '-'):
                        v = 0
                    else:
                        err = 1
                        break;              
                    if(s[0] != 'f'):
                        err = 1
                        break;                  
                    if(s[1] == '0'):
                        rs232_loc.fct0[loc] = v       
                    elif(s[1] == '1'):
                        rs232_loc.fct1[loc] = v       
                    elif(s[1] == '2'):
                        rs232_loc.fct2[loc] = v       
                    elif(s[1] == '3'):
                        rs232_loc.fct3[loc] = v       
                    elif(s[1] == '4'):
                        rs232_loc.fct4[loc] = v       
                    else:
                        err = 1
                        break;              
                    print("SCRIPT: set loc = " + str(loc) + " fct " + s[1] + " to " + str(v))
                prog_index[ch] += 1;
            
            elif(acmd[0] == "tempo"):
                val = 0
                if(acmd[1][-1] == 't'):
                    val = int(acmd[1][:-1])
                else:
                    val = 4 * int(acmd[1][:-1])                    
                if(lcmd == 3):
                    if(acmd[2][-1] == 't'):
                        val2 = int(acmd[2][:-1])
                    else:
                        val2 = 4 * int(acmd[2][:-1])                    
                    val = random.randint(val, val2)
                print("SCRIPT: set tempo = " + str(val) + " tic")
                if(val == 0):
                    prog_index[ch] += 1
                else:
                    prog_timer[ch] = val
                    break
                    
            elif(acmd[0] == "set"):
                i = 1
                while(i<lcmd):
                    s = acmd[i]; i += 1
                    if(s[-1] == '+' or s[-1] == '/'):
                        tdv_set(s[:-1],1)
                        print("SCRIPT: set " + s[:-1])
                    elif(s[-1] == '-'):
                        tdv_set(s[:-1],0)
                        print("SCRIPT: set " + s[:-1])
                    else:
                        dec = s.split("=")   #pwm38=144 -> pwm38 144 
                        if(len(dec) != 2):
                            err = 1
                            break;                              
                        tdv_set(s, 1)
                        print("SCRIPT: set " + dec[0] + " to " + dec[1])                    
                prog_index[ch] += 1;
                
            elif(acmd[0] == "wait"):
                ok = 1
                i = 1
                while(i < lcmd):
                    s = acmd[i]; i += 1
                    if(s[1] >= "0" and s[1] <= "9"):
                        if(s[0] == '!'):
                            v = 0
                            s = s[1:]
                        else:
                            v = 1
                        val = tdv_get(s)
                        if(val != v):
                            ok = 0
                            break   
                    else:
                        val = tdv_get(s)
                        if(val == 0):
                            ok = 0
                            break   
                if(ok):                 
                    print("SCRIPT: ok")
                    prog_index[ch] += 1;
                else:
                    break               

            elif(acmd[0] == "aig"):
                #check res == 0
                ok = 1
                for i in (1, lcmd):
                    s = acmd[i];
                    num = int(s[1:-1]) #remove a & -/
                    board = num // 1000; num = num % 1000
                    if(var[board].res[num]):
                        ok = 0
                        break
                if(ok == 0):
                    break
                #cmd
                for i in range(1, lcmd):
                    s = acmd[i];
                    if(s[-1] == '-'):
                        tdv_set(s[:-1], 0)
                        print("SCRIPT: aig " + s[:-1] + " to direct")
                    elif(s[-1] == '/'):
                        tdv_set(s[:-1], 1)
                        print("SCRIPT: aig " + s[:-1] + " to devie")
                #check pos
                ok = 1
                for i in range(1, lcmd):
                    s = acmd[i];
                    if(s[-1] == '-'):
                        v = tdv_get("d" + s[1:-1])
                        if(v == 0):
                            ok = 0
                    if(s[-1] == '/'):
                        v = tdv_get("e" + s[1:-1])
                        if(v == 0):
                            ok = 0
                if(ok == 0):
                    break               
                print("SCRIPT: aig done")
                prog_index[ch] += 1;
                
            elif(acmd[0] == "sethm"):
                horo_h = 10 * int(acmd[1][0]) + int(acmd[1][1])   # acmd[1][2] is ':'
                horo_m = 10 * int(acmd[1][3]) + int(acmd[1][4])
                prog_index[ch] += 1;
            
            elif(acmd[0] == "waithm"):
                horo_h2 = 10 * int(acmd[1][0]) + int(acmd[1][1])
                horo_m2 = 10 * int(acmd[1][3]) + int(acmd[1][4])
                print("horo_h=" + str(horo_h) + "horo_m=" + str(horo_m) + "horo_h2=" + str(horo_h2) + "horo_m2=" + str(horo_m2))
                if(horo_h == horo_h2 and horo_m == horo_m2):
                    prog_index[ch] += 1;
                else:
                    break               

            elif(acmd[0] == "goto"):                  
                index = script_get_label_index(acmd[1])
                if(index == -1):
                    error_msg = "bad label name"; script_error = 1; break;
                if(lcmd == 2):                
                    #print("SCRIPT: goto" + acmd[1])
                    prog_index[ch] = index                  
                if(lcmd == 3):
                    percent = int(acmd[1][:-1])      #80%
                    val = random.randint(0,99)
                    if(percent > val):
                        print("SCRIPT: goto" + acmd[1])
                        prog_index[ch] = index                  
                    else:
                        prog_index[ch] += 1

            elif(acmd[0] == "run"):
                ok = 0;
                for freech in range(nb_prog):
                    if(prog_index[freech] == -1):
                        ok = 1
                        break
                if(ok == 0):
                    error_msg = "no more free channel"; script_error = 1; break;
                else:               
                    label_name = acmd[1]
                    index = script_get_label_index(label_name)
                    if(index == -1):
                        error_msg = "bad label name"; script_error = 1; break;
                    prog_index[freech] = index
                    prog_timer[freech] = 0      #0 aucun timer
                    prog_loco[freech]  = 0      #0 aucune loco
                    prog_name[freech]  = label_name
                    prog_index[ch] += 1   #on n'attends pas

            elif(acmd[0] == "wait_fin"):
                label_name = acmd[1]
                ok = 1
                for i in range(nb_prog):
                    if(prog_index[i] != -1):
                        if(prog_name[i] == label_name):
                            ok = 0
                if(ok):
                    prog_index[ch] += 1
                else:
                    break               

            elif(acmd[0] == "msg"):
                prog_msg[ch] = acmd[1]
                prog_index[ch] += 1
                
            elif(acmd[0] == "nomsg"):
                prog_msg[ch] = ""
                prog_index[ch] += 1

            elif(acmd[0] == "sound"):
                try:
                    file = acmd[1][1:-1]
                    winsound.PlaySound(file, winsound.SND_FILENAME + winsound.SND_ASYNC)
                    # + winsound.SND_NOSTOP
                    # ne que sous windows  --> uu: trouver un moyen sous Linux
                    # arretes les sons precedents quand on en lance un nouveau son
                    # - son: subprocess.call("tungtungtung.wav", shell = True)    //ouvre le lecteur mais perte de focus
                    # - son: os.system("start tungtungtung.wav")                //ouvre le lecteur mais perte de focus
                    # - son: winsound.PlaySound('tungtungtung.wav', winsound.SND_FILENAME + winsound.ASYNC) # + winsound.SND_NOSTOP   //que sous windows arretes les sons precedents quand on en lance un
                    # - son: winsound.Beep(400,1000)
                except:
                    pass
                prog_index[ch] += 1

            #    cac_init 12 3
            #    cac_start 12 +
            #    cac_wait 12 8
            #    cac_stop 12
            elif(acmd[0] == "cac_init"):
                l = int(acmd[1])
                c = int(acmd[2])
                prog_index[ch] += 1
                if(l == 0):
                    cantons_free(c)
                else:
                    cantons_assign_train(c, l)

            elif(acmd[0] == "cac_start"):
                l = int(acmd[1])
                prog_index[ch] += 1
                cac_side[l] = acmd[2]
                cac_on[l] = 1
                
            elif(acmd[0] == "cac_stop"):
                l = int(acmd[1])
                prog_index[ch] += 1
                cac_on[l] = 0
                rs232_loc.vit[l] = 0
                
            elif(acmd[0] == "cac_wait"):
                l = int(acmd[1])
                c = int(acmd[2])
                if(cantons_is_train(c,l)):
                    prog_index[ch] += 1
                else:   
                    break
                
            else:
                error_msg = "UNKNOWN INSTRUCTION"; script_error = 1; break;
                
        if(script_error):
            break
            
    if(script_error):
        print("SCRIPT: ERROR -> SCRIPT ABORTED")
        print("SCRIPT error msg = " + error_msg)
        error_msg = "SCRIPT ERROR: CH" + str(ch) + " -> " + error_msg
            
    return

    

#---------------------------------------------------------------------------------------
# TCO
#---------------------------------------------------------------------------------------

my_canvas_tco = [ 0 for i in range(10)]

for i in range(10):
    my_canvas_tco[i] = tkinter.Canvas(top, bg = "#5B9BD5", width = 200, height = 100) #petite taille pr ne pas prendre trop de memoire si pas utilise
screen_tco_on = 0
tco_current = 0


def tco_on():
    row = 0
    my_canvas_tco[tco_current].grid(row = row, column = 0)
    
    row = row + 1
    return

def tco_off():
    my_canvas_tco[tco_current].grid_forget()
    return


shape_list = [ 0 for i in range(10) ]
for i in range(10):
    shape_list[i] = []
    
class shape_entry():
    color = 0; old_color = 0; use_color = 0
    size  = 0; old_size  = 0; use_size  = 0
    widget = 0
    
def add_shape(tco, widget, color_, size_):
    global tco_shape    
    sh = shape_entry()
    sh.widget = widget
    sh.old_color = sh.color = color_
    sh.old_size = sh.size = size_
    shape_list[tco].append(sh)

#-----    
action_list = [ 0 for i in range(10) ]
for i in range(10):
    action_list[i] = []

class action_entry():   # uu: mettre des self
    x1 = 0
    y1 = 0
    x2 = 0
    y2 = 0
    var_name = ""
    cmd_name = ""
    
def add_action(tco, x1_, y1_, x2_, y2_, var_name, cmd_name):
    global action_list
    ##print("adding tco="+str(tco)+" x1="+str(x1_)+" y1="+str(y1_)+" x2="+str(x2_)+" y2="+str(y2_)+" var_name="+var_name+" cmd_name="+cmd_name)
    a = action_entry()
    a.x1 = x1_; a.y1 = y1_    
    a.x2 = x2_; a.y2 = y2_    
    a.var_name = var_name
    a.cmd_name = cmd_name
    action_list[tco].append(a)

#-----    
class color_entry():
    var_name = ""
    state = ""
    color = "#000000"
    color2 = "#000000"
    shape_ind = 0
    
color_list = [ 0 for i in range(10) ]
for i in range(10):
    color_list[i] = []
    
def add_color(tco, shape_ind_, var_name_, state_, color_):
    global color_list
    ##print("color tco="+str(tco)+" var_name="+var_name_+" state="+state_+" color="+color_)
    a = color_entry()
    a.var_name = var_name_
    a.state = state_
    a.color = color_
    a.shape_ind = shape_ind_
    color_list[tco].append(a)    

color2_list = [ 0 for i in range(10) ]
for i in range(10):
    color2_list[i] = []    

def add_color2(tco, shape_ind_ , var_name_, state_, color_, color2_):
    global color_list
    ##print("color tco="+str(tco)+" var_name="+var_name_+" state="+state_+" color="+color_)
    a = color_entry()
    a.var_name = var_name_
    a.state = state_
    a.color = color_
    a.color2 = color2_
    a.shape_ind = shape_ind_
    color2_list[tco].append(a)    
    
#-----    
class size_entry():
    var_name = ""
    state = ""
    size = 1
    size2 = 1
    widget = 0
    
size_list = [ 0 for i in range(10) ]
for i in range(10):
    size_list[i] = []
        
def add_size(tco, shape_ind_, var_name_, state_, size_):
    global color_list
    ##print("size tco="+str(tco)+" var_name="+var_name_+" state="+state_+" size="+str(size_))
    a = size_entry()
    a.var_name = var_name_
    a.state = state_
    a.size = size_
    a.shape_ind = shape_ind_
    size_list[tco].append(a)  

size2_list = [ 0 for i in range(10) ]
for i in range(10):
    size2_list[i] = []
        
def add_size2(tco, shape_ind_, var_name_, state_, size_, size2_):
    global color_list
    ##print("size tco="+str(tco)+" var_name="+var_name_+" state="+state_+" size="+str(size_))
    a = size_entry()
    a.var_name = var_name_
    a.state = state_
    a.size = size_
    a.size2 = size2_
    a.shape_ind = shape_ind_
    size2_list[tco].append(a)  

#-----    
class canton_entry():
    canton_num = 0
    widget = 0
    
canton_list = [ 0 for i in range(10) ]
for i in range(10):
    canton_list[i] = []

def add_canton(tco, widget_, canton):
    global canton_list
    a = canton_entry()
    a.widget = widget_
    a.canton_num = canton    
    canton_list[tco].append(a)
    
#-----
   
# update shape color and size for the specified tco  
def update_shape_color_size_for_tco(tco):
    global shape_list

    nb = len(color2_list[tco]) #color2 before color to allow color to overwrite color2
    for i in range(0, nb):
        var     = color2_list[tco][i].var_name
        state   = color2_list[tco][i].state
        sh_ind  = color2_list[tco][i].shape_ind
        zecolor = color2_list[tco][i].color
        zecolor2= color2_list[tco][i].color2
        val = tdv_get(var)
        if((val and state == "") or (val == 0 and state == "!")):
            shape_list[tco][sh_ind].color = zecolor
        else:
            shape_list[tco][sh_ind].color = zecolor2
            
    nb = len(color_list[tco])
    for i in range(0, nb):
        var     = color_list[tco][i].var_name
        state   = color_list[tco][i].state
        sh_ind  = color_list[tco][i].shape_ind
        zecolor = color_list[tco][i].color
        val = tdv_get(var)
        if((val and state == "") or (val == 0 and state == "!")):
            shape_list[tco][sh_ind].color = zecolor
            
    nb = len(size2_list[tco]) #size2 before color to allow size to overwrite color2
    for i in range(0, nb):
        var     = size2_list[tco][i].var_name
        state   = size2_list[tco][i].state
        sh_ind  = size2_list[tco][i].shape_ind
        zesize  = size2_list[tco][i].size
        zesize2 = size2_list[tco][i].size2
        val = tdv_get(var)
        if((val and state == "") or (val == 0 and state == "!")):
            shape_list[tco][sh_ind].size = zesize
        else:
            shape_list[tco][sh_ind].size = zesize2
            
    nb = len(size_list[tco])
    for i in range(0, nb):
        var     = size_list[tco][i].var_name
        state   = size_list[tco][i].state
        sh_ind  = size_list[tco][i].shape_ind
        zesize  = size_list[tco][i].size
        val = tdv_get(var)
        if((val and state == "") or (val == 0 and state == "!")):
            shape_list[tco][sh_ind].size = zesize
   
#update les couleurs et largeurs        
def tco_update(tco):
    global shape_list
    ##print("TCO: update " + str(tco))
    
    update_shape_color_size_for_tco(tco)
    nb = len(shape_list[tco])
    for i in range(0, nb):
        c = shape_list[tco][i].color; oldc = shape_list[tco][i].old_color
        s = shape_list[tco][i].size ; olds = shape_list[tco][i].old_size
        widget_ = shape_list[tco][i].widget
        if(c != oldc):
            shape_list[tco][i].old_color = c
            my_canvas_tco[tco].itemconfigure(widget_, fill = c)
        if(s != olds):
            shape_list[tco][i].old_size = s
            if(s == 0):
                my_canvas_tco[tco].itemconfigure(widget_, state = 'hidden')
            else:
                my_canvas_tco[tco].itemconfigure(widget_, state = 'normal')        
            my_canvas_tco[tco].itemconfigure(widget_, width = s)

    nb = len(canton_list[tco])
    for i in range(0, nb):
        c  = canton_list[tco][i].canton_num
        t = cantons_make_txt(c)
        widget_ = canton_list[tco][i].widget
        my_canvas_tco[tco].itemconfigure(widget_, text = t)

#-----

def tco_create_net_msg(tco):

    update_shape_color_size_for_tco(tco)
    
    msg = "color_size "
    nb = len(shape_list[tco])
    for i in range(0, nb):
        if(shape_list[tco][i].use_color):
            zecolor = shape_list[tco][i].color
            msg += zecolor
            msg += " "
        if(shape_list[tco][i].use_size):
            zesize  = shape_list[tco][i].size
            msg += str(zesize)
            msg += " "

    msg += "canton "
    nb = len(canton_list[tco])
    for i in range(0, nb):
        c  = canton_list[tco][i].canton_num
        t = cantons_make_txt(c)
        msg += t
        msg += " "
        
    if(rs232_loc.au_state):
        msg += "au 1 "
    else:
        msg += "au 0 "
        
    msg += "horo ["
    if(horo_h < 10):
        msg+= "0"
    msg += str(horo_h)
    msg += ":"
    if(horo_m < 10):
        msg+= "0"
    msg += str(horo_m)
    msg += "] "     

    return msg

#-----  
        
tco_file_config = ""  #copie du fichier tco pour le transmettre aux tco distants           
gtco_x1 = 0    
gtco_y1 = 0    
gtco_x2 = 0    
gtco_y2 = 0 
#
# instructions des TCO
#   
# tco     tco_num
# #       commentaire (# seulement sur la premiere colonne)
# grid    dx,dy,gdx,gdy color_grid color_fond
# txt     x,y "txt" color font_size
# txtc    x,y "txt" color font_size
# txt2    "txt" color font_size
# txtc2   "txt" color font_size
# line    x1,y1,x2,y2 color size
# rect    x1,y1,x2,y2 color_border color size
# rectc   xc,yc,dx,dy color_border color size
# ovalc   xc,yc,dx,dy color_border color size
# color   [!]var color              s'applique au dernier line,oval,rect
# color2  [!]var color color0
# size    [!]var size               " (size 0 = invisible) 
# size2   [!]var size size0         " (size 0 = invisible) 
# action  x1,y1,x2,y2 var[+,-,^]    +:mise a 1 / -: mise a 0 / ^:changement d'etat
# actionc xc,yc,dx,dy var[+,-,^]    !:changement color/size si la variable est a 0
# action2 var[+,-,^]    +:mise a 1 / -: mise a 0 / ^:changement d'etat
# cantontxtc2 canton color font_size
#
def tco_load(filename):
    global shape_list
    global my_canvas_tco
    global gtco_x1; global gtco_y1; global gtco_x2; global gtco_y2 
    global tco_file_config    

    tc = 0 #tco current
       
    with open(filename, mode = "r", encoding = "utf-8", errors = 'ignore') as my_file:   #ignore remove non utf-8 chars
        print("TCO: loading file " + filename)
        num_line = 0
        
        #try:
        if(1):
            for line in my_file:
                tco_file_config += line
                num_line += 1
                line = line.rstrip("\n")
                if(line == ""):
                    continue
                if(line[0] == "#"):
                    continue
                #line = line.split("#")[0]       #remove comment
                line = line.rstrip()    
                ##print("line= " + str(num_line) + " > " + line)
                acmd = line.split()    #separateur blanc(s) ou tab
                lcmd = len(acmd)
            
                if(acmd[0] == "tco"):
                    tc = int(acmd[1])
                    continue

                if(acmd[0] == "grid"):
                    axy = acmd[1].split(",")
                    dx = int(axy[0]); dy = int(axy[1]);
                    gdx = int(axy[2]); gdy = int(axy[3]);
                    color_line = acmd[2]; color_fond = acmd[3]
                    my_canvas_tco[tc].config(bg = color_fond)
                    my_canvas_tco[tc].config(width = dx)
                    my_canvas_tco[tc].config(heigh = dy)
                    if(gdy):
                        nb = dy // gdy
                        for j in range (0, nb):
                            my_canvas_tco[tc].create_line(0, gdy * j, dx, gdy * j, fill = color_line, width = 1)
                    if(gdx):
                        nb = dx // gdx
                        for i in range (0,nb):
                            my_canvas_tco[tc].create_line(gdx * i, 0, gdx * i, dy, fill = color_line, width = 1)
                    continue
                    
                if(acmd[0] == "line"):
                    axy = acmd[1].split(",")
                    x1 = int(axy[0]); y1 = int(axy[1])
                    x2 = int(axy[2]); y2 = int(axy[3])
                    color = acmd[2]
                    size = acmd[3]
                    the_line = my_canvas_tco[tc].create_line(x1, y1, x2, y2, fill = color, width = size)
                    add_shape(tc, the_line, color, size)
                    continue
                    
                if(acmd[0] == "rect"):
                    axy = acmd[1].split(",")
                    x1 = int(axy[0]); y1 = int(axy[1])
                    x2 = int(axy[2]); y2 = int(axy[3])
                    gtco_x1 = x1; gtco_y1 = y1; gtco_x2 = x2; gtco_y2 = y2
                    color_border = acmd[2]; color = acmd[3]
                    size = acmd[4]
                    the_rect = my_canvas_tco[tc].create_rectangle(x1, y1, x2, y2, fill = color, outline = color_border, width = size)
                    add_shape(tc, the_rect, color, size)
                    continue
                    
                if(acmd[0] == "rectc"):
                    axy = acmd[1].split(",")
                    xc = int(axy[0]); yc = int(axy[1])
                    dx = int(axy[2]); dy = int(axy[3])
                    x1 = xc-dx; y1 = yc-dy
                    x2 = xc+dx; y2 = yc+dy              
                    gtco_x1 = x1; gtco_y1 = y1; gtco_x2 = x2; gtco_y2 = y2
                    color_border = acmd[2]; color = acmd[3]
                    size = acmd[4]
                    the_rect = my_canvas_tco[tc].create_rectangle(x1, y1, x2, y2, fill = color, outline = color_border, width = size)
                    add_shape(tc, the_rect, color, size)
                    continue

                if(acmd[0] == "ovalc"):
                    axy = acmd[1].split(",")
                    xc = int(axy[0]); yc = int(axy[1])
                    dx = int(axy[2]); dy = int(axy[3])
                    x1 = xc-dx; y1 = yc-dy
                    x2 = xc+dx; y2 = yc+dy              
                    gtco_x1 = x1; gtco_y1 = y1; gtco_x2 = x2; gtco_y2 = y2
                    color_border = acmd[2]; color = acmd[3]
                    size = acmd[4]
                    the_oval = my_canvas_tco[tc].create_oval(x1, y1, x2, y2, fill = color, outline = color_border, width = size)
                    add_shape(tc, the_oval, color, size)
                    continue

                if(acmd[0] == "txt"):
                    axy = acmd[1].split(",")
                    x = int(axy[0]); y = int(axy[1])
                    txt = acmd[2]
                    txt = txt.replace("\"", "")  #pour l'instant pas de blanc ds txt
                    color = acmd[3]
                    size = acmd[4]
                    zefont = ("Times New Roman", size, "bold")  #Helvetica #Courier
                    my_canvas_tco[tc].create_text(x, y, text = txt, font = zefont, fill = color, anchor = tkinter.W)
                    continue

                if(acmd[0] == "txt2"):
                    if(gtco_x2 > gtco_x1):
                        x = gtco_x2 + 2
                    else:
                        x = gtco_x1 + 2
                    y = (gtco_y1 + gtco_y2) // 2
                    txt = acmd[1]
                    txt = txt.replace("\"","")  #pour l'instant pas de blanc ds txt
                    color = acmd[2]
                    size = acmd[3]
                    zefont = ("Times New Roman", size, "bold")  #Helvetica #Courier
                    my_canvas_tco[tc].create_text(x, y, text = txt, font = zefont, fill = color, anchor = tkinter.W)
                    continue
                    
                if(acmd[0] == "txtc"):
                    axy = acmd[1].split(",")
                    x = int(axy[0]); y = int(axy[1])
                    txt = acmd[2]
                    txt = txt.replace("\"", "")  #pour l'instant pas de blanc ds txt
                    color = acmd[3]
                    size = acmd[4]
                    zefont = ("Times New Roman", size, "bold")
                    my_canvas_tco[tc].create_text(x, y, text = txt, font = zefont, fill = color)
                    continue 

                if(acmd[0] == "txtc2"):
                    x = (gtco_x1 + gtco_x2) // 2
                    y = (gtco_y1 + gtco_y2) // 2
                    txt = acmd[1]
                    txt = txt.replace("\"", "")  #pour l'instant pas de blanc ds txt
                    color = acmd[2]
                    size = acmd[3]
                    zefont = ("Times New Roman", size, "bold")
                    my_canvas_tco[tc].create_text(x, y, text = txt, font = zefont, fill = color)
                    continue 

                if(acmd[0] == "cantontxtc2"):
                    x = (gtco_x1 + gtco_x2) // 2
                    y = (gtco_y1 + gtco_y2) // 2
                    c = int(acmd[1])
                    txt = "c" + str(c) + ":!!!" 
                    color = acmd[2]
                    size = acmd[3]
                    zefont = ("Times New Roman", size, "bold")
                    the_txt = my_canvas_tco[tc].create_text(x, y, text = txt, font = zefont, fill = color)
                    add_canton(tc, the_txt, c)
                    continue 
                    
                # action xc,yc,dx,dy var[+,-,^]             
                if(acmd[0] == "action" or acmd[0] == "actionc"):
                    axy = acmd[1].split(",")
                    if(acmd[0] == "action"):
                        x1 = int(axy[0]); y1 = int(axy[1])
                        x2 = int(axy[2]); y2 = int(axy[3])
                    if(acmd[0] == "actionc"):
                        xc = int(axy[0]); yc = int(axy[1])
                        dx = int(axy[2]); dy = int(axy[3])
                        x1 = xc - dx; y1 = yc - dy
                        x2 = xc + dx; y2 = yc + dy
                    if(x1 > x2):
                        x  = x2; x1 = x2; x2 = x;
                    if(y1 > y2):
                        y = y2; y1 = y2; y2 = y;
                    var_name = acmd[2][0:-1]
                    var_action = acmd[2][-1]
                    add_action(tc, x1, y1, x2, y2, var_name, var_action) 
                    continue 

                if(acmd[0] == "action2" or acmd[0] == "actionc2"):
                    x1 = gtco_x1; y1 = gtco_y1
                    x2 = gtco_x2; y2 = gtco_y2
                    if(x1 > x2):
                        x = x2; x1 = x2; x2 = x;
                    if(y1 > y2):
                        y = y2; y1 = y2; y2 = y;
                    var_name = acmd[1][0:-1]
                    var_action = acmd[1][-1]
                    add_action(tc, x1, y1, x2, y2, var_name, var_action) 
                    continue

                # color [!]var color #s'applique au dernier line,oval,rect
                if(acmd[0] == "color"):
                    v = acmd[1]
                    if(v[0] == '!'):
                        var_state = "!"
                        var_name  = v[1:]
                    else:
                        var_state = ""
                        var_name  = v
                    add_color(tc, len(shape_list[tc])-1, var_name, var_state, acmd[2])
                    shape_list[tc][-1].use_color = 1
                    continue

                if(acmd[0] == "color2"):
                    v = acmd[1]
                    if(v[0] == '!'):
                        var_state = "!"
                        var_name  = v[1:]
                    else:
                        var_state = ""
                        var_name  = v
                    add_color2(tc, len(shape_list[tc])-1, var_name, var_state, acmd[2], acmd[3])
                    shape_list[tc][-1].use_color = 1
                    continue
                    
                # size [!]var size  #s'applique au dernier line,oval,rect  (size 0 = invisible) 
                if(acmd[0] == "size"):
                    v = acmd[1]
                    if(v[0] == '!'):
                        var_state = "!"
                        var_name  = v[1:]
                    else:
                        var_state = ""
                        var_name  = v
                    add_size(tc, len(shape_list[tc])-1, var_name, var_state, int(acmd[2]))
                    shape_list[tc][-1].use_size = 1
                    continue

                if(acmd[0] == "size2"):
                    v = acmd[1]
                    if(v[0] == '!'):
                        var_state = "!"
                        var_name  = v[1:]
                    else:
                        var_state = ""
                        var_name  = v
                    add_size2(tc, len(shape_list[tc])-1, var_name, var_state, int(acmd[2]), int(acmd[3]))
                    shape_list[tc][-1].use_size = 1
                    continue
                    
                if(acmd[0] == "end"):
                    break
                    
                print("TCO: UNKNOWN instruction: " + acmd[0])           
                
        #except:
        #    print("TCO: ERROR line= " + str(num_line) + " > " + line)               

    print("TCO: file loaded")
                
        
tco_load("config_tco.txt")


def callback_tco_mouse(tco, mx, my):
    
    l = len(action_list[tco])
    
    for i in range(0,l):
        if(mx < action_list[tco][i].x1):
            continue
        if(mx > action_list[tco][i].x2):
            continue
        if(my < action_list[tco][i].y1):
            continue
        if(my > action_list[tco][i].y2):
            continue
        order = action_list[tco][i].cmd_name
        szVar = action_list[tco][i].var_name
        ##print("match cmd="+order+" var="+szVar)
        if(order == "^"):
            val = tdv_get(szVar)
            tdv_set(szVar, val ^ 1)
        if(order == "+"):
            tdv_set(szVar, 1)
        if(order == "-"):
            tdv_set(szVar, 0)
        
tco_mx = 0
tco_my = 0

def callback_canvas_tco_mouse(event):
    ##print("canvas tco clicked at:" + str(event.x) + "," + str (event.y))
    
    global tco_mx
    global tco_my
    
    tco_mx = event.x
    tco_my = event.y
    
    callback_tco_mouse(tco_current, tco_mx, tco_my)
                

for i in range(0,10):
    my_canvas_tco[i].bind("<Button-1>", callback_canvas_tco_mouse)
        


#---------------------------------------------------------------------------------------
# Reseau (pour les tco)
#---------------------------------------------------------------------------------------

def net_tco_loop(c):
    global neti
    
    neti += 1
    netnum = neti
    print("NET: TCO CONNEXION " + str(netnum))
    
    try:
    #if(1):
        while(1):
            data = c.recv(1024).decode()   #need to use intermediate variable in python3 because there is no more buffer in decode
            ##print("RX" + str(netnum)+">"+data+"<")
            if(data == ""):                #renvoie une chaine vide lorsque la connexion est coupee sinon lecture blocante
                break;
                
            if(data[0:len("init tco")] == "init tco"):     # <--init tco2   (le numero du tco n'est pas utilise)
                tc = int(data[-1])
                msg = str(len(tco_file_config))
                ##print("TX" + str(netnum)+">"+msg+"<")
                c.send(msg.encode())                        # --> size
                
                data = c.recv(1024).decode()                # <--ok
                ##print("RX" + str(netnum)+">"+data+"<")
                
                msg = tco_file_config                       
                c.send(msg.encode())                        # --> tco_file_config
                ##print("TX" + str(netnum)+ " tco_file_config")
                
                data = c.recv(1024).decode()                # <--ok
                ##print("RX" + str(netnum)+">"+data+"<")
                
            elif(data[0:len("update tco")] == "update tco"):# <--update tco2
                tc = int(data[-1])
                msgdata = tco_create_net_msg(tc)
                msg = str(len(msgdata))
                ##print("TX" + str(netnum)+">"+msg+"<")
                c.send(msg.encode())                        # --> size
                
                data = c.recv(1024).decode()                # <--ok
                ##print("RX" + str(netnum)+">"+data+"<")
                
                ##print("update: " + msgdata)
                c.send(msgdata.encode())                    # --> update data
                ##print("TX" + str(netnum)+ " update data")
                
                data = c.recv(1024).decode()                # <--ok
                ##print("RX" + str(netnum)+">"+data+"<")
                
            elif(data[0:len("click tco")] == "click tco"):  # <--click tco2 25,25
                l = len("click tco")
                tc = int(data[l])
                sxy = data[l+2:]
                xy = sxy.split(",")
                x = int(xy[0])
                y = int(xy[1])
                ##print("TCO=" + str(tc) + " X="+str(x)+" Y="+str(y))
                callback_tco_mouse(tc, x, y)
                
            else:
                break        
    except:
        pass
    c.close()    
    print("NET: TCO DECONNEXION " + str(netnum))

    
#---------------------------------------------------------------------------------------
# Reseau (pour les cabines)
#---------------------------------------------------------------------------------------

s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
host = socket.gethostname()
ipadr = socket.gethostbyname(host)
port = fdcc_server_port
net_ok = 1
try:
    s.bind((host, port))
    s.listen(10)
    print("NET: host=\"" + host + "\" IP=" + ipadr + " port=" + str(port))
except:
    net_ok = 0
    tkinter.messagebox.showerror(SZ_TITLE, "Erreur: Le port reseau " + str(port) + " est deja utilise\n\n(Une erreur classique est qu'un autre fdcc_pc tourne deja !)\n\nFonctionement de fdcc_pc sans le reseau")             
    print("NET: ERREUR Le port reseau est deja utilise")

neti = 0
def net_cabine_loop(c):
    global rs232_loc
    global neti
    
    neti += 1
    netnum = neti
    print("NET: CABINE CONNEXION " + str(netnum))
    adr = 0
    
    try:
        while(1):
            data = c.recv(1024).decode()   #need to use intermediate variable in python3 because there is no more buffer in decode
            #print("("+str(netnum)+">"+data+"<")
            if(data == ""):                #renvoie une chaine vide lorsque la connexion est coupee
                break;
                
            acmd = data.split()
            for i in range(0,len(acmd)):
                cmd = acmd[i]
                if(cmd == "au1"):
                    rs232_loc.au_set   = 1
                elif(cmd == "au0"):
                    rs232_loc.au_clr   = 1
                elif(cmd[0] == "a"):
                    adr = int(cmd[1:])
                elif(cmd == "s+"):
                    rs232_loc.sens[adr] = 1
                elif(cmd == "s-"):
                    rs232_loc.sens[adr] = 0
                elif(cmd[0] == "v"):
                    vit = int(cmd[1:])
                    rs232_loc.vit[adr] = vit
                elif(cmd == "f0+"):
                    rs232_loc.fct0[adr] = 1
                elif(cmd == "f0-"):
                    rs232_loc.fct0[adr] = 0
                elif(cmd == "f1+"):
                    rs232_loc.fct1[adr] = 1
                elif(cmd == "f1-"):
                    rs232_loc.fct1[adr] = 0
                elif(cmd == "f2+"):
                    rs232_loc.fct2[adr] = 1
                elif(cmd == "f2-"):
                    rs232_loc.fct2[adr] = 0
                elif(cmd == "f3+"):
                    rs232_loc.fct3[adr] = 1
                elif(cmd == "f3-"):
                    rs232_loc.fct3[adr] = 0
                elif(cmd == "f4+"):
                    rs232_loc.fct4[adr] = 1
                elif(cmd == "f4-"):
                    rs232_loc.fct4[adr] = 0
                else:
                    print("NET: CABINE ERROR: Unknown command: " + cmd)
                
    except:
        pass
    c.close()    
    print("NET: CABINE DECONNEXION " + str(netnum))


def net_connect_loop(i):
    while(1):
        print("NET: Attente")
        connexion, adr = s.accept()
        print("NET: Demande de connexion de, IP %s, port %s." %(adr[0], adr[1]))
        msg = "fdcc2017-pc-srv netv2\n"
        connexion.send(msg.encode())
        data = connexion.recv(1024).decode()
        print("NET: Client string: \"" + data + "\"")
        # creation d'un nouveau thread pour gerer chaque connexion :
        if(data[0:len("cabine netv1")] == "cabine netv1"):
            th = threading.Thread(target = net_cabine_loop, args = (connexion,))
            th.start()      #it = th.getName() # identifiant du thread
        elif(data[0:len("tco netv2")] == "tco netv2"):
            th = threading.Thread(target = net_tco_loop, args=(connexion,))
            th.start()      #it = th.getName() # identifiant du thread
        else:
            print("NET: client not supported: " + data)

if(net_ok):         
    t = threading.Thread(target = net_connect_loop, args = (i,))
    t.start()


        
#---------------------------------------------------------------------------------------
# Accueil
#---------------------------------------------------------------------------------------

zefont = ("Times New Roman", 36, "bold")

l_accueil = tkinter.Label(top, text = "FREE DCC 2016")
my_canvas_accueil = tkinter.Canvas(top, bg = "#5B9BD5", height = 600, width = 800)

my_canvas_accueil.create_text(400, 300,text = SZ_TITLE,font = zefont,fill = "#FFFFFF")

screen_accueil_on = 0

def accueil_on():
    my_canvas_accueil.grid(row = 0, column = 0)
    return

def accueil_off():
    l_accueil.grid_forget()
    my_canvas_accueil.grid_forget()
    return


#---------------------------------------------------------------------------------------
# Menu
#---------------------------------------------------------------------------------------

def screen_off():
    global screen_loco_on
    global screen_accueil_on
    global screen_cvprog_on
    global screen_esin_on
    global screen_esan_on
    global screen_esout_on
    global screen_esled_on
    global screen_esfx_on
    global screen_esaig_on
    global screen_espwm_on
    global screen_essrv_on
    global screen_esneo_on
    
    global screen_script_on
    global screen_tco_on
    
    if(screen_loco_on):
        loco_off()
        screen_loco_on = 0
    if(screen_accueil_on):
        accueil_off()
        screen_accueil_on = 0
    if(screen_cvprog_on):
        cvprog_off()
        screen_cvprog_on = 0
    if(screen_tco_on):
        tco_off()
        screen_tco_on = 0
    if(screen_esin_on):
        esin_off()
        screen_esin_on = 0
    if(screen_esan_on):
        esan_off()
        screen_esan_on = 0
    if(screen_esout_on):
        esout_off()
        screen_esout_on = 0
    if(screen_esled_on):
        esled_off()
        screen_esled_on = 0
    if(screen_esfx_on):
        esfx_off()
        screen_esfx_on = 0
    if(screen_esaig_on):
        esaig_off()
        screen_esaig_on = 0
    if(screen_espwm_on):
        espwm_off()
        screen_espwm_on = 0
    if(screen_essrv_on):
        essrv_off()
        screen_essrv_on = 0
    if(screen_esneo_on):
        esneo_off()
        screen_esneo_on = 0             
    if(screen_script_on):
        script_off()
        screen_script_on = 0
    return
    
def m_accueil():
    global screen_accueil_on
    if(screen_accueil_on == 0):
        screen_off()
        screen_accueil_on = 1
        accueil_on()
    return

def m_loco():
    global screen_loco_on
    if(screen_loco_on == 0):
        screen_off()
        screen_loco_on = 1
        loco_on()
    return

def m_cvprog():
    global screen_cvprog_on
    if(screen_cvprog_on == 0):
        screen_off()
        screen_cvprog_on = 1
        cvprog_on()
    return

def m_au():
    global rs232_loc
    rs232_loc.au_set = 1

def m_au_fin():
    global rs232_loc
    rs232_loc.au_clr = 1

def m_tco(t):
    global screen_tco_on
    global tco_current
    screen_off()
    tco_current = t     
    tco_on()
    screen_tco_on = 1
    return

def m_tcocurrent():
    m_tco(tco_current)
    return
def m_tco0():
    m_tco(0)
    return
def m_tco1():
    m_tco(1)
    return
def m_tco2():
    m_tco(2)
    return
def m_tco3():
    m_tco(3)
    return
def m_tco4():
    m_tco(4)
    return
def m_tco5():
    m_tco(5)
    return
def m_tco6():
    m_tco(6)
    return
def m_tco7():
    m_tco(7)
    return
def m_tco8():
    m_tco(8)
    return
def m_tco9():
    m_tco(9)
    return 
    
def m_script():
    global screen_script_on
    if(screen_script_on == 0):
        screen_off()
        script_on()
        screen_script_on = 1
    return    

def m_es_board():  
    global esb
    global debug_force_redraw
    r = askinteger("FDCC", "Numero de la centrale ? (0-" + str(MAX_BOARD_NB - 1) + ") > ", initialvalue = esb, minvalue = 0, maxvalue = MAX_BOARD_NB - 1)  
    if(r == None):
        return      
    try:
        r = int(r)
    except:
        return
    esb = r
    debug_force_redraw = 1
    return
def m_es_in():
    global screen_esin_on
    if(screen_esin_on == 0):
        screen_off()
        esin_on()
        screen_esin_on = 1
    return
def m_es_an():
    global screen_esan_on
    if(screen_esan_on == 0):
        screen_off()
        esan_on()
        screen_esan_on = 1
    return
def m_es_out():    
    global screen_esout_on
    if(screen_esout_on == 0):
        screen_off()
        esout_on()
        screen_esout_on = 1
    return
def m_es_led():    
    global screen_esled_on
    if(screen_esled_on == 0):
        screen_off()
        esled_on()
        screen_esled_on = 1
    return
def m_es_fx():  
    global screen_esfx_on
    if(screen_esfx_on == 0):
        screen_off()
        esfx_on()
        screen_esfx_on = 1
    return
def m_es_aig():    
    global screen_esaig_on
    if(screen_esaig_on == 0):
        screen_off()
        esaig_on()
        screen_esaig_on = 1
    return
def m_es_pwm():    
    global screen_espwm_on
    if(screen_espwm_on == 0):
        screen_off()
        espwm_on()
        screen_espwm_on = 1
    return
def m_es_srv():    
    global screen_essrv_on
    if(screen_essrv_on == 0):
        screen_off()
        essrv_on()
        screen_essrv_on = 1
    return
def m_es_neo():    
    global screen_esneo_on
    if(screen_esneo_on == 0):
        screen_off()
        esneo_on()
        screen_esneo_on = 1
    return
    
def m_horo_on():
    global horo_on
    horo_on = 1    
def m_horo_off():
    global horo_on
    horo_on = 0    
def m_horo_reset():
    global horo_j; global horo_h; global horo_m; global horo_t
    horo_j = horo_j_dep
    horo_h = horo_h_dep
    horo_m = horo_m_dep
    horo_t = 0    
def m_horo_changer():  
    global horo_h
    global horo_m
    while(1):
        s = askstring("FDCC", "Nouvelle heure ?\n\nformat: HH:MM (ex: 06:25)", initialvalue = "06:00")  
        if(s == None):
            return      
        try:
            h = int(s[0:2])
            m = int(s[3:5])
            horo_h = h
            horo_m = m
            return
        except:
            continue
            
def m_cantons():
    while(1):
        q  = "Changement d'affectation des cantons\n\n"
        q += "format: c<num_canton(0-99)>:<liberation(0)|adresse_de_la_loco(1-50)>\n\n"
        q += "- ex: c2:39 affectation de la loco 39 sur le canton 2\n"
        q += "- ex: c6:0  liberation du canton 6\n" 
        s = askstring("FDCC", q, initialvalue = "c0:0") 
        if(s == None):
            return      
        try:
            s2 = s.split(":")
            print("s2="+str(s2))
            c = int(s2[0][1:])
            print("c="+str(c))
            if(c < 0 or c > 99):
                continue
            l = int(s2[1])
            print("l="+str(l))
            if(l < 0 or l > 50):
                continue
            if(l == 0):
                print("CANTONS: Liberation canton" + str(c))
                cantons_free(c)
            else:
                print("CANTONS: Assignation loco" + str(l) + " sur le canton" + str(c))
                cantons_assign_train(c, l)
            return
        except:
            continue
    return          
    
def m_cantons_report():
    t = cantons_make_report()
    tkinter.messagebox.showinfo("FDCC", "Etat des cantons (les cantons libres ne sont pas affiches):\n\n" + t)             
    
def m_apropos():
    tkinter.messagebox.showinfo(SZ_TITLE, SZ_TITLE  + "\n\nUlysse Delmas-Begue\n\nhttp://udelmas.e-monsite.com/\n\nversion beta test")

def m_manuel_centrale():
    os.system("fdcc2017.docx")
    #subprocess.Popen(["fdcc2017.docx"])    

def m_manuel_tco():
    os.system("fdcc2017.docx")

def m_web():
    os.system("http://udelmas.e-monsite.com/pages/free-dcc-2008-2010.html")

    
screen_accueil_on = 1
accueil_on()

# create a menu
menu = tkinter.Menu(top)
top.config(menu = menu)

generalmenu = tkinter.Menu(menu)
menu.add_cascade(label = "General", menu = generalmenu)
generalmenu.add_command(label = "Accueil", command = m_accueil)
generalmenu.add_command(label = "Sortir", command = exit)

locomenu = tkinter.Menu(menu)
menu.add_cascade(label = "Locos", menu = locomenu)
locomenu.add_command(label = "Selection Locos", command = m_loco)
locomenu.add_command(label = "Programation CV", command = m_cvprog)
locomenu.add_separator()
locomenu.add_command(label = "Arret d'urgence", command = m_au)
locomenu.add_command(label = "Fin Arret d'urgence", command = m_au_fin)
locomenu.add_separator()
locomenu.add_command(label = "Affecter un canton" , command = m_cantons)
locomenu.add_command(label = "Etat des cantons"   , command = m_cantons_report)

tcomenu = tkinter.Menu(menu)
menu.add_cascade(label = "TCO", menu = tcomenu)
tcomenu.add_command(label = "TCO actuel", command = m_tcocurrent)
tcomenu.add_separator()
tcomenu.add_command(label = "TCO 0", command = m_tco0)
tcomenu.add_command(label = "TCO 1", command = m_tco1)
tcomenu.add_command(label = "TCO 2", command = m_tco2)
tcomenu.add_command(label = "TCO 3", command = m_tco3)
tcomenu.add_command(label = "TCO 4", command = m_tco4)
tcomenu.add_command(label = "TCO 5", command = m_tco5)
tcomenu.add_command(label = "TCO 6", command = m_tco6)
tcomenu.add_command(label = "TCO 7", command = m_tco7)
tcomenu.add_command(label = "TCO 8", command = m_tco8)
tcomenu.add_command(label = "TCO 9", command = m_tco9)

scriptmenu = tkinter.Menu(menu)
menu.add_cascade(label = "Scripts", menu = scriptmenu)
scriptmenu.add_command(label = "Scripts", command = m_script)

horlogemenu = tkinter.Menu(menu)
menu.add_cascade(label = "Horloge", menu = horlogemenu)
horlogemenu.add_command(label = "on", command = m_horo_on)
horlogemenu.add_command(label = "off", command = m_horo_off)
horlogemenu.add_command(label = "reset", command = m_horo_reset)
horlogemenu.add_command(label = "changer", command = m_horo_changer)

esmenu = tkinter.Menu(menu)
menu.add_cascade(label = "E/S", menu = esmenu)
esmenu.add_command(label = "Carte", command = m_es_board)
esmenu.add_separator()
esmenu.add_command(label = "In"   , command = m_es_in)
esmenu.add_command(label = "Out"  , command = m_es_out)
esmenu.add_command(label = "Led"  , command = m_es_led)
esmenu.add_command(label = "Aig"  , command = m_es_aig)
esmenu.add_command(label = "Pwm"  , command = m_es_pwm)
esmenu.add_command(label = "Srv"  , command = m_es_srv)
esmenu.add_command(label = "Neo"  , command = m_es_neo)
esmenu.add_command(label = "Fx"   , command = m_es_fx)
esmenu.add_command(label = "An"   , command = m_es_an)

helpmenu = tkinter.Menu(menu)
menu.add_cascade(label = "Aide", menu = helpmenu)
helpmenu.add_command(label = "A propos", command = m_apropos)
#helpmenu.add_command(label = "Manuel Centrale(local)", command = m_manuel_centrale)    ##uu: bloque le programme
#helpmenu.add_command(label = "Manuel TCO (local)", command = m_manuel_tco)
#helpmenu.add_command(label = "Site internet", command = m_web)

toutmenu = tkinter.Menu(menu)
menu.add_cascade(label = "Tout", menu = toutmenu)     #"Carte"
toutmenu.add_command(label = "Accueil", command = m_accueil)
toutmenu.add_command(label = "Sortir", command = exit)
toutmenu.add_separator()
toutmenu.add_command(label = "Selection Locos"    , command = m_loco)
toutmenu.add_command(label = "Programation CV"    , command = m_cvprog)
toutmenu.add_separator()
toutmenu.add_command(label = "Arret d'urgence"    , command = m_au)
toutmenu.add_command(label = "Fin Arret d'urgence", command = m_au_fin)
toutmenu.add_separator()
toutmenu.add_command(label = "Affecter un canton" , command = m_cantons)
toutmenu.add_command(label = "Etat des cantons"   , command = m_cantons_report)
toutmenu.add_separator()
toutmenu.add_command(label = "TCO actuel", command = m_tcocurrent)
toutmenu.add_command(label = "TCO 0", command = m_tco0)
toutmenu.add_command(label = "TCO 1", command = m_tco1)
toutmenu.add_command(label = "TCO 2", command = m_tco2)
toutmenu.add_command(label = "TCO 3", command = m_tco3)
toutmenu.add_command(label = "TCO 4", command = m_tco4)
toutmenu.add_command(label = "TCO 5", command = m_tco5)
toutmenu.add_command(label = "TCO 6", command = m_tco6)
toutmenu.add_command(label = "TCO 7", command = m_tco7)
toutmenu.add_command(label = "TCO 8", command = m_tco8)
toutmenu.add_command(label = "TCO 9", command = m_tco9)
toutmenu.add_separator()
toutmenu.add_command(label = "Scripts", command = m_script)
toutmenu.add_separator()
toutmenu.add_command(label = "on" , command = m_horo_on)
toutmenu.add_command(label = "off", command = m_horo_off)
toutmenu.add_command(label = "reset", command = m_horo_reset)
toutmenu.add_command(label = "changer", command = m_horo_changer)
toutmenu.add_separator()
toutmenu.add_command(label = "Carte", command = m_es_board)
toutmenu.add_separator()
toutmenu.add_command(label = "In" , command = m_es_in)
toutmenu.add_command(label = "Out", command = m_es_out)
toutmenu.add_command(label = "Led", command = m_es_led)
toutmenu.add_command(label = "Aig", command = m_es_aig)
toutmenu.add_command(label = "Pwm", command = m_es_pwm)
toutmenu.add_command(label = "Srv", command = m_es_srv)
toutmenu.add_command(label = "Neo", command = m_es_neo)
toutmenu.add_command(label = "Fx" , command = m_es_fx)
toutmenu.add_command(label = "An" , command = m_es_an)
toutmenu.add_separator()
toutmenu.add_command(label = "A propos", command = m_apropos)

top.bind("<Key>", callback_key)    



#---------------------------------------------------------------------------------------

tic_cli = 0
old_au_state = -1

def title_tic():
    global old_au_state
    
    sz = SZ_TITLE
    if(rs232_loc.au_state):
        sz += "  ARRET D'URGENCE"
        
    screen_es = screen_esin_on + screen_esan_on + screen_esout_on + screen_esled_on + screen_esfx_on + screen_esaig_on + screen_espwm_on + screen_essrv_on + screen_esneo_on
    if(screen_es):
        sz += "  Carte=" + str(esb)
        
    sz+= "         ["
    if(horo_h < 10):
        sz+= "0"
    sz += str(horo_h)
    sz += ":"
    if(horo_m < 10):
        sz+= "0"
    sz += str(horo_m)
    sz += "]"
    
    if(screen_tco_on):
        sz += "         (x"
        sz += str(tco_mx)
        sz += ",y"
        sz += str(tco_my)
        sz += ")"
        
    try:        
        top.title(sz)  # PB chez certains utilisateurs
    except:
        try:
            top.title(SZ_TITLE)
        except:
            pass
    
    if(old_au_state != rs232_loc.au_state):
        old_au_state = rs232_loc.au_state
        if(rs232_loc.au_state):
            my_canvas_loco.configure(background = 'red')
        else:
            my_canvas_loco.configure(background = '#5B9BD5')
    
    return


def tic_cb():
    global tic_cli
    tic_cli ^= 1

    fdcc_and_debug_to_rs232()      # choisit out/led/aigcmd fdcc ou debug    
    if(rs232_ok):                  #
        rs232_update()             # mise a jour de la rs232
    else:                          #
        no_rs232_update()          # simulation rs232 (surtout pour le retour aiguillages)    
    rs232_to_fdcc_and_debug()      # met a jour in de debug (faire tj meme si fenetre fermee)

    cantons_maj()                  # mise a jour du suivi des trains
    
    tdv_tic()                      # execute les equations
    
    sig_tic()                      # execute la signalisation
    
    script_tic()                   # execute les scripts

    iti_tic()                      # met a jour les itineraires
    
    cac_maj()                      # execute la conduite automatique en mode cantonnement
    
    if(rs232_loc.au_state == 0):
        horo_tic()                 # met a jour l'horloge de modelisme
    
    title_tic()                    # met a jour le titre
    
    if(screen_tco_on):             # met a jour les voyants
         tco_update(tco_current)

    if(screen_esin_on):            # met la fenetre des e/s
        debug_in_redraw_all()
    if(screen_esan_on):
        debug_an_redraw_all()
    if(screen_esout_on):
        debug_out_redraw_all()
    if(screen_esled_on):
        debug_led_redraw_all()
    if(screen_esfx_on):
        debug_fx_redraw_all()
    if(screen_esaig_on):
        debug_aig_redraw_all()
    if(screen_esneo_on):
        debug_neo_redraw_all()
    if(screen_espwm_on):
        debug_pwm_redraw_all()
    if(screen_essrv_on):
        debug_srv_redraw_all()
     
    if(screen_loco_on):            # met a jour la fenetre des locos et les regus ouverts
        loco_redraw_au()
        loco_update() #uu: essayer de faire moins lourd

    independant_regu_redraw_au()
    independant_regu_redraw_all()
    
    if(screen_script_on):          # met a jour la fenetre des scripts
        update_script_windows()
    
    return
    
        
        
ze_cpt = 0
real_time_err_cpt = 0
isr_on = 1

def tic_isr_fct():      #uu: refaire a la mode lwa
  global ze_cpt
  global real_time_err_cpt
  global isr_on
  
  if(isr_on == 0):
      return
  
  t1 = time.time()
  tic_cb()
  t2 = time.time()
  
  dt = int(1000 * (t2 - t1))
  dt2 = 250 - dt
  if(dt2 < 0):
      dt2 = 0
  if(dt2 > 250):
      dt2 = 250
  if(dt > 250):
      real_time_err_cpt += 1

  if(normal_dbg):      
      print("cpt=" + str(ze_cpt) + " dt=" + str(dt) + " freedt=" + str(dt2) + " rterr=" + str(real_time_err_cpt) + " "  + rs232_stats)
  threading.Timer(float(dt2 / 1000), tic_isr_fct).start()
  ze_cpt += 1
  return

tic_isr_fct()

top.mainloop()

print("BYE")
isr_on = 0

time.sleep(1)
#quit()

#---------------------------------------------------------------------------------------







